<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.codevision.io
 * @since      1.0.0
 *
 * @package    Photobooth_Gallery
 * @subpackage Photobooth_Gallery/admin
 */

use Photobooth\CustomPostTypes;
use Photobooth\Defuse\Crypto\Key;
use Photobooth\Elementor\Importer;
use Photobooth\FileManagement;
use Photobooth\Helper;
use Photobooth\Plugins\LockMixin;
use Photobooth\Plugins\UploaderPro;
use Photobooth\Renderer;
use Photobooth\SocialMedia;
use Photobooth\Statistics\SocialMediaStatistics;
use Photobooth\Statistics\XMLStatistics;
use Photobooth\Transients;

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Photobooth_Gallery
 * @subpackage Photobooth_Gallery/admin
 * @author     codevision.io <info@codevision.io>
 */
class Photobooth_Gallery_Admin
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    private $settings;

    /**
     * @var Renderer
     */
    private $render;

    /**
     * @var SocialMedia
     */
    private $social_manager;

    /**
     * @var FileManagement
     */
    private $file_management;

    /**
     * @var XMLStatistics
     */
    private $xml_stats;

    /**
     * @var SocialMediaStatistics
     */
    private $social_stats;

    /**
     * @var bool
     */
    private $updated;

    /**
     * Initialize the class and set its properties.
     *
     * @param string $plugin_name The name of this plugin.
     * @param string $version The version of this plugin.
     *
     * @since    1.0.0
     *
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        $this->render = new Renderer();
        $this->social_manager = new SocialMedia();
        $this->file_management = new FileManagement();

        $this->xml_stats = new XMLStatistics();
        $this->social_stats = new SocialMediaStatistics();

        $this->updated = $this->is_updateable();
    }

    private function is_updateable()
    {

        if (!is_admin()) {
            return false;
        }

        $installed_version = get_option(PG_NAMESPACE.'installed-version', false);

        if ($installed_version === PHOTOBOOTH_GALLERY_VERSION) {
            return false;
        }

        update_option(PG_NAMESPACE.'installed-version', PHOTOBOOTH_GALLERY_VERSION);

        $this->show_quick_install(true);

        return true;
    }

    private function show_quick_install($yes)
    {

        $key = PG_NAMESPACE.'-show-quick-install';

        if ($yes) {
            update_option($key, true);
        } else {
            delete_option($key);
        }
    }

    public static function pluginInit()
    {

        if (wp_doing_ajax()) {
            return;
        }

        SocialMedia::init();
        FileManagement::init();
        XMLStatistics::init();
    }

    function init()
    {

        $this->settings = Helper::get_plugin_settings();
        $post_types = new CustomPostTypes();
        $post_types->init();

        $this->add_custom_image_sizes();
        $this->verify_config_settings();
        $this->verify_encryption();

        $this->add_rewrite_rules();

        Helper::handle_cron($this->settings['is_enabled_cron'], 'pg-cron-scan', $this->settings['scan_interval']);

        // Set the Clean up interval if specified to be executed at 2o'Clock in the morning
        Helper::handle_cron(
            $this->settings['auto_clean_interval'] !== 'disabled',
            'pg-cron-cleanup',
            'daily',
            strtotime('+1day 02:00:00')
        );

        // Set the Mail Cron to deliver emails to us hourly
        Helper::handle_cron($this->settings['error_reporting'], 'pg-cron-error-reporting', 'hourly');

        $this->xml_stats->wp_init();
        $this->social_stats->wp_init();

        $this->handle_plugin_update();

        $this->handle_admin_requests();

        $this->handle_quick_install_view();

        do_action('photobooth/init/admin');
    }

    private function add_custom_image_sizes()
    {

        // Double size for retina displays
        add_image_size('pg-thumbnail', 300, 300);

        // Preview image maximum 1024 pixel
        add_image_size('pg-preview', 1024);
    }

    /**
     * STARTING PRIVATE METHODS
     */

    private function verify_config_settings()
    {

        $settings = Helper::load_all_settings();

        if (!$settings) {

            Helper::log(sprintf('[ Config Verification ] No settings found, skipping...'));

            return;
        }

        foreach ($settings as $name => $setting) {
            $this->verify_setting($name, $setting);
        }

    }

    private function verify_setting($name, $setting)
    {

        if (!array_key_exists('is_create_media_folder', $setting)) {
            return false;
        }

        // Create media folder if allowed and set
        if ($setting['is_create_media_folder'] && strlen($setting['media_folder'])) {
            $folder = Helper::ensureTrailingBackslash(
                $setting['media_folder'],
                PG_BACKSLASH_PREFIX | PG_BACKSLASH_SUFFIX
            );
            $folder = WP_CONTENT_DIR.$folder;

            if (!is_dir($folder)) {

                Helper::log(
                    sprintf(
                        '[ Config Verification ] Photobooth Folder "%s" in config <%s> not found, creating folder now...',
                        $folder,
                        $name
                    )
                );

                if (!@mkdir($folder, 0755)) {
                    Helper::log(
                        sprintf(
                            '[ Config Verification ] Photobooth Folder "%s" in config <%s> could not be created! Please check Permissions!',
                            $folder,
                            $name
                        ),
                        'error'
                    );

                    return false;
                }
            }

            // Make sure the silence is golden index.php is present in this folder to disallow folder view
            if (!file_exists($folder.'index.php')) {

                if (!copy(__DIR__.'/../index.php', $folder.'index.php')) {
                    Helper::log(
                        sprintf('[ Config Verification ] Could not copy index.php to folder "%s"!', $folder),
                        'error'
                    );
                }

            }

            // Copy the demo file into the newly created folder
            if (!file_exists($folder.'admin-ABC12345-ABCDE123.jpg')) {

                if (!copy(
                    PG_IMAGE_PATH_ADMIN.'admin-ABC12345-ABCDE123.jpg',
                    $folder.'admin-ABC12345-ABCDE123.jpg'
                )) {
                    Helper::log(
                        sprintf(
                            '[ Config Verification ] Could not copy admin-ABC12345-ABCDE123.jpg to folder "%s"!',
                            $folder
                        ),
                        'error'
                    );
                }

            }
        }

        return true;
    }

    private function verify_encryption()
    {

        if (get_option('pg-secret-encryption-key')) {
            return;
        }

        $key = Key::createNewRandomKey();

        update_option('pg-secret-encryption-key', $key->saveToAsciiSafeString());
    }

    private function add_rewrite_rules()
    {

        $this->add_rewrite_endpoint('pg-code');
        $this->add_rewrite_endpoint('pg-social-share');
        $this->add_rewrite_endpoint('pg-upload');

        add_rewrite_endpoint('pg-code', EP_PERMALINK | EP_PAGES | EP_ROOT);
        add_rewrite_endpoint('pg-payload', EP_PERMALINK | EP_PAGES | EP_ROOT);
        add_rewrite_endpoint('pg-webhook', EP_ROOT);
        add_rewrite_endpoint('pg-download', EP_ROOT);
        add_rewrite_endpoint('pg-social-share', EP_PERMALINK | EP_PAGES | EP_ROOT);
        add_rewrite_endpoint('pg-async-scan-request', EP_PERMALINK | EP_PAGES | EP_ROOT);

        add_rewrite_endpoint('pg-upload', EP_PERMALINK | EP_PAGES | EP_ROOT);
    }

    /**
     * Add rewrite endpoint specific to page set as homepage
     *
     * @param $endpoint
     */
    private function add_rewrite_endpoint($endpoint)
    {

        $endpoint = sanitize_text_field($endpoint);

        add_rewrite_tag("%{$endpoint}%", '([^&]+)');

        $page_on_front = get_option('page_on_front');

        add_rewrite_rule(
            '^'.$endpoint.'/([^/]*)/?',
            'index.php?page_id='.$page_on_front.'&'.$endpoint.'=$matches[1]',
            'top'
        );
        add_rewrite_rule(
            '^(.*)/'.$endpoint.'2/([^/]*)/?',
            'index.php?pagename=$matches[1]&static=true&'.$endpoint.'=$matches[2]',
            'top'
        );
    }

    private function handle_plugin_update()
    {

        if (!$this->updated) {
            return;
        }

        $this->activate();
    }

    private function activate()
    {

        $this->add_default_settings();

        $this->add_elementor_templates();

        flush_rewrite_rules(true);
    }

    private function add_default_settings()
    {

        $page = get_page_by_title('Default Settings', OBJECT, CustomPostTypes::SETTINGS_SLUG);

        if ($page) {
            return;
        }

        $post_id = wp_insert_post([
            'post_type' => CustomPostTypes::SETTINGS_SLUG,
            'post_title' => 'Default Settings',
            'post_status' => 'publish',
        ]);

        if (!$post_id) {
            Helper::log('[ Plugin Initialization ] could not create Default Settings!', 'error');

            return;
        }

        $share_id = $this->add_share_page();

        $default_settings = Photobooth_Gallery::get_config_defaults();

        if ($share_id !== false) {
            $default_settings['social_media_page'] = $share_id;
        }

        foreach ($default_settings as $key => $value) {
            update_field($key, $value, $post_id);
        }

        foreach (Photobooth_Gallery::get_defaults() as $key => $value) {
            update_field($key, $value, 'options');
        }

        // try to add the test page
        if ($this->add_test_page()) {

            // Test page added successfully, now set rewrite settings
            $this->initialize_rewrite_settings();

            // Initialize the Sync process once
            $this->file_management->scan_upload_folder();
        }


    }

    private function add_share_page()
    {

        $page = get_page_by_title(__('PBG-Share', 'photobooth-gallery'));

        if ($page) {
            return $page->ID;
        }

        $post_id = wp_insert_post([
            'post_type' => 'page',
            'post_title' => __('PBG-Share', 'photobooth-gallery'),
            'post_status' => 'publish',
            'post_content' => '[view-photobooth-gallery settings="Default Settings"]',
        ]);

        if (!$post_id) {
            Helper::log('[ Plugin Initialization ] could not create PBG-Share Page', 'error');

            return false;
        }

        return $post_id;
    }

    private function add_test_page()
    {

        $page = get_page_by_title(__('PBG', 'photobooth-gallery'));

        if ($page) {
            return $page->ID;
        }

        $post_id = wp_insert_post([
            'post_type' => 'page',
            'post_title' => __('PBG', 'photobooth-gallery'),
            'post_status' => 'publish',
            'post_content' => '[photobooth-gallery settings="Default Settings"]',
        ]);

        if (!$post_id) {
            Helper::log('[ Plugin Initialization ] could not create Test Page PBG', 'error');

            return false;
        }

        return $post_id;
    }

    private function initialize_rewrite_settings()
    {

        $set_rewrite = get_option('pg-rewrite-init');

        if ($set_rewrite) {
            return;
        }

        global $wp_rewrite;

        $wp_rewrite->set_permalink_structure('/%postname%/');

        update_option("pg-rewrite-init", true);

        $wp_rewrite->flush_rules(true);
    }

    private function add_elementor_templates()
    {

        if (!defined('PG_ELEMENTOR_DIR')) {
            return;
        }

        $importer = new Importer(PG_ELEMENTOR_DIR.'/templates/', 'photobooth-gallery');
        $importer->scan_folder();
    }

    private function handle_admin_requests()
    {

        if (!is_admin()) {
            return;
        }

        if (!current_user_can('manage_options')) {
            return;
        }

        if (array_key_exists('pg-cron-cleanup', $_GET)) {
            $this->cron_cleanup();
        }

        if (array_key_exists('pg-filter-media-on', $_GET)) {
            $this->filter_media(true);
        }

        if (array_key_exists('pg-filter-media-off', $_GET)) {
            $this->filter_media(false);
        }

        if (array_key_exists('pg-show-quick-install', $_GET)) {
            $this->show_quick_install(true);
        }
    }

    function cron_cleanup()
    {

        try {

            $skip_codes = $this->settings['skip_auto_clean'];
            $skip_codes = explode(',', $skip_codes);

            $response = $this->file_management->cleanup_images($this->settings['auto_clean_interval'], $skip_codes);

            if (!$response) {

                Helper::log(sprintf('[ Image Cleanup Cron ] Auto Cleanup is disabled, skipping ...'));

                return;
            }

            $result = [];

            foreach ($response as $config => $file_operations) {

                if (!$file_operations) {
                    $result[] = sprintf(
                        __(
                            'Removing Media Files from config "%s" was not successful!',
                            'photobooth-gallery'
                        ),
                        $config
                    );
                    continue;
                }

                $resp = array_values($file_operations);
                array_unshift($resp, $config);

                $result[] = vsprintf(
                    __(
                        '%s: Found: %d, Moved: %d, Errors: %d, Skipped: %d, Removed from File-Management: %d',
                        'photobooth-gallery'
                    ),
                    $resp
                );
            }

            Helper::log(sprintf('[ Image Cleanup Cron ] Result: %s ...', json_encode($result)));

            $zips_removed = $this->file_management->cleanup_zipfiles($this->settings['auto_clean_interval']);

            Helper::log(sprintf('[ Zip Cleanup Cron ] Result: %d expired zip archives removed ...', $zips_removed));

            $meta_removed = $this->file_management->clean_imported_attachments();

            Helper::log(
                sprintf('[ Orphaned Files Cron ] Result: %d orphaned image meta data removed ...', $meta_removed)
            );

        } catch (Exception $e) {
            Helper::log($e, 'error');
        }

    }

    private function filter_media($yes)
    {

        $key = 'pg-filter-media-files';

        if ($yes) {
            update_option($key, 'all');
        } else {
            delete_option($key);
        }
    }

    private function handle_quick_install_view()
    {

        if (!get_option(PG_NAMESPACE.'-show-quick-install')) {
            return;
        }

        add_action('admin_notices', [$this, 'render_one_click_install']);
    }

    function init_acf()
    {

        $img_path = 'data:image/svg+xml;base64,'.base64_encode(
                file_get_contents(PG_BASE_DIR.'/admin/img/admin-photobooth-gallery.svg')
            );

        acf_add_options_page(array(
            'page_title' => __('Photobooth Gallery', 'photobooth-gallery'),
            'menu_title' => __('Photobooth Gallery', 'photobooth-gallery'),
            'menu_slug' => 'photobooth-gallery',
            'icon_url' => $img_path,
            'capability' => 'manage_options',
            'redirect' => true,
        ));

        acf_add_options_sub_page(array(
            'page_title' => __('Global Settings', 'photobooth-gallery'),
            'menu_title' => __('Global Settings', 'photobooth-gallery'),
            'menu_slug' => 'acf-options-global-settings',
            'parent_slug' => 'photobooth-gallery',
            'icon_url' => $img_path,
        ));

        if (defined('PG_ELEMENTOR_DIR')) {

            acf_add_options_sub_page(array(
                'page_title' => __('Statistics', 'photobooth-gallery'),
                'menu_title' => __('Statistics', 'photobooth-gallery'),
                'menu_slug' => 'acf-options-pg-statistics',
                'parent_slug' => 'photobooth-gallery',
                'icon_url' => $img_path,
            ));

        }

        acf_add_options_sub_page(array(
            'page_title' => __('Plugin Integration', 'photobooth-gallery'),
            'menu_title' => __('Plugin Integration', 'photobooth-gallery'),
            'menu_slug' => 'acf-options-plugin-extensions',
            'parent_slug' => 'photobooth-gallery',
            'icon_url' => $img_path,
        ));

    }

    function render_cpu_field($field)
    {

        $field['max'] = (Helper::num_cpus() - 2) * 2;

        return $field;
    }

    function load_default_share($field)
    {

        if ($field['value']) {
            return $field;
        }

        $field['value'] = $this->add_share_page();

        return $field;
    }

    function render_default_settings($field)
    {

        if (strlen($field['value'])) {
            return $field;
        }

        $defaults = Photobooth_Gallery::get_config_defaults();

        if (!array_key_exists($field['_name'], $defaults)) {
            return $field;
        }

        $field['value'] = $defaults[$field['_name']];

        if (array_key_exists('placeholder', $field)) {
            $field['placeholder'] = $field['value'];
        }

        return $field;
    }

    function render_elementor_version($field)
    {

        if (get_option('pbg-elementor-active') === 'active' && defined('ELEMENTOR_VERSION')) {
            $field['message'] = sprintf(
                __(
                    'The shipped Elementor free plugin is currently <strong>not active</strong>, because you have enabled Elementor (Version: %s) via plugins directly.',
                    'photobooth-gallery'
                ),
                ELEMENTOR_VERSION
            );

            return $field;
        }

        if (!defined('PG_ELEMENTOR_DIR')) {
            $field['message'] = __(
                'The shipped Elementor free plugin is currently not <strong>active</strong> on this wordpress installation.',
                'photobooth-gallery'
            );

            return $field;
        }

        $field['message'] = sprintf(
            __(
                'The shipped Elementor free plugin is currently <strong>active</strong> (Version: %s)',
                'photobooth-gallery'
            ),
            ELEMENTOR_VERSION
        );

        return $field;

    }

    /**
     * Add Menu to Page
     */
    function add_admin_page()
    {

        add_submenu_page(
            'acf-options-global-settings',
            __('Log File', 'photobooth-gallery'),
            __('Log File', 'photobooth-gallery'),
            'manage_options',
            'photobooth-gallery-logs',
            array(
                $this,
                'load_log_content',
            )
        );


        add_submenu_page(
            'acf-options-global-settings',
            __('Documentation', 'photobooth-gallery'),
            __('Documentation', 'photobooth-gallery'),
            'manage_options',
            'photobooth-gallery-documentation',
            array(
                $this,
                'load_iframe_docu',
            )
        );

        add_submenu_page(
            'acf-options-global-settings',
            __('Addons', 'photobooth-gallery'),
            __('Addons', 'photobooth-gallery'),
            'manage_options',
            'photobooth-gallery-addons',
            array(
                $this,
                'load_iframe_upsell',
            )
        );

        global $submenu;

        $submenu['acf-options-global-settings'][] = [
            __('Quick Install Guide', 'photobooth-gallery'),
            'manage_options',
            admin_url('?pg-show-quick-install'),
        ];
    }

    function media_delete_attachments($post_id)
    {

        if ($this->social_manager->share_count($post_id) === 0) {
            return;
        }

        wp_die(__('This Attachment is currently shared, you cannot delete it!', 'photobooth-gallery'));
    }

    function settings_row_actions($actions, $post)
    {

        if ($post->post_type !== CustomPostTypes::SETTINGS_SLUG) {
            return $actions;
        }

        if (!array_key_exists('inline hide-if-no-js', $actions)) {
            return $actions;
        }

        unset($actions['inline hide-if-no-js']);

        return $actions;
    }

    function settings_columns($cols)
    {

        Helper::array_insert($cols, 'date', ['pb_regex' => __('Regex Code', 'photobooth-gallery')]);
        Helper::array_insert($cols, 'date', ['pb_folder' => __('Upload Folder', 'photobooth-gallery')]);
        Helper::array_insert($cols, 'date', ['pb_stats' => __('Transfer Stats', 'photobooth-gallery')]);

        $cols['pb_actions photobooth-gallery-bootstrap'] = __('Actions', 'photobooth-gallery');

        unset($cols['categories']);

        return $cols;
    }

    function settings_columns_values($column_name, $id)
    {

        $meta = false;

        if ($column_name === 'pb_regex') {
            $meta = get_field('regex', $id);
        }

        if ($column_name === 'pb_folder') {
            $meta = get_field('media_folder', $id);
        }

        if ($column_name === 'pb_stats') {
            $file_stats = -1;

            if (apply_filters('photobooth/settings/column/pb_stats', true) === true) {
                $file_stats = $this->file_management->transferred_files_lifetime($id);
            }

            $last_scan_date = get_transient(sprintf('pg-last-scan-for-%d', $id));

            $stats = [
                sprintf(__('Total Transfers: %d Media Files', 'photobooth-gallery'), $file_stats),
            ];

            if ($last_scan_date) {
                $stats[] = sprintf(
                    __('Last known Scan: %s', 'photobooth-gallery'),
                    wp_date_localised(__('Y-m-d H:i', 'photobooth-gallery'), $last_scan_date)
                );
            }

            $meta = implode('<br/>', $stats);
        }

        if ($column_name === 'pb_actions photobooth-gallery-bootstrap') {

            // This may take long if you have thousands of images, we will store the result in a transient
            $size = get_transient($column_name.'-'.$id);

            if (!$size) {

                $size = -1;

                if (apply_filters('photobooth/settings/column/pb_actions', true) === true) {
                    $size = $this->file_management->removeable_images($id);
                }

                set_transient($column_name.'-'.$id, $size, 15 * MINUTE_IN_SECONDS);
            }

            if (is_array($size)) {
                $size = $size['moved'];
            }

            $meta = $this->render->view('pg_settings-actions', [
                'shortcode' => sprintf('[photobooth-gallery settings="%s"]', get_the_title($id)),
                'id' => $id,
                'size' => $size,
                'is_uploader' => get_field('is_webhook_upload', $id),
            ]);
        }

        if (!$meta) {

            echo '<span class="empty">&minus;</span>';

            return;
        }

        echo $meta;
    }

    function media_columns($cols)
    {

        $cols['pb_filename'] = __('Filename', 'photobooth-gallery');
        $cols['pb_filedate'] = __('File Creation Date', 'photobooth-gallery');
        $cols['pb_social_media_shares'] = __('Social Media Views', 'photobooth-gallery');
        $cols['pb_is_deleteable'] = __('Is deleteable', 'photobooth-gallery');
        $cols['pb_is_autoremoveable'] = __('Autoremoval Date', 'photobooth-gallery');

        return $cols;
    }

    function media_row_actions($actions)
    {

        global $post;

        if ($this->social_manager->share_count($post) === 0) {
            return $actions;
        }

        return [];
    }

    function media_columns_values($column_name, $id)
    {

        $meta = 'no-matching-column';

        $data = get_post_meta($id, 'type', true);

        if ($data !== 'photobooth-gallery-image') {

            return;
        }

        if ($column_name === 'pb_filename') {
            $meta = get_post_meta($id, 'real_filename', true);
        }

        if ($column_name === 'pb_filedate') {
            $meta = get_post_meta($id, 'file_date', true);

            if ($meta) {

                date_default_timezone_set('Europe/Berlin');

                $meta = date('d.m.Y H:i:s', $meta);
            }
        }

        if ($column_name === 'pb_social_media_shares') {

            global $post;

            $meta = $this->social_manager->share_count($post);
        }

        if ($column_name === 'pb_is_deleteable') {

            global $post;

            $meta = $this->social_manager->is_deleteable($post);

            if (is_bool($meta)) {
                $meta = __($meta ? 'Yes' : 'No', 'photobooth-gallery');
            }
        }

        if ($column_name === 'pb_is_autoremoveable') {

            global $post;

            $meta = $this->file_management->is_autoremovable_file($post, $this->settings['auto_clean_interval']);

            if ($meta) {
                $meta = __('Yes', 'photobooth-gallery');
            } elseif ($this->settings['auto_clean_interval'] === 'disabled') {
                $meta = __('Disabled', 'photobooth-gallery');
            } else {

                $date = strtotime(
                    get_the_date('Y-m-d H:i:s', $post->post_id).'+'.str_replace(
                        '_',
                        '',
                        $this->settings['auto_clean_interval']
                    )
                );

                $meta = wp_date_localised(__('Y-m-d H:i:s', 'photobooth-gallery'), $date);
            }
        }

        if ($meta === 'no-matching-column') {
            return;
        }


        if (!$meta) {

            echo '<span class="empty">&minus;</span>';

            return;
        }

        echo $meta;
    }

    function media_columns_sortable($cols)
    {

        $cols['pb_filename'] = 'meta_real_filename';
        $cols['pb_filedate'] = 'meta_date';

        return $cols;
    }

    /**
     * @param WP_Query $query
     */
    function media_columns_ordering($query)
    {

        if (!$query->is_main_query()) {
            return;
        }

        if (!is_admin()) {
            return;
        }

        $orderby = $query->get('orderby');


        switch ($orderby) {

            case 'meta_real_filename':

                $query->set('meta_key', 'real_filename');
                $query->set('orderby', 'meta_value');

                break;
            case 'meta_date':
                $query->set('meta_key', 'file_date');
                $query->set('orderby', 'meta_value_num');

                break;
        }

        $screen = get_current_screen();

        if ($screen === null) {
            return;
        }

        if ($screen->id !== 'upload') {
            return;
        }

        $filter_media = get_option('pg-filter-media-files');

        if ($filter_media === 'all') {
            $query->set('meta_key', 'type');
            $query->set('meta_value', 'photobooth-gallery-image');
            $query->set('meta_compare', 'NOT EXISTS');
        }

        $search_term = $query->query_vars['s'];

        $query->query_vars['s'] = '';

        if ($search_term) {
            $meta_query = array('relation' => 'OR');

            array_push($meta_query, array(
                'key' => 'real_filename',
                'value' => $search_term,
                'compare' => 'LIKE',
            ));

            $query->set('meta_query', $meta_query);

            add_filter('get_meta_sql', function ($sql) use ($search_term) {

                global $wpdb;

                // Only run once:
                static $nr = 0;
                if (0 != $nr++) {
                    return $sql;
                }

                // Modify WHERE part:
                $sql['where'] = sprintf(
                    " AND ( %s OR %s ) ",
                    $wpdb->prepare("{$wpdb->posts}.post_title like '%%%s%%'", $search_term),
                    mb_substr($sql['where'], 5, mb_strlen($sql['where']))
                );

                return $sql;
            });
        }

    }

    function save_elementor_status($value)
    {

        if ($value === '1') {
            Helper::set_flag(Pbg_Elementor::ELEMENTOR);
        } else {
            Helper::remove_flag(Pbg_Elementor::ELEMENTOR);
        }

        return $value;
    }

    function save_statistic_password_field($value, $post_id, $field)
    {

        if (!$value) {
            return $value;
        }

        $value = preg_replace("/[^A-Za-z0-9]/", '', $value);
        $value = str_replace(' ', '', $value);

        $value = strtolower($value);

        $value = sanitize_title($value);

        return $value;
    }

    function save_regex_field($value, $post_id, $field)
    {

        $value = trim($value);

        return $value;
    }

    function add_bootstrap_class($field_group)
    {

        $field_group['style'] .= ' photobooth-gallery-bootstrap';

        return $field_group;
    }

    function render_email_sender_field($data)
    {

        if (strpos($data['value'], '@') !== false) {
            return $data;
        }

        $data['value'] = 'info@'.parse_url(site_url(), PHP_URL_HOST);

        return $data;
    }

    function render_webhook_password_field($data)
    {

        if (!strlen($data['value'])) {
            $data['value'] = uniqid();
        }

        $data['value'] = sanitize_title($data['value']);

        return $data;
    }

    function render_delete_button($data)
    {

        if ($data['key'] !== 'field_5b64b9f534646') {
            return;
        }

        $size = $this->file_management->removeable_images(false, false);

        if (is_array($size)) {
            $size = $size['moved'];
        }

        printf('<span class="delete-button-data" data-size="%d"></span>', $size);
    }

    function render_folder_field($field)
    {

        $base_instruction = __(
            'Please enter the media folder here. The folder will be interpreted as relative to the WP_CONTENT_DIR folder. Hint: <strong>Do not use the wordpress upload folder as source folder</strong>. This would create an endless loop when trying to import media files!',
            'photobooth-gallery'
        );

        $meta = [];

        $meta[] = sprintf(
            __(
                'Final Path to photobooth-media folder: %s',
                'photobooth-gallery'
            ).'<span class="wp-content-dir"></span>',
            WP_CONTENT_DIR
        );

        $meta = '<ul class="plugin-info">'.implode('', Helper::make_list($meta)).'</ul>';

        $field['instructions'] = $base_instruction.$meta;

        return $field;
    }

    function render_logfile_info($field)
    {

        if (!get_field('pg_logfile_prefix', 'option')) {
            return $field;
        }

        $file = $this->logfile('log/*user-log*.log', PG_BASE_DIR);

        if (!$file) {
            return $field;
        }

        $filesize = Helper::filesize($file);
        $date = date('d.m.Y H:i:s', filemtime($file));

        $file = pathinfo($file, PATHINFO_BASENAME);

        $field['message'] = sprintf(
            '<a href="%s" target="_blank">%1$s</a> <small>( %s, %s )</small>',
            plugins_url('log/'.$file, __DIR__),
            $filesize,
            $date
        );

        return $field;
    }

    public function logfile($file_pattern = 'log/*photobooth-gallery*.log', $base_dir = false)
    {

        $files = glob(Helper::baseLogDir($base_dir).$file_pattern);

        usort($files, function ($a, $b) {

            return filemtime($b) <=> filemtime($a);
        });

        return reset($files);
    }

    function render_one_click_install()
    {

        $settings = Helper::load_setting('Default Settings');
        $globals = Helper::get_plugin_settings();

        $uploader = new UploaderPro();

        $downloader = $uploader->setSettingsId($settings['settings-post-id'])
            ->get_download_link();

        $base_url = get_page_link($this->add_test_page());

        $webhook = [
            rtrim($base_url, '/'),
            'pg-upload',
            sprintf('%s-%s', $settings['webhook_upload_password'], $settings['settings-post-id']),

        ];

        $event_url = [
            rtrim($base_url, '/'),
            'pg-code',
            '[event-code]',
        ];

        $base = [
            'test_code' => 'ABC12345',
            'base_url' => $base_url,
            'event_url' => implode('/', $event_url),
            'admin_url' => implode('/', $event_url).'?PBGADMIN='.$settings['admin_code'],
            'webhook' => implode('/', $webhook),
            'download_tool' => $downloader,
            'logfile' => Helper::abs_path_to_url($this->logfile()),
        ];


        if (($stat = get_page_by_title('PBG-Statistic'))) {
            $url = [
                rtrim(get_page_link($stat), '/'),
                'pg-code',
                '[event-code]',
            ];

            $base['statistics_en'] = implode('/', $url);
        }

        if (($stat = get_page_by_title('PBG-Statistik'))) {
            $url = [
                rtrim(get_page_link($stat), '/'),
                'pg-code',
                '[event-code]',
            ];
            $base['statistics_de'] = implode('/', $url);
        }

        delete_option(PG_NAMESPACE.'-show-quick-install');

        echo $this->render->view('one-click-install', compact('settings', 'globals', 'base'));
    }

    function render_webhook($field)
    {

        $uuid = get_option('pg-webhook-uuid', false);

        if (!get_field('pg_cron_webhook', 'option')) {

            if ($uuid) {

                delete_option('pg-webhook-uuid');

                Helper::log('[ Webhook Configuration ] Webhook disabled, removing Webhook Link');

            }

            return $field;
        }

        if (!$uuid) {
            $uuid = uniqid('photobooth-gallery-');

            update_option('pg-webhook-uuid', $uuid);

            Helper::log(
                sprintf(
                    '[ Webhook Configuration ] Webhook enabled, generating new Webhook Link %s',
                    home_url('pg-webhook/'.$uuid)
                )
            );
        }

        $field['message'] = sprintf('<a href="%s" target="_blank">%1$s</a>', home_url('pg-webhook/'.$uuid));

        return $field;
    }

    function webhooks()
    {

        if (PG_DEBUG) {
            $this->admin_debug();
        }

        $webhook = get_query_var('pg-webhook');

        if (!$webhook) {
            return;
        }

        if ($webhook && get_field('pg_cron_webhook', 'option') && $webhook === get_option('pg-webhook-uuid', false)) {

            Helper::log('[ Webhook ] Media Library scan started by webhook action!');

            $this->cron_scan();
        }

    }

    private function admin_debug()
    {

    }

    function cron_scan(): void
    {

        $lock = LockMixin::createLock(Transients::DOING_CRON, 10 * MINUTE_IN_SECONDS);

        try {
            if ($lock->acquire() === false) {
                Helper::log('[ Cron Job ] Scan Cron still running, skipping this call...');

                return;
            }

            $this->file_management->scan_upload_folder();

        } catch (Exception $e) {
            Helper::log($e, 'error');
        } finally {
            $lock->release();
        }
    }

    function add_schedules($schedules)
    {

        $schedules['every_30_minutes'] = array(
            'interval' => 30 * 60,
            'display' => __('Every 30 Minutes', 'photobooth-gallery'),
        );

        $schedules['every_minute'] = array(
            'interval' => 1 * 60,
            'display' => __('Every Minute', 'photobooth-gallery'),
        );

        $schedules['every_15_minutes'] = array(
            'interval' => 15 * 60,
            'display' => __('Every 15 Minutes', 'photobooth-gallery'),
        );

        $schedules['every_5_minutes'] = array(
            'interval' => 5 * 60,
            'display' => __('Every 5 Minutes', 'photobooth-gallery'),
        );


        return $schedules;
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    function enqueue_styles()
    {

        wp_enqueue_style(
            $this->plugin_name.'-backend',
            plugin_dir_url(__FILE__)."css/{$this->plugin_name}-backend.min.css",
            array(),
            $this->version,
            'all'
        );

        if (!Helper::is_plugin_screen($this->plugin_name)) {
            return;
        }

        wp_enqueue_style(
            $this->plugin_name,
            plugin_dir_url(__FILE__)."css/{$this->plugin_name}.min.css",
            [],
            $this->version,
            'all'
        );
        wp_enqueue_style(
            'fontawesome-v5-free',
            plugin_dir_url(__FILE__)."css/all.css",
            array(),
            $this->version,
            'all'
        );
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    function enqueue_scripts()
    {

        wp_enqueue_script(
            'photobooth-backend-js',
            plugin_dir_url(__FILE__)."js/backend.min.js",
            array(
                'jquery',
            ),
            $this->version,
            true
        );

        $filter_files = get_option('pg-filter-media-files');

        wp_localize_script('photobooth-backend-js', 'pg_i18n', array(
            'pb_media_button' => $filter_files === 'all' ? __('Show Photobooth Media Files', 'photobooth-gallery') : __(
                'Hide Photobooth Media Files',
                'photobooth-gallery'
            ),
            'media_param' => $filter_files === 'all' ? 'pg-filter-media-off' : 'pg-filter-media-on',
        ));

        if (!Helper::is_plugin_screen($this->plugin_name)) {
            return;
        }

        wp_enqueue_script(
            $this->plugin_name,
            plugin_dir_url(__FILE__)."js/{$this->plugin_name}.min.js",
            array(
                'jquery',
                'clipboard-js',
                'underscore-js',
            ),
            $this->version,
            true
        );

        wp_enqueue_script(
            'clipboard-js',
            plugin_dir_url(__FILE__)."js/clipboard.js",
            array('jquery'),
            $this->version,
            true
        );
        wp_enqueue_script(
            'bootstrap',
            plugin_dir_url(__FILE__)."js/bootstrap.js",
            array('jquery'),
            $this->version,
            true
        );
        wp_enqueue_script(
            'underscore-js',
            plugin_dir_url(__FILE__)."js/underscore.js",
            array('jquery'),
            $this->version,
            true
        );

        wp_localize_script($this->plugin_name, Photobooth_Gallery::PREFIX.'i18n', array(
            'clean_up' => __(
                'Do you really want to remove <%- size %> Photobooth Media Files from the Wordpress Media Library?',
                'photobooth-gallery'
            ),
            'clean_up_setting' => __(
                'Do you really want to remove <%- size %> Photobooth Media Files, which belong to this configuration, from the Wordpress Media Library?',
                'photobooth-gallery'
            ),
            'statistic_import' => __(
                'Do you really want to import the statistical evaluation micro sites? Please keep in mind that any changes to these sites will be lost!',
                'photobooth-gallery'
            ),
            'reset_statistic' => __(
                'Do you really want to remove all statistical data from this wordpress installation? Every data regarding interactions with the gallery will be removed permanently and cannot be restored!',
                'photobooth-gallery'
            ),
        ));

        wp_localize_script($this->plugin_name, Photobooth_Gallery::PREFIX.'nonces', array(
            'scan_folder' => wp_create_nonce("pg-scan-folder"),
            'clean_up' => wp_create_nonce("pg-clean-up"),
            'scan_status' => wp_create_nonce('pg-scan-status'),
            'folder_size' => wp_create_nonce('pg-folder-size'),
            'statistic_import' => wp_create_nonce('pg-statistic-import'),
            'reset_statistic' => wp_create_nonce('pg-reset-statistic'),
            'download_tool' => wp_create_nonce('pg-download-tool'),
        ));

    }

    function handle_selection_value($value, $post_id, $field)
    {

        if ($field['name'] !== Photobooth_Gallery::PREFIX.'allowed_extensions') {
            return $value;
        }

        if (is_array($value)) {
            return $value;
        }

        $old_value = get_field(Photobooth_Gallery::PREFIX.'allowed_extensions', 'option');

        $value = $old_value;

        if (!is_array($old_value)) {
            $value = [];
        }

        return $value;
    }

    function cron_error_report()
    {

        // No Error, no cry ;)
        if (!get_transient(Transients::IS_ERROR)) {
            return;
        }

        // E-Mail has been disabled, skipping
        if (!$this->settings['error_reporting']) {
            return;
        }

        delete_transient(Transients::IS_ERROR);

        $subject = sprintf('Error Report from %s', site_url());

        $field = [
            'message' => '',
            'disable-php-info' => true,
        ];

        $field = $this->render_plugin_info($field);
        $logfile = $this->logfile();
        $attachments = [];

        if ($logfile) {
            $attachments = [$logfile];
        }

        $msg = str_replace('</li>', PHP_EOL, $field['message']);
        $msg = str_replace('<br/>&minus;&nbsp;', PHP_EOL, $msg);

        $msg = wp_strip_all_tags($msg);

        wp_mail(PG_DEV_MAIL, $subject, $msg, '', $attachments);

        Helper::log(sprintf('[ Cron Error Report] Sending Error Report to %s', PG_DEV_MAIL), 'info');
    }

    function render_plugin_info($field)
    {

        global $wp_version;

        $site = get_site_url();

        $path_info = parse_url($site);

        $extensions = get_loaded_extensions();
        array_unshift($extensions, '');
        $extensions = implode('<br/>&minus;&nbsp;', $extensions);

        $meta = [];

        $meta[] = sprintf('Plugin Version: %s', PHOTOBOOTH_GALLERY_VERSION);
        $meta[] = sprintf('WP_CONTENT_DIR: %s', WP_CONTENT_DIR);
        $meta[] = sprintf(__('Website Protocol: %s', 'photobooth-gallery'), $path_info['scheme']);
        $meta[] = sprintf('Website-Domain: %s', $path_info['host']);
        $meta[] = sprintf('Website-URL: %s', $site);
        $meta[] = sprintf(__('Absolute Path to Plugin: %s', 'photobooth-gallery'), PG_BASE_DIR);
        $meta[] = sprintf('WordPress-Version: %s', $wp_version);
        $meta[] = sprintf('System: %s', function_exists('php_uname') ? php_uname() : '');
        $meta[] = sprintf('PHP Version: %s', function_exists('phpversion') ? phpversion() : 'N/A');
        $meta[] = sprintf('PHP Max Execution Time: %s sec (>60 recommended)', ini_get('max_execution_time'));
        $meta[] = sprintf('PHP Memory Limit: %s (512M recommended)', ini_get('memory_limit'));
        $meta[] = sprintf('PHP Loaded Extensions: %s', $extensions);

        $meta = '<ul class="plugin-info">'.implode('', Helper::make_list($meta)).'</ul>';

        if (class_exists('DOMDocument') && !isset($field['disable-php-info'])) {
            ob_start();
            phpinfo();
            $phpinfo = ob_get_clean();

            $html = new DOMDocument();
            $html->preserveWhiteSpace = false;
            $html->loadHTML($phpinfo);

            $body = $html->getElementsByTagName('body');
            $tables = $html->getElementsByTagName('table');

            foreach ($tables as $table) {
                $table->setAttribute('class', 'wp-list-table widefat fixed striped php-info-table');
            }

            $node = '';

            foreach ($body as $node) {
                $node = $node->ownerDocument->saveXML($node);
                break;
            }

            $meta .= sprintf(
                '<a class="btn btn-primary" data-toggle="collapse" href="#php-info" role="button" aria-expanded="false" aria-controls="collapseExample">%s</a><div class="collapse" id="php-info">%s</div>',
                __('PHP Info for Devs', 'photobooth-gallery'),
                $node
            );
        }

        $field['message'] = $meta;

        return $field;
    }

    function error_shutdown()
    {

        $error = error_get_last();

        if (!$error) {
            return;
        }

        $msg = $error['message'] ?? false;

        if (!$msg) {
            return;
        }

        // For now, we will log only things our plugin did wrong
        if (strpos($error['file'], 'photobooth-gallery') === false) {
            return;
        }

        // We don't want to see deprecation logs
        if ($error['type'] & E_DEPRECATED) {
            return;
        }

        Helper::log('[ Shutdown Handler ] '.json_encode($error), 'error');

        // We only want to report on fatal errors once every 6 hours
        if ($error['type'] & ~E_ERROR) {
            return;
        }

        // We are only interested in fatal errors by now
        // The error is active 6 hours max
        set_transient(Transients::IS_ERROR, true, 6 * HOUR_IN_SECONDS);

        $this->remove_transients();
    }

    private function remove_transients()
    {

        delete_transient(Transients::CURRENTLY_SCANNING);
        delete_transient(Transients::DOING_CRON);
        delete_transient(Transients::SCAN_STATUS);
        delete_transient(Transients::DOING_XML_IMPORT);
    }

    function allowed_media_types($types)
    {


        $types['svg'] = 'image/svg+xml';
        $types['mp4'] = 'video/mp4';
        $types['xml'] = 'application/xml';

        return $types;
    }

    function ajax_download_tool()
    {

        check_ajax_referer('pg-download-tool', 'nonce');

        $uploader = new UploaderPro();

        $settings_id = intval(sanitize_text_field($_REQUEST['reference'] ?? ''));

        if ($settings_id <= 0) {
            return;
        }

        $url = $uploader->setSettingsId($settings_id)
            ->get_download_link();
        wp_send_json([
            'download_link' => $url,
        ]);
    }

    function ajax_reset_statistic()
    {

        check_ajax_referer('pg-reset-statistic', 'security');

        $deleted_rows = $this->social_stats->reset();

        wp_send_json([
            'done' => $deleted_rows,
        ]);
    }

    function ajax_import_statistic()
    {

        check_ajax_referer('pg-statistic-import', 'security');

        $this->add_elementor_templates();
        $this->add_rewrite_rules();
        $this->flush_rewrite_rules();

        wp_send_json([
            'done' => true,
        ]);
    }

    function flush_rewrite_rules()
    {

        if (!Helper::is_plugin_screen($this->plugin_name)) {
            return;
        }

        flush_rewrite_rules(true);
    }

    function ajax_folder_size()
    {

        check_ajax_referer('pg-folder-size', 'security');

        $settings_id = $_REQUEST['settings-id'] ?? false;

        if ($settings_id !== false) {
            $settings_id = sanitize_text_field($settings_id);
        }

        wp_send_json([
            'data' => $this->file_management->removeable_images($settings_id),
        ]);
    }

    function ajax_scan_status()
    {

        check_ajax_referer('pg-scan-status', 'security');

        $settings_id = $_REQUEST['settings-id'] ?? 'full';
        $view = $_REQUEST['is-view'] ?? false;

        if ($settings_id !== 'full') {
            $settings_id = sanitize_text_field($settings_id);
        }

        if ($view !== false) {
            $view = sanitize_text_field($view) === 'true';
        }

        $data = get_transient(Transients::SCAN_STATUS);

        if (!$data) {

            wp_send_json([
                'data' => false,
            ]);

            exit;
        }

        if ($settings_id !== 'full' && $data['config-id'] !== $settings_id) {
            $data = [];
        }

        if (!$view) {

            wp_send_json([
                'data' => $data,
            ]);

            exit;
        }

        echo $this->render->view('components/progressbar', $data);

        exit;
    }

    function ajax_clean_up()
    {

        check_ajax_referer('pg-clean-up', 'security');

        $settings_id = $_REQUEST['settings-id'] ?? false;

        if ($settings_id !== false) {
            $settings_id = intval(sanitize_text_field($settings_id));
        }

        $response = $this->clean_up_media_library($settings_id);

        if (!$response) {
            wp_send_json([
                'error' => __('Could not scan upload folders!', 'photobooth-gallery'),
            ]);

            exit;
        }

        $result = [];

        foreach ($response as $config => $file_operations) {

            if (!$file_operations) {
                $result[] = sprintf(
                    __('Scanning files from config "%s" was not successful!', 'photobooth-gallery'),
                    $config
                );
                continue;
            }

            $resp = array_values($file_operations);
            array_unshift($resp, $config);

            $result[] = vsprintf(
                __(
                    '%s: Found: %d, Moved: %d, Errors: %d, Skipped: %d, Removed from File-Management: %d',
                    'photobooth-gallery'
                ),
                $resp
            );
        }

        wp_send_json([
            'messages' => $result,
        ]);
    }

    private function clean_up_media_library($settings_id = false)
    {

        $response = [];

        $response['clean-up-media-library'] = $this->file_management->revert_images($settings_id);

        return $response;
    }

    function ajax_scan_folder()
    {

        check_ajax_referer('pg-scan-folder', 'security');

        if (get_transient(Transients::DOING_CRON)) {

            Helper::log('[ Ajax Scan Folder Button ] Cron currently running, skipping this request...');

            wp_send_json([
                'messages' => [__('Cron currently running, please wait!', 'photobooth-gallery')],
            ]);

            exit;
        }

        $settings_id = $_REQUEST['settings-id'] ?? false;

        if ($settings_id !== false) {
            $settings_id = intval(sanitize_text_field($settings_id));
        }

        $response = $this->file_management->scan_upload_folder($settings_id);

        if (!$response) {
            wp_send_json([
                'error' => __('Could not scan upload folders (Maybe currently running?) !', 'photobooth-gallery'),
            ]);

            exit;
        }

        $result = [];

        foreach ($response as $config => $file_operations) {

            if (!$file_operations) {
                $result[] = sprintf(
                    __('Scanning files from config "%s" was not successful!', 'photobooth-gallery'),
                    $config
                );
                continue;
            }

            $resp = array_values($file_operations);
            array_unshift($resp, $config);

            $result[] = vsprintf(
                __(
                    '%s: Total: %d, Valid: %d, New: %d, Duplicates: %d, Errors: %d, Skipped: %d',
                    'photobooth-gallery'
                ),
                $resp
            );
        }

        wp_send_json([
            'messages' => $result,
        ]);
    }

    function handle_image_size($sizes, $meta, $attachment_id)
    {

        $image_type = get_post_meta($attachment_id, 'type', true);

        if ($image_type !== 'photobooth-gallery-image') {
            return $sizes;
        }

        $sizes = array_filter(
            $sizes,
            function ($key) {

                if ($key === 'thumbnail') {
                    return true;
                }

                return strpos($key, 'pg-') !== false;
            },
            ARRAY_FILTER_USE_KEY
        );

        return $sizes;
    }

    public function load_log_content()
    {

        if (!current_user_can('manage_options')) {
            return;
        }

        require_once plugin_dir_path(__FILE__).'partials/logging.php';
    }

    function redirect_canonical($redirect)
    {

        if (!is_page()) {
            return $redirect;
        }

        $front_page = get_option('page_on_front');

        if (!is_page($front_page)) {
            return $redirect;
        }

        return false;
    }

    public function remove_all_crons()
    {

        $crons = [
            'pg-cron-scan',
            'pg-cron-cleanup',
            'pg-cron-error-reporting',
        ];

        foreach ($crons as $cron) {

            if (!wp_next_scheduled($cron)) {
                continue;
            }

            wp_clear_scheduled_hook($cron);
        }

        $this->xml_stats->deactivate();
        $this->remove_transients();
    }

    /**
     * @return bool
     */
    public function isUpdated(): bool
    {

        return $this->updated;
    }

    function load_iframe_docu()
    {

        include_once __DIR__.'/partials/iframe-documentation.php';
    }

    function load_iframe_upsell()
    {

        include_once __DIR__.'/partials/iframe-upsell.php';
    }
}

