<?php

/**
 * Define the internationalization functionality
 *
 * Loads and defines the internationalization files for this plugin
 * so that it is ready for translation.
 *
 * @link       https://www.codevision.io
 * @since      1.0.0
 *
 * @package    Photobooth_Gallery
 * @subpackage Photobooth_Gallery/includes
 */

use Codevision\Language\ACFPolyLang;

/**
 * Define the internationalization functionality.
 *
 * Loads and defines the internationalization files for this plugin
 * so that it is ready for translation.
 *
 * @since      1.0.0
 * @package    Photobooth_Gallery
 * @subpackage Photobooth_Gallery/includes
 * @author     codevision.io <info@codevision.io>
 */
class Photobooth_Gallery_i18n
{

    /**
     * Photobooth_Gallery_i18n constructor.
     */
    public function __construct()
    {
        add_filter('plugin_locale', [$this, 'handle_locales'], 1, 2);
        add_action('plugins_loaded', [$this, 'load_plugin_textdomain']);
        do_action('pbg/acfpolylang/register-textdomain');
    }

    function load_plugin_textdomain()
    {
        load_plugin_textdomain(PG_SLUG, false, dirname(dirname(plugin_basename(__FILE__))).'/languages/');

        $admin_url = admin_url('admin.php?page=acf-options-global-settings#acf-field_5b87ecc56b000');

        $poly_lang = new ACFPolyLang(
            PG_SLUG, apply_filters('pbg/acfpolylang/slugs', [
            'acf-options-global-settings',
            'pg_settings',
            'acf-options-pg-statistics',
        ]), apply_filters('pbg/acfpolylang/data', [
                'license_link_en' => '<a href="https://www.photobooth-deluxe.de/" target="_BLANK">here</a>',
                'license_link_de' => '<a href="https://www.photobooth-deluxe.de/" target="_BLANK">hier</a>',
                'admin_code' => '<b class="pg-admin-code">AdminCode</b>',
                'enter_title_de' => '<strong class="pg-settings" data-default="Titel hier eingeben">Titel hier eingeben</strong>',
                'enter_title_en' => '<strong class="pg-settings" data-default="Enter title here">Enter title here</strong>',
                'ul_css' => 'style="list-style-type: decimal; padding-left: 15px;"',
                'view_shortcode_en' => '<a href="#" class="btn btn-success btn-sm copy-shortcode-dyn" data-code="photobooth-gallery" data-target="#title">Copy Shortcode <i class="fa fa-check hidden"></i></a>',
                'view_shortcode_de' => '<a href="#" class="btn btn-success btn-sm copy-shortcode-dyn" data-code="photobooth-gallery" data-target="#title">Shortcode kopieren <i class="fa fa-check hidden"></i></a>',
                'social_media_shortcode_en' => '<a href="#" class="btn btn-success btn-sm copy-shortcode-dyn" data-code="view-photobooth-gallery" data-target="#title">Copy Shortcode <i class="fa fa-check hidden"></i></a>',
                'social_media_shortcode_de' => '<a href="#" class="btn btn-success btn-sm copy-shortcode-dyn" data-code="view-photobooth-gallery" data-target="#title">Shortcode kopieren <i class="fa fa-check hidden"></i></a>',
                'preview_shortcode_en' => '<a href="#" class="btn btn-success btn-sm copy-shortcode-dyn" data-code="preview-photobooth-gallery" data-target="#title">Copy Shortcode <i class="fa fa-check hidden"></i></a>',
                'preview_shortcode_de' => '<a href="#" class="btn btn-success btn-sm copy-shortcode-dyn" data-code="preview-photobooth-gallery" data-target="#title">Shortcode kopieren <i class="fa fa-check hidden"></i></a>',
                'uploader_pro_de' => '<a href="'.$admin_url.'" target="_blank">hier</a>',
                'uploader_pro_en' => '<a href="'.$admin_url.'" target="_blank">here</a>',
            ])
        );

        $poly_lang->init();
    }

    /**
     * Set the global english speaking domain to en_EN Photobooth_instead of using
     * en_UK, en_US and so on
     *
     * @param $locale
     * @param $text_domain
     *
     * @return string
     */
    function handle_locales($locale, $text_domain)
    {

        if ($text_domain !== PG_SLUG) {
            return $locale;
        }

        if (function_exists('get_field') === false) {
            return $locale;
        }

        $fixed_language = get_field('pg_i18n', 'option');

        if ($fixed_language !== 'default') {
            return $fixed_language;
        }

        if (str_contains($locale, 'de_') === true) {
            return 'de_DE';
        }

        return 'en_EN';
    }

    /**
     *
     * Translations for the setting page of this plugin
     *
     */
    protected function settings()
    {

        $i18n = [
            __('Plugin Information', 'photobooth-gallery'),
            __('Send Error Reports to Dev Team', 'photobooth-gallery'),
            __(
                'If this option is enabled, we will receive an email whenever an error with this plugin occurs. The email contains the error message, the system data you see here and the current error logfile. This helps us to provide you assistance and also to identify bugs.',
                'photobooth-gallery'
            ),
            __('Logfile Prefix', 'photobooth-gallery'),
            __('Please enter a unique prefix for your logfiles', 'photobooth-gallery'),
            __('Logfile Location', 'photobooth-gallery'),
            __('After the logfile has been generated you will find a link to it here', 'photobooth-gallery'),
            __('Schedule Upload Folder Scan', 'photobooth-gallery'),
            __('Scan Interval', 'photobooth-gallery'),
            __('Every 30 minutes', 'photobooth-gallery'),
            __('Every 15 minutes', 'photobooth-gallery'),
            __('Every 5 minutes', 'photobooth-gallery'),
            __('Enable Webhook Scan Initiation', 'photobooth-gallery'),
            __(
                'By enabling Webhook Scan Intiation you will find a special link here, which will initiate the Upload Folder Scan upon click',
                'photobooth-gallery'
            ),
            __('This field will be replaced by the webhook url after saving this form', 'photobooth-gallery'),
            __('Removal Timeout for Social Media shares', 'photobooth-gallery'),
            __(
                'Minimum age of the most recent access to a social media shared file before it can be deleted from the wordpress media library. <br/> Default: Access + 30 days',
                'photobooth-gallery'
            ),
            __('14 Days', 'photobooth-gallery'),
            __('30 Days', 'photobooth-gallery'),
            __('60 Days', 'photobooth-gallery'),
            __('90 Days', 'photobooth-gallery'),
            __('120 Days', 'photobooth-gallery'),
            __('180 Days', 'photobooth-gallery'),
            __('365 Days', 'photobooth-gallery'),
            __('Enable Auto Cleanup', 'photobooth-gallery'),
            __(
                'Automatically deletes all media data stored in the wordpress media library for longer than the selected time period. Already shared files are excluded unless the expiration date is exceeded',
                'photobooth-gallery'
            ),
            __('Disabled', 'photobooth-gallery'),
            __('Temporarily disable specific codes', 'photobooth-gallery'),
            __('Please enter codes you wish to exclude from the gallery view', 'photobooth-gallery'),

            __('Track Extended GEO-Data', 'photobooth-gallery'),
            __(
                'This service uses https://ipstack.com/ to extend GEO-data tracking. The API key provided is a free key which has limited usage throughout the month. If the API limit is reached and you whish to use this feature, please checkout IPStack.com at https://ipstack.com/',
                'photobooth-gallery'
            ),
            __('Allowed Multimedia Files Types', 'photobooth-gallery'),
            __('JPG Images', 'photobooth-gallery'),
            __('PNG Images', 'photobooth-gallery'),
            __('GIF Images', 'photobooth-gallery'),
            __('MP4 Video Files', 'photobooth-gallery'),
            __('Delete Media Files after Import', 'photobooth-gallery'),

            __(
                'The orignal files will be deleted if they have been successfully transferred to the media library',
                'photobooth-gallery'
            ),
            __('Cleanup media library', 'photobooth-gallery'),
            __(
                'Deletes all transferred files from the media library and copies the original files back to their respective Photobooth folder, provided the option "Delete Media Files after Import" is enabled. <br/>Attention: This also applies to files that are currently shared and are not allowed to be deleted. The <em>Internal Duplicate File Protection</em> will also be reset so that multimedia files that were already in the library can be added once again',
                'photobooth-gallery'
            ),
            __('Cleanup media library NOW', 'photobooth-gallery'),
            __('Media File Transfer', 'photobooth-gallery'),
            __(
                'All files from the defined photobooth folders of all active configurations are copied to the media library. Depending on the size of individual files and amount of files within the photobooth folders, this may take a while. If a timeout occurs, just restart the process.<br/>On decent hosting environments, one image takes about 1 second to be converted and transferred',
                'photobooth-gallery'
            ),

            __('Scan currently in progress...', 'photobooth-gallery'),
            __('Total Files found', 'photobooth-gallery'),
            __('Valid Files', 'photobooth-gallery'),
            __('Skipped Files', 'photobooth-gallery'),
            __('New', 'photobooth-gallery'),
            __('Duplicates', 'photobooth-gallery'),
            __('Erroneous', 'photobooth-gallery'),

            __('Skip specific Codes on Auto Clean Up', 'photobooth-gallery'),
            __(
                'Please enter codes you wish to exclude from auto clean up. Separate them by commata',
                'photobooth-gallery'
            ),

            __('Enter your Webhook password', 'photobooth-gallery'),
            __('Select target configuration', 'photobooth-gallery'),
            __('The password has to be lowercase and may only contain numbers and letters.', 'photobooth-gallery'),
            __('Disable Bootstrap Javascript', 'photobooth-gallery'),
            __(
                'If you have problems with the shipped bootstrap overriding / changing the behaviour of your wordpress site you can disable the inclusion of the bootstrap.js here. Please make sure that you have a bootstrap (Version 3.0+) loaded, otherwise the gallery will not work properly.',
                'photobooth-gallery'
            ),
            //__( '', 'photobooth-gallery' ),
        ];
    }

    protected function oneClickInstall()
    {
        $i18n = [
            __('Welcome to the Quick Install Guide for your Photobooth Gallery Version', 'photobooth-gallery'),
            __('Event specific information', 'photobooth-gallery'),
            __('Entry page for guests (microsite)', 'photobooth-gallery'),
            __('Complete url to specific event for QRCode usage', 'photobooth-gallery'),
            __('Test Now', 'photobooth-gallery'),
            __('Url to event gallery with admin privileges', 'photobooth-gallery'),
            __('German statistics page', 'photobooth-gallery'),
            __('English statistics page', 'photobooth-gallery'),
            __('Show me', 'photobooth-gallery'),
            __('Important Information', 'photobooth-gallery'),
            __('Recommended filename template in DSLRRemotePro for images and printfiles', 'photobooth-gallery'),
            __('Current date', 'photobooth-gallery'),
            __('Current time', 'photobooth-gallery'),
            __('The eventcode. You can define it in DSLRRemotePro.', 'photobooth-gallery'),
            __('The unique identifier', 'photobooth-gallery'),
            __('Current Logfile', 'photobooth-gallery'),
            __('Settings for Breeze Gallery Uploader and Breeze Booth for IPad', 'photobooth-gallery'),
        ];
    }

    /**
     * Translations for the configuration part of the plugin
     */
    protected function configs()
    {

        $i18n = [
            __('Yes', 'photobooth-gallery'),
            __('No', 'photobooth-gallery'),

            __('Manual', 'photobooth-gallery'),
            __(
                '<strong>Note: All files in the media folder will be processed. Subfolders will also be included.</Strong> <ul {ul_css}><li> Give your configuration "{enter_title_en}" a meaningful name.</li> <li>Save your configuration by clicking <strong>"Refresh"</strong> on the right. </li> <li>Use the following shortcode to display the input mask for this configuration: {view_shortcode_en}  </li> <li> If you have social media enabled: Select a view page. Within this page paste the following shortcode: {social_media_shortcode_en} This is required for the social media sharing experience. You can have multiple shortcodes with different configurations on the same page. </li><li> The <strong>anchor</strong> attribute of the shortcodes is optional and can be omitted. </li></ul>',
                'photobooth-gallery'
            ),
            __('Photobooth Media Folder', 'photobooth-gallery'),
            __('Create missing Photobooth Media Folder', 'photobooth-gallery'),
            __('Enable Preview Gallery', 'photobooth-gallery'),
            __(
                'Allow the usage of a new shortcode to show all images from this configuration Photobooth_without entering a gallery code',
                'photobooth-gallery'
            ),
            __('{preview_shortcode_en}', 'photobooth-gallery'),
            __('Regular expression', 'photobooth-gallery'),
            __(
                'Please enter your regular expression for the code validation here.<br/>The {uid} token from DSLRRemotePro consists of 3 letters and 5 digits which can be validated with: [A-Z][A-Z][A-Z][0-9][0-9][0-9][0-9][0-9] <br/>The {uid2} token from DSLRRemotePro consists of 5 letters and 3 digits which can be validated with: [A-Z][A-Z][A-Z][A-Z][A-Z][0-9][0-9][0-9] <br/>Notice: You may only use uppercase letters! The input field will automatically convert all lowercase to uppercase',
                'photobooth-gallery'
            ),
            __(
                'Please use the Admin Code like so: www.domain.de/my-photobooth-page?PBGADMIN={admin_code}',
                'photobooth-gallery'
            ),
            __('Display real file names for admin', 'photobooth-gallery'),
            __('Works only in conjunction with the Admin Code', 'photobooth-gallery'),
            __('Placeholder for the code input field', 'photobooth-gallery'),
            __('Autoscroll on click', 'photobooth-gallery'),
            __(
                'The browser scrolls back to the main view after loading the clicked image. Useful in the mobile version',
                'photobooth-gallery'
            ),
            __('Allow Direct Code Input', 'photobooth-gallery'),
            __(
                'Allows you to enter the code via the URL. The code must be entered as follows: www.domain.de/my-photobooth-page/pg-code/{the-code}',
                'photobooth-gallery'
            ),
            __('Enable Social Media Sharing', 'photobooth-gallery'),
            __('Headline for Social Media Buttons', 'photobooth-gallery'),
            __('Subject for Sharing with Social Media', 'photobooth-gallery'),
            __('Description for Sharing with Social Media', 'photobooth-gallery'),
            __('What should be shared', 'photobooth-gallery'),
            __('Current Image/Video', 'photobooth-gallery'),
            __('Complete Gallery of the code', 'photobooth-gallery'),
            __('Choose Social Media View Page', 'photobooth-gallery'),
            __(
                'If <strong>"Current Image/Video"</strong> is selected, no additional page is required. You may select the main page in which you inserted/want to insert the main shortcode of this configuration.<br/>With the selection <strong>"Complete Gallery of the code"</strong> you have to choose another page in which you insert the following shortcode {social_media_shortcode_en}',
                'photobooth-gallery'
            ),
            __('Facebook Messenger (mobile only)', 'photobooth-gallery'),
            __('Images per page', 'photobooth-gallery'),
            __('Pagination limits', 'photobooth-gallery'),
            __('Maximum pages to be shown in the pagination', 'photobooth-gallery'),
            __('Show first page / last page Buttons', 'photobooth-gallery'),
            __('Introduction text for the form', 'photobooth-gallery'),
            __('Error message for wrong code inputs', 'photobooth-gallery'),
            __('Enable IP Blocking', 'photobooth-gallery'),
            __('Here you can set the maximum number of failed attempts before blocking the user', 'photobooth-gallery'),
            __('Timeout in seconds', 'photobooth-gallery'),
            __('Maximum number of failed attempts', 'photobooth-gallery'),
            __('Error message if an User (IP-Block) was temporarily blocked', 'photobooth-gallery'),
            __('Download Gallery', 'photobooth-gallery'),
            __('iOS Video Share Sender E-Mail Address', 'photobooth-gallery'),
            __(
                'Due to iOS MP4 download limitations, video files will be send by email to the user requesting to download them. Please enter the sender mail address you wish to send the emails from. To prevent being added to spam lists, make sure to use an email address with the current domain.',
                'photobooth-gallery'
            ),
            __('Custom Image Ordering', 'photobooth-gallery'),
            __('Please select the Order By', 'photobooth-gallery'),
            __('Order by real filename', 'photobooth-gallery'),
            __('Order by date of upload', 'photobooth-gallery'),
            __('Custom Order direction', 'photobooth-gallery'),
            __('Ascending', 'photobooth-gallery'),
            __('Descending', 'photobooth-gallery'),
            __('Secure Statistic Page Access', 'photobooth-gallery'),
            __(
                'When activated the statistical evaluation page will only be displayed if a valid code has been supplied via the /pg-code/ url parameter field.',
                'photobooth-gallery'
            ),
            __('Admin access password', 'photobooth-gallery'),
            __(
                'Please enter a password to allow extended statistical requests via the /pg-code/ url extension. Only letters and numbers are allowed, anything else will be stripped from the password upon saving this form. The password must be appended to the request as following:<br/>{your_statistic_page}/pg-code/{your_code}/password/{your_password}<br/>If you wish to receive a global statistical evaluation omit {your_code}:<br/>{your_statistic_page}/pg-code/password/{your_password}<br/><strong>Passwords will automatically be cast to lowercase and all characters not allowed will be removed!</strong>',
                'photobooth-gallery'
            ),
            __('Import Statistic Page Data', 'photobooth-gallery'),
            __('Hide Download All Button', 'photobooth-gallery'),
            __(
                'You can choose to hide the download all button if the amount of media files is less than a specific number.',
                'photobooth-gallery'
            ),
            __('Choose Number of Media Files', 'photobooth-gallery'),
            __('Reset Statistics', 'photobooth-gallery'),
            __('Enable HTTP Post Upload', 'photobooth-gallery'),
            __('Enable slick.js gallery view', 'photobooth-gallery'),
            __('Content below Social Media Buttons', 'photobooth-gallery'),
            __(
                'Can be used for special tracking links or javascript snippets like proven experts',
                'photobooth-gallery'
            ),
            __(
                'After enabling the uploader and saving this form you can download an uploader tool for windows os in the configuration overview.',
                'photobooth-gallery'
            ),
            __('Share your Photo...', 'photobooth-gallery'),
            __('Here is your Photo!', 'photobooth-gallery'),
            __( 'Gallery Configuration', 'photobooth-gallery' ),
            __( 'Autorefresh on missing Code', 'photobooth-gallery' ),
            __( 'Enabling this will refresh the code input page every 30 seconds when no media for the given code was found', 'photobooth-gallery' ),
            __( 'Show custom 404 image', 'photobooth-gallery' ),
            __( 'Will display an image Photobooth_instead of the generic errormessage when no media file has been found yet', 'photobooth-gallery' ),
        ];
    }

    protected function license()
    {

        $i18n = [
            __('License Key', 'photobooth-gallery'),
            __('License Information', 'photobooth-gallery'),
            __('Payment Id', 'photobooth-gallery'),
            __(
                'You are using Photobooth Gallery Photobooth_without a valid License Key. You have <strong>%d</strong> days left for your trial.',
                'photobooth-gallery'
            ),
            __(
                'You are using Photobooth Gallery Photobooth_without a valid License Key. You have only <strong>one</strong> day left for your trial.',
                'photobooth-gallery'
            ),
            __(
                'Could not verify the License Key. Please enter a new License Key <a href=%s>here</a>. As long as no valid License Key is found, the Plugin will not work correctly.',
                'photobooth-gallery'
            ),

            __(
                'Please enter your License Key and our Payment Id which your received by email here. If you do not already have a (valid) License Key you can request one {license_link_en}.',
                'photobooth-gallery'
            ),

            __('You have a valid License Key, thank you for your support', 'photobooth-gallery'),
            __('Your Product', 'photobooth-gallery'),
            __('Last Payment', 'photobooth-gallery'),
            __('Next Payment', 'photobooth-gallery'),
            __('Paid until', 'photobooth-gallery'),
            __('Trial Start', 'photobooth-gallery'),
            __('Trial End', 'photobooth-gallery'),
        ];
    }

    protected function views()
    {

        $i18n = [
            __('Copy Shortcode', 'photobooth-gallery'),
            __('Scan Now', 'photobooth-gallery'),
            __('Clean Up', 'photobooth-gallery'),
            __('Clean Up Event', 'photobooth-gallery'),
            __(
                'This gallery only works with javascript enabled, please allow javascript on this page to use the gallery.',
                'photobooth-gallery'
            ),
            __(
                'If you download does not start within 3 seconds, please click the Download Now button to download your file.',
                'photobooth-gallery'
            ),

            //__( '', 'photobooth-gallery' ),
        ];
    }

    protected function modals()
    {

        $i18n = [
            __('Receive Video per Mail', 'photobooth-gallery'),
            __(
                'Due to limitations of iOS with regards to MP4 video downloading, we can only offer to send the selected video to you.',
                'photobooth-gallery'
            ),
            __(
                'Please enter your email address to receive the video by email. We will <strong>not</strong> store your email address, nor send other mails to you!',
                'photobooth-gallery'
            ),
            __('Email address', 'photobooth-gallery'),
            __('Close', 'photobooth-gallery'),
            __('Submit', 'photobooth-gallery'),

        ];
    }

}
