<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.codevision.io
 * @since      1.0.0
 *
 * @package    Photobooth_Gallery
 * @subpackage Photobooth_Gallery/includes
 */

use Photobooth\CustomPostTypes;
use Photobooth\Plugins\DisableApiAccessToPhotoboothImagesMixin;
use Photobooth\Plugins\VideoImageMixin;
use Photobooth\Plugins\WhatsappBusinessMixin;
use Photobooth\Plugins\AdvertisementMixin;
use Photobooth\Plugins\SmartTVMixin;
use Photobooth\Plugins\SwiperMixin;
use Photobooth\Plugins\SwiperTemplateMixin;

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Photobooth_Gallery
 * @subpackage Photobooth_Gallery/includes
 * @author     codevision.io <info@codevision.io>
 */
class Photobooth_Gallery
{

    const SETTINGS_SLUG = 'edit.php?post_type=pg_settings';
    const PREFIX = 'pg_';


    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Photobooth_Gallery_Loader $loader Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $plugin_name The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $version The current version of the plugin.
     */
    protected $version;

    /**
     * The License Manager
     *
     * @var \Codevision\Licensing\Manager
     */
    private $license_manager;

    /**
     * Photobooth_Gallery constructor.
     *
     * @param $license_manager
     */
    public function __construct($license_manager)
    {

        if (defined('PHOTOBOOTH_GALLERY_VERSION')) {
            $this->version = PHOTOBOOTH_GALLERY_VERSION;
        } else {
            $this->version = '1.0.0';
        }

        $this->plugin_name = 'photobooth-gallery';
        $this->license_manager = $license_manager;

        $this->load_dependencies();

        $this->define_admin_hooks();
        $this->define_public_hooks();

        $this->initialize_plugins();

        $this->loader->run();

    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Photobooth_Gallery_Loader. Orchestrates the hooks of the plugin.
     * - Photobooth_Gallery_i18n. Defines internationalization functionality.
     * - Photobooth_Gallery_Admin. Defines all hooks for the admin area.
     * - Photobooth_Gallery_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies()
    {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)).'includes/class-photobooth-gallery-loader.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)).'public/class-photobooth-gallery-public.php';

        $this->loader = new Photobooth_Gallery_Loader();
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks()
    {


        $plugin_admin = new Photobooth_Gallery_Admin($this->get_plugin_name(), $this->get_version());

        $this->loader->add_filter('acf/prepare_field/key=field_5b87ed6a01bdf', $plugin_admin, 'render_webhook');
        $this->loader->add_filter('acf/prepare_field/key=field_5b8809abe3cd2', $plugin_admin, 'render_logfile_info');
        $this->loader->add_filter('acf/prepare_field/key=field_5b62d0fcdb53c', $plugin_admin, 'render_folder_field');
        $this->loader->add_filter(
            'acf/prepare_field/key=field_5c0ed55f8a154',
            $plugin_admin,
            'render_email_sender_field'
        );

        $this->loader->add_filter(
            'acf/prepare_field/key=field_5d78fd209e7e5',
            $plugin_admin,
            'render_webhook_password_field'
        );

        $this->loader->add_filter(
            'acf/prepare_field/key=field_5cee2b5c97d59',
            $plugin_admin,
            'render_lib_versions'
        );
        $this->loader->add_filter('acf/prepare_field/key=field_5cf121380aa82', $plugin_admin, 'render_cpu_field');


        $this->loader->add_filter(
            'acf/prepare_field/key=field_5b86f1c7241b4',
            $plugin_admin,
            'render_default_settings'
        );
        $this->loader->add_filter(
            'acf/prepare_field/key=field_5b62d153db53f',
            $plugin_admin,
            'render_default_settings'
        );
        $this->loader->add_filter(
            'acf/prepare_field/key=field_5b62d2b6e290b',
            $plugin_admin,
            'render_default_settings'
        );
        $this->loader->add_filter(
            'acf/prepare_field/key=field_5b62d2d4e290c',
            $plugin_admin,
            'render_default_settings'
        );
        $this->loader->add_filter(
            'acf/prepare_field/key=field_5b7fe73715d7c',
            $plugin_admin,
            'render_default_settings'
        );
        $this->loader->add_filter(
            'acf/prepare_field/key=field_5b62d1a3db542',
            $plugin_admin,
            'render_default_settings'
        );
        $this->loader->add_filter(
            'acf/prepare_field/key=field_5b62d1b9db543',
            $plugin_admin,
            'render_default_settings'
        );

        $this->loader->add_filter('acf/prepare_field/key=field_5b757fb81b2ca', $plugin_admin, 'load_default_share');


        $this->loader->add_filter('acf/update_value/key=field_5b62d134db53e', $plugin_admin, 'save_regex_field', 10, 3);
        $this->loader->add_filter(
            'acf/update_value/key=field_5c69d3bc07a72',
            $plugin_admin,
            'save_statistic_password_field',
            10,
            3
        );
        $this->loader->add_filter(
            'acf/update_value/key=field_5cecee9d5c93f',
            $plugin_admin,
            'save_elementor_status',
            20
        );


        $this->loader->add_action('acf/render_field/type=button_group', $plugin_admin, 'render_delete_button');
        $this->loader->add_filter('acf/load_field/key=field_5b93dc653c837', $plugin_admin, 'render_plugin_info');
        $this->loader->add_filter('acf/get_field_group', $plugin_admin, 'add_bootstrap_class');

        $this->loader->add_action('init', $plugin_admin, 'init');
        $this->loader->add_action('init', $plugin_admin, 'init_acf');
        $this->loader->add_action('admin_menu', $plugin_admin, 'add_admin_page', 105);

        $this->loader->add_filter('manage_media_columns', $plugin_admin, 'media_columns');
        $this->loader->add_action('manage_media_custom_column', $plugin_admin, 'media_columns_values', 10, 2);
        $this->loader->add_filter('manage_upload_sortable_columns', $plugin_admin, 'media_columns_sortable');
        $this->loader->add_filter('media_row_actions', $plugin_admin, 'media_row_actions');
        $this->loader->add_action('delete_attachment', $plugin_admin, 'media_delete_attachments');


        $this->loader->add_filter(
            sprintf('manage_%s_posts_columns', CustomPostTypes::SETTINGS_SLUG),
            $plugin_admin,
            'settings_columns'
        );
        $this->loader->add_action(
            sprintf('manage_%s_posts_custom_column', CustomPostTypes::SETTINGS_SLUG),
            $plugin_admin,
            'settings_columns_values',
            10,
            2
        );
        $this->loader->add_filter('post_row_actions', $plugin_admin, 'settings_row_actions', 10, 2);


        $this->loader->add_action('pre_get_posts', $plugin_admin, 'media_columns_ordering', 1);

        $this->loader->add_action('acf/save_post', $plugin_admin, 'flush_rewrite_rules');

        $this->loader->add_filter('cron_schedules', $plugin_admin, 'add_schedules');

        $this->loader->add_action('pg-cron-scan', $plugin_admin, 'cron_scan');
        $this->loader->add_action('pg-cron-cleanup', $plugin_admin, 'cron_cleanup');
        $this->loader->add_action('pg-cron-error-reporting', $plugin_admin, 'cron_error_report');

        $this->loader->add_filter('acf/update_value/type=select', $plugin_admin, 'handle_selection_value', 10, 3);

        $this->loader->add_action('template_redirect', $plugin_admin, 'webhooks');
        $this->loader->add_filter('redirect_canonical', $plugin_admin, 'redirect_canonical');

        $this->loader->add_action('wp_ajax_pg-scan-folder', $plugin_admin, 'ajax_scan_folder');
        $this->loader->add_action('wp_ajax_pg-statistic-import', $plugin_admin, 'ajax_import_statistic');
        $this->loader->add_action('wp_ajax_pg-reset-statistic', $plugin_admin, 'ajax_reset_statistic');
        $this->loader->add_action('wp_ajax_pg-download-tool', $plugin_admin, 'ajax_download_tool');

        $this->loader->add_action('wp_ajax_pg-clean-up', $plugin_admin, 'ajax_clean_up');
        $this->loader->add_action('wp_ajax_pg-clean-up-event', $plugin_admin, 'ajax_clean_up_event');
        $this->loader->add_action('wp_ajax_pg-scan-status', $plugin_admin, 'ajax_scan_status');
        $this->loader->add_action('wp_ajax_pg-folder-size', $plugin_admin, 'ajax_folder_size');

        $this->loader->add_action('wp_ajax_pg-admin-delete', $plugin_admin, 'ajax_delete_images');
        $this->loader->add_action('wp_ajax_nopriv_pg-admin-delete', $plugin_admin, 'ajax_delete_images');

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

        $this->loader->add_filter('upload_mimes', $plugin_admin, 'allowed_media_types');

        // Handle Image Resizing on Upload
        $this->loader->add_filter(
            'pb/file_management/intermediate_image_sizes_advanced',
            $plugin_admin,
            'handle_image_size',
            10,
            3
        );

        register_shutdown_function([$plugin_admin, 'error_shutdown']);
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @return    string    The name of the plugin.
     * @since     1.0.0
     */
    public function get_plugin_name()
    {

        return $this->plugin_name;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @return    string    The version number of the plugin.
     * @since     1.0.0
     */
    public function get_version()
    {

        return $this->version;
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks()
    {

        $plugin_public = new Photobooth_Gallery_Public($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts', 9999);

        if (!$this->license_manager->is_valid()) {
            return;
        }

        $this->loader->add_action('init', $plugin_public, 'init');

        $this->loader->add_action('wp_ajax_pg-enter-the-code', $plugin_public, 'ajax_validate_code');
        $this->loader->add_action('wp_ajax_nopriv_pg-enter-the-code', $plugin_public, 'ajax_validate_code');

        $this->loader->add_action('wp_ajax_pg-download-zip', $plugin_public, 'ajax_generate_zip_archive');
        $this->loader->add_action('wp_ajax_nopriv_pg-download-zip', $plugin_public, 'ajax_generate_zip_archive');

        $this->loader->add_action('wp_ajax_pg-log-action', $plugin_public, 'ajax_log_action');
        $this->loader->add_action('wp_ajax_nopriv_pg-log-action', $plugin_public, 'ajax_log_action');

        $this->loader->add_action('wp_ajax_pg-admin-delete', $plugin_public, 'ajax_delete_images');
        $this->loader->add_action('wp_ajax_nopriv_pg-admin-delete', $plugin_public, 'ajax_delete_images');

        $this->loader->add_action('wp_ajax_pg-zip-status', $plugin_public, 'ajax_zip_status');
        $this->loader->add_action('wp_ajax_nopriv_pg-zip-status', $plugin_public, 'ajax_zip_status');


        $this->loader->add_action('wp_ajax_pg-mp4-mailer', $plugin_public, 'ajax_mp4_mailer');
        $this->loader->add_action('wp_ajax_nopriv_pg-mp4-mailer', $plugin_public, 'ajax_mp4_mailer');

        $this->loader->add_action('wp_ajax_pg-next-page', $plugin_public, 'ajax_next_page');
        $this->loader->add_action('wp_ajax_nopriv_pg-next-page', $plugin_public, 'ajax_next_page');


        $this->loader->add_action('template_redirect', $plugin_public, 'webhooks');

        $this->loader->add_action('wp_head', $plugin_public, 'add_meta_tags');

        $this->loader->add_filter('language_attributes', $plugin_public, 'doctype_opengraph');

        $this->loader->add_filter('query_vars', $plugin_public, 'add_query_vars');

        $this->loader->add_filter('posts_clauses', $plugin_public, 'allow_images_by_settings', 10, 2);

        $this->loader->add_action('pre_get_posts', $plugin_public, 'handle_incorrect_pg_codes');

    }

    public static function registerTextDomains() : void {
        WhatsappBusinessMixin::registerTextDomain();
    }

    private function initialize_plugins()
    {

        (new VideoImageMixin())->init();
        (new DisableApiAccessToPhotoboothImagesMixin())->init();
        (new WhatsappBusinessMixin())->init();
        (new AdvertisementMixin())->init();
        (new SmartTVMixin())->init();
        (new SwiperMixin())->init();
        (new SwiperTemplateMixin())->init();
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run()
    {

        $this->loader->run();
    }

    public static function get_config_defaults()
    {

        $blog_name = get_option('blogname');
        $blog_url = site_url();

        $base_config = [
            'media_folder' => '../../photobooth',
            'is_create_media_folder' => true,
            'regex' => '[A-Z][A-Z][A-Z][0-9][0-9][0-9][0-9][0-9]',
            'admin_code' => uniqid(),
            'is_show_filenames' => true,
            'placeholder' => __('Please enter your personal PhotoPIN', 'photobooth-gallery'),
            'is_auto_scroll' => true,
            'is_code_in_url_allowed' => true,
            'is_social_media' => true,
            'is_preview_gallery' => false,
            'is_custom_image_order' => false,
            'images_per_page' => 6,
            'pagination_size' => 6,
            'is_show_first_last_page' => true,
            'is_ip_ban_enabled' => true,
            'ip_ban_expire_in_seconds' => 30,
            'maxium_retries' => 10,
            'is_http_post_upload_enabled' => true,
            'is_slick_gallery' => false,
            'is_show_404' => true,
            'is_code_refresh' => false,
            'social_media_output' => '',
            'is_webhook_upload' => true,
            'webhook_upload_password' => uniqid(),
            'introtext' => sprintf(__('Welcome to %s', 'photobooth-gallery'), $blog_name),
            'is_show_ads' => false,
            'ad_filename_pattern' => '_ad_',
            'ad_distance' => 5,
            'ad_seconds_to_show' => 6,
            'social_media_description' => sprintf(
                __(
                    'This content was created by %s. If you want to know more about us, please visit %s!',
                    'photobooth-gallery'
                ),
                $blog_name,
                $blog_url
            ),
            'error_to_many_code_tries' => __(
                'You have tried entering too many codes. Try again later',
                'photobooth-gallery'
            ),
            'error_wrong_code' => __(
                'The Code you have entered is not correct, please try again!',
                'photobooth-gallery'
            ),
            'social_media_headline' => __('Share your Photo...', 'photobooth-gallery'),
            'social_media_subject' => __('Here is your Photo!', 'photobooth-gallery'),
            'social_media_buttons' => [
                'download-all',
                'download',
                'e-mail',
                'twitter',
                'facebook',
                'fb-messenger',
                'linked-in',
                'pinterest',
                'whatsapp',
                'wechat',
            ],
        ];

        return $base_config;
    }

    public static function get_defaults()
    {

        $prefix = substr(uniqid('', true), 0, 5).'-';

        $global_config = [

            self::PREFIX.'is_enabled_cron' => false,
            self::PREFIX.'unlink' => true,
            self::PREFIX.'allowed_extensions' => ['jpg,jpeg', 'png', 'mp4', 'xml'],
            self::PREFIX.'scan_interval' => 'every_30_minutes',
            self::PREFIX.'cron_webhook' => false,
            self::PREFIX.'auto_clean_interval' => 'disabled',
            self::PREFIX.'logfile_prefix' => $prefix,
            self::PREFIX.'max_social_share' => '30_days',
            self::PREFIX.'is_extended_geo_data' => false,
            self::PREFIX.'ip_stack_api_key' => '28be4a19703fe5b77fd8add13afac118',
            self::PREFIX.'statistic_complete' => false,
            self::PREFIX.'statistic_configuration' => 'Default Settings',
            self::PREFIX.'statistic_from' => '20180101',
            self::PREFIX.'statistic_to' => '20501231',
            self::PREFIX.'error_reporting' => false,
            self::PREFIX.'skip_auto_clean' => 'ABCDE123,ABC12345',
            self::PREFIX.'skip_codes' => '',

            self::PREFIX.'is_bootstrap_js_disabled' => false,
            self::PREFIX.'is_video-js_js_disabled' => false,
        ];

        $statistics_config = [
            self::PREFIX.'is_statistic_secured' => true,
            self::PREFIX.'statistic_password' => uniqid(),
            self::PREFIX.'is_elementor_disabled' => false,
        ];

        return array_merge($global_config, $statistics_config);
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @return    Photobooth_Gallery_Loader    Orchestrates the hooks of the plugin.
     * @since     1.0.0
     */
    public function get_loader()
    {

        return $this->loader;
    }

    public function deactivate()
    {

        $plugin_admin = new Photobooth_Gallery_Admin($this->get_plugin_name(), $this->get_version());

        $plugin_admin->remove_all_crons();

    }


}
