<?php
/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 2019-01-09
 * Time: 07:48
 */

namespace Photobooth\Elementor;


use Photobooth\Helper;

define('PG_ELEMENTOR_IMPORT_VERSION', '1.0.0');

class Importer {

	const IDENTIFIER = 'Photobooth Gallery Elementor Importer';

	/**
	 * @var string
	 */
	private $source_folder;

	/**
	 * @var string
	 */
	private $language_slug;

	/**
	 * Importer constructor.
	 *
	 * @param string $source_folder
	 * @param string $language_slug
	 */
	public function __construct( string $source_folder, string $language_slug ) {
		$this->source_folder = Helper::ensureTrailingBackslash( $source_folder );
		$this->language_slug = $language_slug;
	}

	public function scan_folder( $import_all = true ) {

		foreach ( glob( $this->source_folder . '*.json' ) as $template ) {

			$data = json_decode( file_get_contents( $template ), true );

			if ( ! $data ) {
				continue;
			}

			$this->check_node( $data );

			$this->add_page( $data );

			if ( ! $import_all ) {
				break;
			}
		}

	}

	/**
	 * Add new Elementor Page
	 *
	 * @param $content
	 *
	 * @return bool|int|\WP_Error
	 */
	private function add_page( $content ) {

		// We must find a title attribute
		if ( ! array_key_exists( 'title', $content ) ) {
			return false;
		}

		// We must find a content attribute
		if ( ! array_key_exists( 'content', $content ) ) {
			return false;
		}

		// Translate the generic title, if applicable
		//$title = __( $content['title'], $this->language_slug );
		$title = $content['title'];

		$page = get_page_by_title( $title );

		// Always remove the previous generated page
		if ( $page instanceof \WP_Post) {
			wp_delete_post( $page->ID, true );
		}

		$post_id = wp_insert_post( [
			'post_type'   => 'page',
			'post_title'  => $title,
			'post_status' => 'publish',
		] );

		if ( ! $post_id ) {
			Helper::log( sprintf( '[ Elementor Importer ] Could not create new elementor page %s', $title ), 'error' );

			return false;
		}

		// Add the postmeta for elementor usage
		$meta = [
			'_elementor_edit_mode'     => 'builder',
			'_elementor_template_type' => 'post',
			'_wp_page_template'        => 'elementor_canvas',
			'_elementor_data'          => wp_slash( json_encode( $content['content'] ) ),
			'imported_by'              => self::IDENTIFIER,
			'import_version'           => PG_ELEMENTOR_IMPORT_VERSION
		];

		foreach ( $meta as $key => $value ) {
			update_post_meta( $post_id, $key, $value );
		}

		return $post_id;
	}


	private function check_node( & $node, $meta = [] ) {

		if ( ! is_array( $node ) ) {
			return;
		}

		foreach ( $node as $key => & $sub_node ) {

			if ( is_array( $sub_node ) ) {

				$this->check_node( $sub_node, $meta );

				continue;

			}

			if ( $key !== 'url' ) {

				continue;
			}

			if ( strpos( $sub_node, 'http' ) === false ) {

				continue;
			}

			$img_data = $this->handle_elementor_image( $sub_node, $meta );

			if ( is_string( $img_data ) ) {

				continue;
			}

			list( $attachment, $attachment_id ) = array_values( $img_data );

			// Attachment is the attachment source from wordpress
			$sub_node = reset( $attachment );

			// Because we deal with an URL | ID array here, we must check if there is an key named 'id' here
			// to set the correct image id, otherwise elementor will display the right image in
			// editor and load the wrong one otherwise
			if ( ! array_key_exists( 'id', $node ) ) {
				continue;
			}

			$node['id'] = $attachment_id;
		}
	}

	private function handle_elementor_image( $url, $meta = [] ) {

		$attachment_id = $this->remote_upload( $url, $meta );

		if ( $attachment_id === false ) {
			return $url;
		}

		$attachment = wp_get_attachment_image_src( $attachment_id, 'original' );

		if ( ! is_array( $attachment ) ) {
			return $url;
		}

		return compact( 'attachment', 'attachment_id' );
	}

	private function get_real_filename( $url ) {

		// Special case this file is part of a shared file from an owncloud
		if ( strpos( $url, 'index.php' ) !== false ) {

			$parts = explode('=', $url );

			return end( $parts );
		}

		$file_name = basename( $url );

		return $file_name;
	}

	/**
	 * Uploads an image to the Wordpress media library but checks beforehand if this image already exists
	 * the unique id is the hashed url here
	 *
	 * @param $url
	 * @param array $meta_data
	 *
	 * @return bool|int|\WP_Error
	 */
	private function remote_upload( $url, $meta_data = [] ) {

		$post_id = $this->image_exists( $url );

		if ( $post_id !== false ) {
			return $post_id;
		}

		$get = wp_remote_get( $url );

		if ( $get instanceof \WP_Error ) {

			Helper::log( sprintf( '[ Image Uploader ] Could not upload remote url <%s> to media folder ( %s )', $url, $get->get_error_message() ), 'error' );

			return false;
		}


		$file_name = $this->get_real_filename( $url );

		$upload_file = wp_upload_bits( $file_name, null, wp_remote_retrieve_body( $get ) );

		if ( $upload_file['error'] ) {

			Helper::log( sprintf( '[ Image Uploader ] Could not upload File to media folder <%s>: %s', $file_name, $upload_file['error'] ), 'error' );

			return false;
		}

		$wp_filetype = wp_check_filetype( $file_name, null );

		$attachment = array(
			'post_mime_type' => $wp_filetype['type'],
			'post_title'     => preg_replace( '/\.[^.]+$/', '', $file_name ),
			'post_content'   => '',
			'post_status'    => 'inherit'
		);

		$attachment_id = wp_insert_attachment( $attachment, $upload_file['file'] );

		if ( is_wp_error( $attachment_id ) ) {
			Helper::log( sprintf( '[ Image Uploader ] Could not upload File to media folder <%s>: %s', $file_name, json_encode( $attachment_id->get_error_messages() ) ), 'error' );

			return false;
		}

		$meta_data['remote-upload-url-hash'] = sha1( $url );

		foreach ( $meta_data as $key => $value ) {
			update_post_meta( $attachment_id, $key, $value );
		}

		require_once( ABSPATH . "wp-admin" . '/includes/media.php' );
		require_once( ABSPATH . "wp-admin" . '/includes/image.php' );

		$attachment_data = wp_generate_attachment_metadata( $attachment_id, $upload_file['file'] );

		wp_update_attachment_metadata( $attachment_id, $attachment_data );

		return $attachment_id;
	}

	/**
	 * checks if an image with the given url has already been uploaded
	 *
	 * @param $url
	 *
	 * @return bool|int
	 */
	private function image_exists( $url ) {
		global $wpdb;

		$query = sprintf( "SELECT pm.post_id AS post_id FROM %s pm WHERE pm.meta_key = '%s' AND pm.meta_value = '%s'", $wpdb->postmeta, esc_sql( 'remote-upload-url-hash' ), esc_sql( sha1( $url ) ) );

		$result = $wpdb->get_results( $query );

		if ( ! $result ) {
			return false;
		}

		if ( ! is_array( $result ) ) {
			return false;
		}

		$result = reset( $result );

		return intval( $result->post_id );
	}
}