<?php
/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 24.09.2018
 * Time: 15:38
 */

namespace Photobooth;

use DateTime;
use Photobooth\GuzzleHttp\Client;
use Photobooth\GuzzleHttp\Pool;
use Photobooth\GuzzleHttp\Psr7\Request;
use Photobooth\Plugins\LockMixin;

define('PHOTOBOOTH_GALLERY_DB_VERSION_UPLOAD', '1.0.3');

class FileManagement
{

    /**
     * @var array
     */
    private $file_management = [];

    public static function init()
    {

        $installed_ver = get_option('PHOTOBOOTH_GALLERY_DB_VERSION_UPLOAD');

        if ($installed_ver === PHOTOBOOTH_GALLERY_DB_VERSION_UPLOAD) {
            return;
        }

        $table_name = self::table();

        $sql = "CREATE TABLE {$table_name} (
                id mediumint(9) NOT NULL AUTO_INCREMENT,
                `c_hash` VARCHAR(64) NOT NULL,
                `c_transferred` TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                `c_reference` INT NOT NULL,
                `c_filename` MEDIUMTEXT NOT NULL,
                `c_source` MEDIUMTEXT NOT NULL,
                `c_config` VARCHAR(255) NOT NULL,
                `c_config_id` INT NOT NULL,
                
                INDEX `hash` (`c_hash`),
                INDEX `reference` (`c_reference`),
                INDEX `config` (`c_config`),
                INDEX `config_id` (`c_config_id`),
                PRIMARY KEY (id)
        );";

        require_once(ABSPATH.'wp-admin/includes/upgrade.php');

        dbDelta($sql);

        update_option('PHOTOBOOTH_GALLERY_DB_VERSION_UPLOAD', PHOTOBOOTH_GALLERY_DB_VERSION_UPLOAD);
    }

    public function cleanup_zipfiles($interval)
    {

        // Set default value for interval to be disabled
        if (!$interval) {
            $interval = 'disabled';
        }

        if ($interval === 'disabled') {

            return 0;
        }

        $removed = $this->remove_expired_zip($interval);

        return $removed;
    }

    private function remove_expired_zip($interval)
    {

        $folder = PG_CACHE_PATH;

        if (!is_dir($folder)) {
            return 0;
        }

        $zip_files = Helper::recursive_search($folder, 'zip');
        $min_age = strtotime('-'.str_replace('_', '', $interval));

        $removed = 0;
        foreach ($zip_files as $file) {
            $file_date = filectime($file);

            if ($file_date > $min_age) {
                continue;
            }

            if (!@unlink($file)) {
                continue;
            }

            $removed++;
        }

        return $removed;
    }

    public function cleanup_images($interval, $skip_codes = [])
    {

        // Set default value for interval to be disabled
        if (!$interval) {
            $interval = 'disabled';
        }

        if ($interval === 'disabled') {

            return false;
        }


        return [
            'clean-up-media-library-cron' => $this->revert_images(false, $interval, $skip_codes),
        ];
    }

    public function revert_images($settings_id = false, $interval = false, $skip_codes = [])
    {

        $result = [
            'found' => 0,
            'moved' => 0,
            'error' => 0,
            'skipped' => 0,
            'storage' => 0,
        ];

        $attachments = $this->get_imported_attachments();

        if ($settings_id === false && !$attachments && $interval === false) {

            $result['storage'] = $this->cleanup_filestorage();

            return $result;
        }

        if (!$attachments) {
            return $result;
        }

        if ($settings_id !== false) {
            $this->match_attachements_with_files($attachments);
        }

        $result['found'] = sizeof($attachments);

        $id_set = [];
        $error_set = [];
        $update_option = 0;
        foreach ($attachments as $attachment) {

            if ($update_option++ % 15) {
                Helper::set_php_environment();
            }


            $file = get_attached_file($attachment->post_id);

            if (!$file) {
                Helper::log(
                    sprintf(
                        '[ Image Restore ] Could not get real File Path from Attachment %d ...',
                        $attachment->post_id
                    ),
                    'error'
                );

                $result['error']++;
                continue;
            }

            // If the file is newer than the minimum date required to be removed
            if (!$this->is_autoremovable_file($attachment, $interval, $skip_codes)) {

                $result['skipped']++;
                continue;
            }

            $managed_file = $this->file_management[$attachment->post_id] ?? false;

            // Skip the file if it does not belong to the given setting
            if ($managed_file !== false && $managed_file->c_config_id && $settings_id !== intval(
                    $managed_file->c_config_id
                )) {
                $result['skipped']++;
                continue;
            }

            // Do not remove files, which may not be removed at all
            if ($managed_file !== false && $managed_file->is_deleteable === false) {
                $result['skipped']++;
                continue;
            }

            if (!wp_delete_attachment($attachment->post_id, true)) {

                Helper::log(
                    sprintf('[ Image Restore ] Could not delete Attachment %d ...', $attachment->post_id),
                    'error'
                );

                $error_set[] = $attachment->post_id;

                $result['error']++;
                continue;
            }

            if ($settings_id === false) {
                continue;
            }

            $id_set[] = $attachment->post_id;

            Helper::log(sprintf('[ Image Restore ] Successfully removed %d ...', $attachment->post_id), 'info');
        }

        $this->cleanup_metadata($error_set);

        if ($settings_id === false && $interval === false) {
            $result['storage'] = $this->cleanup_filestorage($id_set);
        }

        return $result;
    }

    public function get_imported_attachments()
    {

        global $wpdb;

        $query = sprintf(
            "SELECT posts.ID as post_id 
								 FROM {$wpdb->posts} as posts RIGHT JOIN {$wpdb->postmeta} as post_meta ON (post_meta.post_id = posts.id) 
								 WHERE post_meta.meta_key = 'type' AND post_meta.meta_value = 'photobooth-gallery-image' 
								 AND posts.post_type = 'attachment' ORDER BY post_id ASC"
        );

        $attachments = $wpdb->get_results($query);

        return $attachments;
    }

    private function cleanup_filestorage($id_set = [])
    {

        global $wpdb;

        $table = $this->get_table();

        $query = "DELETE FROM {$table}";

        if ($id_set) {
            $query .= sprintf(' WHERE c_reference IN (%s)', implode(',', $id_set));
        }

        Helper::log(sprintf('[ cleanup_filestorage ] Running Query %s ...', $query), 'info');

        return $wpdb->query($query);
    }

    public function get_table()
    {
        return self::table();
    }

    public static function table()
    {

        global $wpdb;

        $table_name = $wpdb->prefix.'pg_file_management';

        return esc_sql($table_name);
    }

    private function match_attachements_with_files($attachements)
    {

        $this->file_management = [];

        $id_list = array_reduce(
            $attachements,
            function ($carry, $attachment) {

                $carry[] = $attachment->post_id;

                return $carry;
            },
            []
        );

        if (!$id_list) {
            return;
        }

        global $wpdb;

        $max_age = get_field('pg_max_social_share', 'options');

        if (!$max_age) {
            $max_age = '30_days';
        }

        $max_age = strtotime('-'.str_replace('_', '', $max_age));

        $query = sprintf("SELECT * FROM {$this->get_table()} WHERE c_reference IN (%s) ", implode(',', $id_list));

        $files = $wpdb->get_results($query);

        $social_sharing = SocialMedia::getinstance();
        $stats = $social_sharing->get_latest_social_stats_for($id_list);

        foreach ($files as $file) {

            if ($file->c_config_id) {
                $file->c_config_id = intval($file->c_config_id);
            }

            $file->is_deleteable = true;

            $stat = $stats[$file->c_reference] ?? false;

            if ($stat) {
                $date = new DateTime($stat->c_timestamp);

                $file->is_deleteable = $date->getTimestamp() <= $max_age;
            }

            $this->file_management[$file->c_reference] = $file;
        }

    }

    public function is_autoremovable_file($attachment, $interval, $skip_codes = [])
    {

        if (!$attachment) {
            return false;
        }

        if (!$interval) {
            return true;
        }

        if ($interval === 'disabled') {
            return true;
        }

        // Skip this file if the filename contains at least one code from the skip_codes list
        if ($skip_codes) {

            $filename = get_post_meta($attachment->post_id, 'real_filename', true);

            if (!$filename) {
                return true;
            }

            $filename = strtolower($filename);

            foreach ($skip_codes as $code) {

                if (!$code) {
                    continue;
                }

                $code = strtolower($code);

                if (strpos($filename, $code) !== false) {
                    return false;
                }
            }

        }

        $date = get_the_date('Y-m-d H:i:s', $attachment->post_id);


        // cannot get the date (get rid of the file anyway here)
        if (!$date) {
            return true;
        }

        $timestamp = strtotime($date);

        $min_age = strtotime('-'.str_replace('_', '', $interval));

        // The file is deleteable if the timestamp of the file is older than the minimum date which comes from the interval
        return $min_age > $timestamp;
    }

    private function cleanup_metadata($id_set = [])
    {

        global $wpdb;

        if (!$id_set) {
            return false;
        }

        $query = sprintf("DELETE FROM {$wpdb->postmeta} WHERE post_id IN (%s)", implode(',', $id_set));

        Helper::log(sprintf('[ cleanup_metadata ] Running Query %s ...', $query), 'info');

        return $wpdb->query($query);
    }

    public function get_image_from_url($url)
    {

        global $wpdb;

        $url = pathinfo($url, PATHINFO_FILENAME);
        $url = explode('.', $url);
        $url = reset($url);

        if (!$url) {
            return 0;
        }

        $attachment = $wpdb->get_col(
            $wpdb->prepare("SELECT ID FROM {$wpdb->posts} WHERE post_title = %s LIMIT 1;", $url)
        );

        if (!$attachment) {
            return 0;
        }

        return $attachment[0];
    }

    public function clean_imported_attachments()
    {
        global $wpdb;

        $query = sprintf(
            "DELETE FROM {$wpdb->postmeta} 
								  WHERE meta_key = 'type' 
								  AND meta_value = 'photobooth-gallery-image' 
								  AND post_id NOT IN (SELECT ID FROM {$wpdb->posts})"
        );

        return $wpdb->query($query);
    }

    public function removeable_images($settings_id = false, $interval = false)
    {

        global $wpdb;

        $result = [
            'found' => 0,
            'error' => 0,
            'skipped' => 0,
            'storage' => 0,
            'moved' => 0,
        ];

        $query = sprintf(
            "SELECT posts.ID as post_id 
								 FROM {$wpdb->posts} as posts RIGHT JOIN {$wpdb->postmeta} as post_meta ON (post_meta.post_id = posts.id) 
								 WHERE post_meta.meta_key = 'type' AND post_meta.meta_value = 'photobooth-gallery-image' 
								 AND posts.post_type = 'attachment'"
        );

        $attachments = $wpdb->get_results($query);

        if (!$attachments) {
            return $result;
        }

        if ($settings_id !== false) {
            $this->match_attachements_with_files($attachments);
        }

        $result['found'] = sizeof($attachments);
        $update_option = 0;
        foreach ($attachments as $attachment) {

            if ($update_option++ % 15) {
                Helper::set_php_environment();
            }

            $file = get_attached_file($attachment->post_id);

            if (!$file) {

                $result['error']++;
                continue;
            }

            // If the file is newer than the minimum date required to be removed
            if (!$this->is_autoremovable_file($attachment, $interval)) {

                $result['skipped']++;
                continue;
            }

            $managed_file = $this->file_management[$attachment->post_id] ?? false;

            // Skip the file if it does not belong to the given setting
            if ($managed_file !== false && $managed_file->c_config_id && $settings_id !== intval(
                    $managed_file->c_config_id
                )) {

                $result['skipped']++;
                continue;
            }

            // Do not remove files, which may not be removed at all
            if ($managed_file !== false && $managed_file->is_deleteable === false) {

                $result['skipped']++;
                continue;
            }

            $target = get_post_meta($attachment->post_id, 'real_filepath', true);

            if (!$target) {
                $result['error']++;
                continue;
            }

            $result['moved']++;
        }

        return $result;
    }

    public function scan_upload_folder($settings_id = false)
    {

        $lock = LockMixin::createLock(Transients::CURRENTLY_SCANNING, 5 * MINUTE_IN_SECONDS);

        if ($lock->acquire() === false) {
            Helper::log('[ Scan Folder ] Currently running, skipping this request...');

            return false;
        }

        $settings = Helper::load_all_settings($settings_id);

        if (!$settings) {

            Helper::log(sprintf('[ Scan Folder ] No settings found, skipping...'));
            $lock->release();

            return false;
        }

        $response = [];

        try {
            foreach ($settings as $name => $setting) {
                $response[$name] = $this->scan_images($setting);
            }
        } finally {
            $lock->release();
        }


        return $response;
    }

    public function scan_images($settings)
    {

        $folder = $settings['media_folder'];

        if (!$folder) {
            return false;
        }

        $folder = Helper::ensureTrailingBackslash($folder, PG_BACKSLASH_PREFIX | PG_BACKSLASH_SUFFIX);

        $folder = WP_CONTENT_DIR.$folder;

        Helper::log(
            sprintf(
                '[ Image Scanner ] Scanning folder <%s> for new media files to be added to wordpress gallery...',
                $folder
            )
        );

        $files = Helper::recursive_search($folder);

        usort(
            $files,
            function ($a, $b) {

                return filemtime($b) <=> filemtime($a);
            }
        );

        Helper::log(
            sprintf('[ Image Scanner ] Found %d potential new files in folder <%s> ...', sizeof($files), $folder)
        );

        $allowed_extensions = [];

        foreach ($settings['allowed_extensions'] as $extension) {
            $allowed_extensions = array_merge($allowed_extensions, explode(',', $extension));
        }

        $result = [
            'total' => sizeof($files),
            'found' => 0,
            'new' => 0,
            'duplicate' => 0,
            'error' => 0,
            'skipped' => 0,
        ];

        $hard_limit = 250;
        foreach ($files as $file) {

            if ($result['new'] >= $hard_limit) {
                Helper::log(
                    sprintf(
                        '[ Image Scanner ] Stopping this import after adding %d Images in one Session to prevent memory leaks...',
                        $hard_limit
                    ),
                    'info'
                );
                break;
            }

            $suffix = strtolower(pathinfo($file, PATHINFO_EXTENSION));

            // we skip php files
            if ($suffix === 'php') {

                $result['skipped']++;
                continue;
            }

            // Skip zip files also
            if ($suffix === 'zip') {


                $result['skipped']++;
                continue;
            }


            if (!in_array($suffix, $allowed_extensions)) {

                $result['skipped']++;
                Helper::log(
                    sprintf(
                        '[ Image Scanner ] Found File <%s> with suffix <%s> which is not allowed by config!',
                        $file,
                        $suffix
                    ),
                    'warning'
                );

                continue;
            }

            $result['found']++;

            $hashed_filename = sha1($file).'.'.$suffix;

            if ($this->file_exists($hashed_filename)) {

                $result['duplicate']++;

                continue;
            }

            if (!$this->verify_image($file)) {

                $result['skipped']++;

                Helper::log(
                    sprintf(
                        '[ Image Scanner ] Identified File <%s> with suffix <%s> as incomplete/defective, skipping...',
                        $file,
                        $suffix
                    ),
                    'warning'
                );

                continue;
            }

            $meta_data = [
                'real_filename' => basename($file),
                'real_filepath' => $file,
                'file_date' => filemtime($file),
                '_hash' => $hashed_filename,
                'type' => 'photobooth-gallery-image',
                'pb-extension' => strtolower(pathinfo($file, PATHINFO_EXTENSION)),
            ];

            // Allow plugins to change the import behaviour
            if (apply_filters('photobooth/scan_images/skip_import', false, $suffix, $file, $meta_data, $settings)) {

                $result['skipped']++;

                continue;
            }

            $attachment_id = Helper::upload($file, $hashed_filename, $meta_data);

            if (!$attachment_id) {
                $result['error']++;
                continue;
            }

            $this->insert_row([
                'c_hash' => $hashed_filename,
                'c_reference' => $attachment_id,
                'c_filename' => basename($file),
                'c_source' => $file,
                'c_config' => $settings['settings-name'],
                'c_config_id' => $settings['settings-post-id'],
            ],
                ['%s', '%d', '%s', '%s', '%s', '%d']);

            if ($settings['unlink']) {

                unlink($file);
            }

            Helper::log(
                sprintf(
                    '[ Image Scanner ] Successfully added new media file <%s> with hashed name <%s> to wordpress gallery ',
                    $file,
                    $hashed_filename
                )
            );
            $result['new']++;

            $config = [
                'config' => $settings['settings-name'],
                'config_id' => $settings['settings-post-id'],
            ];

            set_transient(Transients::SCAN_STATUS, array_merge($config, $result), 2 * MINUTE_IN_SECONDS);
        }

        set_transient(sprintf('pg-last-scan-for-%d', $settings['settings-post-id']), time());

        delete_transient(Transients::SCAN_STATUS);

        return $result;
    }

    private function file_exists($hash)
    {

        if ($this->file_is_known($hash)) {
            return true;
        }

        return $this->check_for_migration($hash);
    }

    public function file_is_known($hash)
    {

        global $wpdb;

        $table = $this->get_table();

        $query = sprintf("SELECT * FROM {$table} WHERE c_hash = '%s'", $hash);

        $results = $wpdb->get_results($query);

        if (sizeof($results) > 0) {
            return reset($results);
        }

        return false;
    }

    private function check_for_migration($hash)
    {

        global $wpdb;

        $query = sprintf(
            "SELECT posts.ID as post_id 
								 FROM {$wpdb->posts} as posts RIGHT JOIN {$wpdb->postmeta} as post_meta ON (post_meta.post_id = posts.id) 
								 WHERE post_meta.meta_key = '_hash' AND post_meta.meta_value = '%s' 
								 AND posts.post_type = 'attachment' LIMIT 1",
            $hash
        );

        $result = $wpdb->get_results($query);

        if (!$result) {
            return false;
        }

        $result = reset($result);

        $attachment_id = $result->post_id;

        $file = get_post_meta($attachment_id, 'real_filepath', true);

        if (!$file) {

            Helper::log(
                sprintf('[ File Management ] Could not retrieve real file path from attachment %d', $attachment_id),
                'error'
            );

            return true;
        }

        $this->insert_row([
            'c_hash' => $hash,
            'c_reference' => $attachment_id,
            'c_filename' => basename($file),
            'c_source' => $file,
            'c_config' => 'N/A',
            'c_config_id' => 0,
        ],
            ['%s', '%d', '%s', '%s', '%s', '%d']);

        Helper::log(
            sprintf(
                '[ File Management Migration ] Migrating previously added File %s to new File Storage Validation...',
                $file
            ),
            'error'
        );

        return true;
    }

    private function insert_row($data, $definition)
    {

        global $wpdb;

        $table_name = $this->get_table();

        $result = $wpdb->insert($table_name, $data, $definition);

        if (!$result) {
            Helper::log(
                sprintf('[ File Management ] Could not INSERT %s into %s', json_encode($data), $table_name),
                'error'
            );

            return false;
        }

        return $wpdb->insert_id;
    }

    private function verify_image($file)
    {

        $mime_type = mime_content_type($file);

        switch ($mime_type) {
            case "image/jpg":
            case "image/jpeg":
                $img = imagecreatefromjpeg($file);
                break;
            case "image/png":
                $img = imagecreatefrompng($file);
                break;
            case "image/gif":
                $img = imagecreatefromgif($file);
                break;
            default:
                return true;
        }

        if (!$img) {
            return false;
        }

        if (strpos($mime_type, 'jp') === false) {
            return true;
        }

        $jpgdata = fopen($file, 'r'); // 'r' is for reading
        fseek($jpgdata, -2, SEEK_END); // move to EOF -2
        $eofdata = fread($jpgdata, 2);
        fclose($jpgdata);

        return $eofdata === "\xFF\xD9";
    }

    public function get_zip_archive($args)
    {

        /**
         * @var $manager SocialMedia
         * @var $query \WP_Query
         */
        list($query, $the_code, $settings, $manager, $uuid) = array_values($args);

        $response = [
            'error' => true,
            'message' => __('There was an error while trying to generate the zip archive.', 'photobooth-gallery'),
        ];

        $zip = new \ZipArchive();

        $hash = $this->get_zip_file_hash($query);

        $filename = $hash.'.zip';

        $zip_archives = get_option(PG_NAMESPACE.'zip-downloads');

        if (!$zip_archives) {
            $zip_archives = [];
        }

        $posts = $query->get_posts();
        $file_count = sizeof($posts);
        $zip_data = [];

        if (array_key_exists($hash, $zip_archives)) {

            $zip_data = $zip_archives[$hash]['zip_data'];
            $filename = $zip_archives[$hash]['human_readable_filename'];
        }

        if (!$zip_data) {

            $url = site_url('', 'http');
            $human_readable_filename = sanitize_title(
                    sprintf('%s-%s-%s-%d', $url, $hash, $the_code, $file_count)
                ).'.zip';
            $human_readable_filename = str_replace('http-', '', $human_readable_filename);

            $zip_archives[$hash] = [
                'human_readable_filename' => $human_readable_filename,
                'archive_size' => $file_count,
            ];

            foreach ($posts as $post) {
                $zip_data[] = [
                    'id' => $post->ID,
                    'filename' => get_post_meta($post->ID, 'real_filename', true),
                    'mime' => get_post_mime_type($post->ID),
                ];
            }

            $zip_archives[$hash]['zip_data'] = $zip_data;

            $filename = $human_readable_filename;

            update_option(PG_NAMESPACE.'zip-downloads', $zip_archives);
        }

        foreach ($zip_data as $additional_data) {

            $action = 'download';
            $code = $the_code;
            $config = $settings['settings-name'];

            $manager->log(compact('action', 'code', 'config', 'additional_data'));
        }


        $folder_url = Helper::ensureTrailingBackslash(PG_CACHE_URL);
        $folder = Helper::ensureTrailingBackslash(PG_CACHE_PATH);

        if (file_exists($folder.$filename)) {
            Helper::log(sprintf('[ Generate Zip Archive ] Already generated archive file <%s> found ...', $filename));

            $response['error'] = false;
            $response['message'] = __('Your Zip Archive is now ready for download!', 'photobooth-gallery');
            $response['redirect'] = $folder_url.$filename;

            return $response;
        }

        if ($zip->open($folder.$filename, \ZipArchive::CREATE | \ZipArchive::OVERWRITE) !== true) {
            Helper::log(
                sprintf('[ Generate Zip Archive ] Could not generate Zip File <%s>...', $folder.$filename),
                'error'
            );

            return $response;
        }

        $i = 0;
        if ($uuid) {
            set_transient($uuid.'-file-index', $i, 60 * MINUTE_IN_SECONDS);
        }
        foreach ($posts as $post) {

            Helper::set_php_environment();

            $attachment_file = get_attached_file($post->ID);
            $ext = pathinfo($attachment_file, PATHINFO_EXTENSION);
            $file_date = get_post_meta($post->ID, 'file_date', true);

            if (!$file_date) {
                $file_date = time();
            }

            $file_date = date('Y-m-d-H-i-s', $file_date);

            $attachment_name = sprintf('%s-%s-%03d.%s', sanitize_title($the_code), $file_date, ++$i, $ext);

            $zip->addFile($attachment_file, $attachment_name);
            $zip->setCompressionName($attachment_name, \ZipArchive::CM_STORE);

            if (($i % 10) === 0) {
                Helper::log(
                    sprintf('[ Generate Zip Archive ] Added %d/%d files to Zip File ...', $i, $file_count),
                    'info'
                );
            }

            if ($uuid) {
                set_transient($uuid.'-file-index', $i, 60 * MINUTE_IN_SECONDS);
            }

        }
        Helper::log(sprintf('[ Generate Zip Archive ] Generating Zip file on disk, please stand by ...'), 'info');

        $zip->close();

        $response['error'] = false;
        $response['redirect'] = $folder_url.$filename;
        $response['message'] = __('Your Zip Archive is now ready for download!', 'photobooth-gallery');

        Helper::log(
            sprintf('[ Generate Zip Archive ] Generated new Zip Archive <%s> for Code <%s> ...', $filename, $the_code)
        );

        return $response;
    }

    private function get_zip_file_hash(\WP_Query $query)
    {

        $hashes = [];
        foreach ($query->get_posts() as $post) {
            $attachment_file = get_attached_file($post->ID);
            $hashes[] = sha1_file($attachment_file);
        }

        return sha1(json_encode($hashes));
    }

    public function transferred_files_lifetime($settings_id)
    {

        global $wpdb;

        $table = $this->get_table();

        $query = sprintf("SELECT COUNT(id) as amount FROM {$table} WHERE c_config_id = %d", $settings_id);

        $results = $wpdb->get_results($query);

        return intval(reset($results)->amount);
    }

    public function verified_files_id_set($setting)
    {

        if (!is_array($setting)) {
            return [];
        }

        global $wpdb;

        $table_name = $this->get_table();

        $query = sprintf(
            "SELECT c_reference AS id FROM {$table_name} WHERE c_config_id = %d",
            $setting['settings-post-id']
        );

        $results = $wpdb->get_results($query, ARRAY_N);

        if (!is_array($results)) {
            return [];
        }

        $results = array_reduce(
            $results,
            function ($carry, $item) {

                $carry[] = $item->id;

                return $carry;

            },
            []
        );

        return $results;
    }

    public function handle_file_download($code, $file)
    {

        $ext = pathinfo($file, PATHINFO_EXTENSION);

        $is_download = $_GET['download'] ?? false;

        if (!$is_download) {
            return $this->handle_file_view($code, $file);
        }

        $counter = mt_rand(150, 999999);

        //obfuscate the code for download
        $code = substr($counter, 2);

        $file_name = sprintf('photobooth-%s-%d.%s', $code, $counter, $ext);

        header('Content-Description: File Transfer');
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename="'.$file_name.'"');
        header('Content-Transfer-Encoding: binary');
        header('Expires: 0');
        header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
        header('Pragma: public');
        header('Content-Length: '.filesize($file));

        if (ob_get_contents()) {
            ob_clean();
            flush();
        }

        readfile($file);

        exit();
    }

    public function handle_file_view($code, $file)
    {

        $ext = pathinfo($file, PATHINFO_EXTENSION);

        $counter = mt_rand(150, 999999);

        $file_name = sprintf('photobooth-%s-%d.%s', $code, $counter, $ext);

        $parts = [
            site_url(),
            'pg-download',
            get_query_var('pg-download'),
        ];

        $mime = mime_content_type($file);

        $stream = implode('/', $parts).'?download=true';

        if (strpos($mime, 'video') !== false) {
            $stream = Helper::abs_path_to_url($file);
        }

        $data = [
            'media' => [
                'mime' => $mime,
                'uuid' => uniqid(),
                'filename' => $file_name,
                'src' => implode('/', $parts).'?download=true',
                'stream' => $stream,
                'title' => sprintf(__('Photobooth Download for Code %s', 'photobooth-gallery'), $code),
            ],
            'css' => [
                'src' => plugin_dir_url(PG_BOOTSTRAP_FILE).'public/css/microsite.min.css',
            ],
        ];

        return $data;
    }

    public function stream_video($file)
    {

        $stream_size = filesize($file);
        header('Content-Description: File Transfer');
        header('Content-type: video/mp4');
        header("Content-length: ".filesize($file));
        header("Expires: 0");
        header("Content-Transfer-Encoding: binary");
        header("Accept-Ranges: 0-$stream_size");
        $file = fopen($file, 'r');
        echo stream_get_contents($file);
        fclose($file);
    }

    private function async_scan()
    {

        $plugin = Helper::get_plugin_settings();

        $requests = function ($total) {

            $uri = site_url('pg-async-scan-request');

            for ($i = 0; $i < $total; $i++) {
                yield new Request('GET', $uri);
            }
        };

        $client = new Client();

        $pool = new Pool($client, $requests(100), [
            'concurrency' => $plugin['cpu_usage'],
            'fulfilled' => function ($response, $index) {
                // this is delivered each successful response
            },
            'rejected' => function ($reason, $index) {
                // this is delivered each failed request
            },
        ]);

        $promise = $pool->promise();

        $promise->wait();
    }
}
