<?php
/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 30.07.2018
 * Time: 14:09
 */

namespace Photobooth;

use Exception;
use Photobooth\Defuse\Crypto\Crypto;
use Photobooth\Defuse\Crypto\Key;
use Photobooth\Monolog\Formatter\LineFormatter;
use Photobooth\Monolog\Handler\RotatingFileHandler;
use Photobooth\Monolog\Handler\StreamHandler;
use Photobooth\Monolog\Level;
use Photobooth\Monolog\Logger;
use Photobooth\Monolog\Processor\IntrospectionProcessor;
use Photobooth\Monolog\Processor\PsrLogMessageProcessor;
use Photobooth\Monolog\Processor\UidProcessor;
use Photobooth_Gallery;

define('PG_BACKSLASH_SUFFIX', 1);

define('PG_BACKSLASH_PREFIX', 2);

class Helper
{

    // Singleton Loggers
    private static array $logger = [];

    public static function set_php_environment()
    {

        $current_limit = intval(ini_get('max_execution_time'));

        if ($current_limit === 0) {
            return true;
        }

        if ($current_limit >= 300) {
            return true;
        }

        // We will suppress warnings here, because otherwise we will generate potential emails with things we cannot change anyways
        @set_time_limit(300);

        $current_limit = ini_get('max_execution_time');

        if ($current_limit !== '300') {
            self::log(
                sprintf(
                    '[ PHP MaxExecutionTime ] Error, could not set timeout to 300, current value %s ',
                    $current_limit
                ),
                'error'
            );

            return false;
        }

        return true;
    }

    public static function log($msg, $type = 'info', $text_prefix = false)
    {

        if (is_array($msg)) {
            $msg = json_encode($msg, JSON_PRETTY_PRINT);
        }

        if ($text_prefix) {
            $msg = sprintf('[ %s ] %s', $text_prefix, $msg);
        }

        if (array_key_exists('photobooth-gallery', self::$logger)) {

            $logger = self::$logger['photobooth-gallery'];

            $logger->$type($msg);

            return;
        }

        $prefix = get_field('pg_logfile_prefix', 'option');

        if (!$prefix) {
            $prefix = '';
        }

        $logger = self::getLogger('photobooth-gallery', false, false, true, $prefix);

        $logger->$type($msg);
    }

    public static function getLogger(
        $name = 'default',
        $base_dir = false,
        $stdout = false,
        $rotate = true,
        $prefix = ''
    ) {

        $logger = new Logger($name);
        $logger->setTimezone(new \DateTimeZone('Europe/Berlin'));

        $base_dir = self::ensureTrailingBackslash(self::baseLogDir($base_dir));

        $stream = new StreamHandler($base_dir."log/{$prefix}{$name}.log", Level::Debug, true);

        if ($rotate) {
            $stream = new RotatingFileHandler(
                $base_dir."log/{$prefix}{$name}.log",
                5,
                Level::Debug,
                true
            );
        }

        $stream->setFormatter(
            new LineFormatter(
                "%datetime% - %level_name% - %message% [ %extra.request% ]\n",
                "Y-m-d H:i:s"
            )
        );
        $stream->pushProcessor(new IntrospectionProcessor('debug'));

        $stream->pushProcessor(new UidProcessor());
        $stream->pushProcessor(new PsrLogMessageProcessor());
        $stream->pushProcessor(function ($record) {

            $record['extra']['request'] = $_REQUEST;

            return $record;
        });

        $logger->pushHandler($stream);
        if ($stdout) {
            $sysout = new StreamHandler('php://stdout', Level::Debug);
            $sysout->setFormatter(
                new LineFormatter(
                    "%datetime% - %level_name% - %message%\n", "Y-m-d H:i:s"
                )
            );
            $sysout->pushProcessor(new IntrospectionProcessor('debug'));
            $sysout->pushProcessor(new UidProcessor());
            $sysout->pushProcessor(new PsrLogMessageProcessor());
            $logger->pushHandler($sysout);
        }


        self::$logger[$name] = $logger;

        return $logger;
    }

    public static function ensureTrailingBackslash($path, $flags = PG_BACKSLASH_SUFFIX, $dir_separator = '/')
    {

        if ($flags & PG_BACKSLASH_SUFFIX) {
            $path = rtrim($path, $dir_separator).$dir_separator;
        }

        if ($flags & PG_BACKSLASH_PREFIX) {
            $path = $dir_separator.ltrim($path, $dir_separator);
        }

        return $path;
    }

    public static function baseLogDir($base_dir = false)
    {

        if ($base_dir) {
            return self::ensureTrailingBackslash($base_dir);
        }

        if (defined('LOG_DIR')) {
            $base_dir = self::ensureTrailingBackslash(LOG_DIR);
        }

        if (!$base_dir) {
            $base_dir = self::ensureTrailingBackslash(PG_BASE_DIR);
        }

        return $base_dir;
    }

    public static function u_log($msg, $type = 'info', $text_prefix = false)
    {

        if (is_array($msg)) {
            $msg = json_encode($msg, JSON_PRETTY_PRINT);
        }

        if ($text_prefix) {
            $msg = sprintf('[ %s ] %s', $text_prefix, $msg);
        }

        if (array_key_exists('user-log', self::$logger)) {

            $logger = self::$logger['user-log'];

            $logger->$type($msg);

            return;
        }

        $prefix = get_field('pg_logfile_prefix', 'option');

        if (!$prefix) {
            $prefix = '';
        }

        $logger = self::getLogger('user-log', PG_BASE_DIR, false, false, $prefix);

        $logger->$type($msg);
    }

    public static function abs_path_to_url($path = '')
    {

        $url = str_replace(wp_normalize_path(untrailingslashit(ABSPATH)), site_url(), wp_normalize_path($path));

        return esc_url_raw($url);
    }

    public static function filesize($file, $is_file = true)
    {

        if ($is_file) {
            $size = filesize($file);
        } else {
            $size = $file;
        }

        $units = array('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB');
        $power = $size > 0 ? floor(log($size, 1024)) : 0;

        return number_format($size / pow(1024, $power), 2, '.', ',').' '.$units[$power];
    }

    public static function removeTrailingBackslash($path, $dir_separator = '/')
    {

        $path = rtrim($path, $dir_separator);

        return $path;
    }


    public static function sanitize($key, $array, $default = '')
    {

        if (!array_key_exists($key, $array)) {
            return $default;
        }

        return $array[$key];
    }

    public static function is_plugin_screen($plugin_name)
    {

        $current_screen = get_current_screen();

        // We cannot extract the current screen thus no decision can be made
        if (!$current_screen) {
            return false;
        }

        // only if we are within our own plugin screen, the plugin name is contained within the id of the screen
        if (strpos($current_screen->id, $plugin_name) !== false) {
            return true;
        }

        if (strpos($current_screen->id, 'acf-options-global-settings') !== false) {
            return true;
        }

        if (strpos($current_screen->id, 'pg_settings') !== false) {
            return true;
        }

        if (strpos($current_screen->id, 'acf-options-pg-statistics') !== false) {
            return true;
        }

        return false;

    }

    public static function load_setting($title = false)
    {

        global $wpdb;

        $conditions = '';

        if (is_string($title) && strlen($title)) {
            $title = esc_sql($title);
            $conditions = "AND posts.post_title = '{$title}'";
        }

        $settings_slug = CustomPostTypes::SETTINGS_SLUG;
        $query = "SELECT posts.ID AS post_id, posts.post_title AS post_title FROM {$wpdb->posts} AS posts WHERE posts.post_type = '{$settings_slug}' AND posts.post_status = 'publish' {$conditions} ORDER BY post_id DESC LIMIT 1";

        $results = $wpdb->get_results($query);

        if (!$results) {
            Helper::log(
                sprintf('[ Settings Loader ] Could not get Settings from database for boothbox <%s>', $title),
                'error'
            );

            return false;
        }

        $post = reset($results);
        $post_id = $post->post_id;

        $global_options = self::get_plugin_settings();

        $local_settings = get_fields($post_id);

        if (!$local_settings) {
            $local_settings = [];
        }

        $base_settings = [
            'settings-name' => $post->post_title,
            'settings-post-id' => $post_id,
        ];

        $config_defaults = Photobooth_Gallery::get_config_defaults();

        $settings = array_merge($global_options, $base_settings, $config_defaults, $local_settings);

        return $settings;
    }

    public static function get_plugin_settings()
    {

        $defaults = Photobooth_Gallery::get_defaults();

        $global_options = get_fields('option');

        if (!is_array($global_options)) {
            $global_options = [];
        }

        $global_options = array_filter(
            $global_options,
            function ($key) {

                return strpos($key, Photobooth_Gallery::PREFIX) === 0;

            },
            ARRAY_FILTER_USE_KEY
        );

        // Remove the prefix from options
        $options = [];
        $_options = array_merge($defaults, $global_options);
        foreach ($_options as $key => $value) {

            $key = str_replace(Photobooth_Gallery::PREFIX, '', $key);

            $options[$key] = $value;
        }

        $options['current_version'] = PHOTOBOOTH_GALLERY_VERSION;

        return $options;
    }

    public static function load_all_settings($settings_id = false)
    {

        global $wpdb;
        $settings_slug = CustomPostTypes::SETTINGS_SLUG;

        $query = "SELECT posts.ID AS post_id, posts.post_title AS post_title FROM {$wpdb->posts} AS posts WHERE posts.post_type = '{$settings_slug}' AND posts.post_status = 'publish'";

        if (intval($settings_id) > 0) {
            $query .= ' AND posts.ID = '.esc_sql($settings_id);
        }

        if (is_string($settings_id)) {
            $query .= ' AND posts.post_title = "'.esc_sql($settings_id).'"';
        }


        $results = $wpdb->get_results($query);

        if (!$results) {
            return false;
        }

        $global_options = self::get_plugin_settings();

        $base_config_data = Photobooth_Gallery::get_config_defaults();

        $settings = [];
        foreach ($results as $post) {
            $field_data = get_fields($post->post_id);

            if (!is_array($field_data)) {
                $field_data = [];
            }

            $field_data['settings-name'] = $post->post_title;
            $field_data['settings-post-id'] = $post->post_id;

            $settings[sanitize_title($post->post_title)] = array_merge($global_options, $base_config_data, $field_data);
        }

        return $settings;
    }

    public static function remote_upload($url, $meta_data = [])
    {

        $get = wp_remote_get($url);

        if ($get instanceof \WP_Error) {

            Helper::log(
                sprintf(
                    '[ Image Uploader ] Could not upload remote url <%s> to media folder ( %s )',
                    $url,
                    $get->get_error_message()
                ),
                'error'
            );

            return false;
        }

        $file_name = basename($url);

        $upload_file = wp_upload_bits($file_name, null, wp_remote_retrieve_body($get));

        if ($upload_file['error']) {

            Helper::log(
                sprintf(
                    '[ Image Uploader ] Could not upload File to media folder <%s>: %s',
                    $file_name,
                    $upload_file['error']
                ),
                'error'
            );

            return false;
        }

        $wp_filetype = wp_check_filetype($file_name, null);

        $attachment = array(
            'post_mime_type' => $wp_filetype['type'],
            'post_title' => preg_replace('/\.[^.]+$/', '', $file_name),
            'post_content' => '',
            'post_status' => 'inherit',
        );

        $attachment_id = wp_insert_attachment($attachment, $upload_file['file']);

        if (is_wp_error($attachment_id)) {
            Helper::log(
                sprintf(
                    '[ Image Uploader ] Could not upload File to media folder <%s>: %s',
                    $file_name,
                    json_encode($attachment_id->get_error_messages())
                ),
                'error'
            );

            return false;
        }

        foreach ($meta_data as $key => $value) {
            update_post_meta($attachment_id, $key, $value);
        }

        require_once(ABSPATH."wp-admin".'/includes/media.php');
        require_once(ABSPATH."wp-admin".'/includes/image.php');

        $attachment_data = wp_generate_attachment_metadata($attachment_id, $upload_file['file']);

        wp_update_attachment_metadata($attachment_id, $attachment_data);

        return $attachment_id;
    }

    public static function upload($file, $file_name = false, $meta_data = [])
    {

        if (!$file_name) {
            $file_name = basename($file);
        }

        $upload_file = wp_upload_bits($file_name, null, file_get_contents($file));

        if ($upload_file['error']) {

            Helper::log(
                sprintf(
                    '[ Image Uploader ] Could not upload File to media folder <%s>: %s',
                    $file_name,
                    $upload_file['error']
                ),
                'error'
            );

            return false;
        }

        $wp_filetype = wp_check_filetype($file_name, null);

        $attachment = array(
            'post_mime_type' => $wp_filetype['type'],
            'post_title' => preg_replace('/\.[^.]+$/', '', $file_name),
            'post_content' => '',
            'post_status' => 'inherit',
        );

        $attachment_id = wp_insert_attachment($attachment, $upload_file['file']);

        if (is_wp_error($attachment_id)) {
            Helper::log(
                sprintf(
                    '[ Image Uploader ] Could not upload File to media folder <%s>: %s',
                    $file_name,
                    json_encode($attachment_id->get_error_messages())
                ),
                'error'
            );

            return false;
        }

        require_once(ABSPATH."wp-admin".'/includes/media.php');
        require_once(ABSPATH."wp-admin".'/includes/image.php');


        $fn = function ($sizes, $meta) use ($attachment_id) {

            $image_sizes = apply_filters(
                'pb/file_management/intermediate_image_sizes_advanced',
                $sizes,
                $meta,
                $attachment_id
            );

            return $image_sizes;
        };

        add_filter('intermediate_image_sizes_advanced', $fn, 10, 2);

        $attachment_data = wp_generate_attachment_metadata($attachment_id, $upload_file['file']);

        remove_filter('intermediate_image_sizes_advanced', $fn);

        wp_update_attachment_metadata($attachment_id, $attachment_data);

        foreach ($meta_data as $key => $value) {
            update_post_meta($attachment_id, $key, $value);
        }

        do_action('photobooth/upload_media_file/after_upload', $file_name, $attachment_id, $attachment);

        return $attachment_id;
    }

    public static function handle_cron($enable, $cron, $interval, $time = false)
    {

        if (!$time) {
            $time = time();
        }

        if (!$enable) {

            if (!wp_next_scheduled($cron)) {
                return;
            }

            wp_clear_scheduled_hook($cron);

            self::log(sprintf('[ Cron Handler ] Disable Cron %s -> %s ', $cron, $interval));

            return;
        }

        $key = 'pg-cron-interval-'.sanitize_title($cron);

        $last_interval = get_option($key);

        if (!$last_interval) {

            $last_interval = $interval;

            update_option($key, $interval);
        }

        if ($last_interval !== $interval) {

            update_option($key, $interval);

            wp_clear_scheduled_hook($cron);

            self::log(
                sprintf('[ Cron Handler ] Resetting Cron %s from <%s> to <%s> ', $cron, $last_interval, $interval)
            );
        }

        if (wp_next_scheduled($cron)) {
            return;
        }

        wp_schedule_event($time, $interval, $cron);

        self::log(
            sprintf('[ Cron Handler ] Enable Cron [ %s ] %s -> %s ', date('Y-m-d H:i:s', $time), $cron, $interval)
        );
    }


    public static function encrypt($text)
    {

        $key = get_option('pg-secret-encryption-key');

        if (!$key) {
            return false;
        }


        try {

            return Crypto::encrypt(
                $text,
                Key::loadFromAsciiSafeString($key)
            );

        } catch (Exception $e) {
            self::log($e, 'error');
        }

        return false;
    }

    public static function decrypt($text)
    {

        $key = get_option('pg-secret-encryption-key');

        if (!$key) {
            return false;
        }

        try {

            return Crypto::decrypt(
                $text,
               Key::loadFromAsciiSafeString($key)
            );

        } catch (Exception $e) {
            self::log($e, 'error');
        }

        return false;

    }

    public static function to_array($object)
    {

        if (!is_object($object)) {
            return $object;
        }

        return json_decode(json_encode($object), true);
    }

    public static function make_list($list)
    {

        array_walk(
            $list,
            function (&$row) {

                $row = sprintf('<li>%s</li>', $row);
            }
        );

        return $list;
    }

    public static function get_client_ip()
    {

        if (PG_DEBUG) {
            return '2001:0db8:85a3:0000:0000:8a2e:0370:7334';
        }

        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }

        return $ip;
    }


    public static function phpinfo_array()
    {

        $entitiesToUtf8 = function ($input) {

            // http://php.net/manual/en/function.html-entity-decode.php#104617
            return preg_replace_callback(
                "/(&#[0-9]+;)/",
                function ($m) {

                    return mb_convert_encoding($m[1], "UTF-8", "HTML-ENTITIES");
                },
                $input
            );
        };
        $plainText = function ($input) use ($entitiesToUtf8) {

            return trim(html_entity_decode($entitiesToUtf8(strip_tags($input))));
        };
        $titlePlainText = function ($input) use ($plainText) {

            return '# '.$plainText($input);
        };

        ob_start();
        phpinfo(-1);

        $phpinfo = array('phpinfo' => array());

        // Strip everything after the <h1>Configuration</h1> tag (other h1's)
        if (!preg_match('#(.*<h1[^>]*>\s*Configuration.*)<h1#s', ob_get_clean(), $matches)) {
            return array();
        }

        $input = $matches[1];
        $matches = array();

        if (preg_match_all(
            '#(?:<h2.*?>(?:<a.*?>)?(.*?)(?:<\/a>)?<\/h2>)|'.'(?:<tr.*?><t[hd].*?>(.*?)\s*</t[hd]>(?:<t[hd].*?>(.*?)\s*</t[hd]>(?:<t[hd].*?>(.*?)\s*</t[hd]>)?)?</tr>)#s',
            $input,
            $matches,
            PREG_SET_ORDER
        )) {
            foreach ($matches as $match) {
                $fn = strpos($match[0], '<th') === false ? $plainText : $titlePlainText;
                if (strlen($match[1])) {
                    $phpinfo[$match[1]] = array();
                } elseif (isset($match[3])) {
                    $keys1 = array_keys($phpinfo);
                    $phpinfo[end($keys1)][$fn($match[2])] = isset($match[4]) ? array(
                        $fn($match[3]),
                        $fn($match[4]),
                    ) : $fn($match[3]);
                } else {
                    $keys1 = array_keys($phpinfo);
                    $phpinfo[end($keys1)][] = $fn($match[2]);
                }

            }
        }

        return $phpinfo;
    }


    public static function get_url($add = [])
    {

        $parts = [
            content_url(),
        ];

        $parts = array_merge($parts, $add);

        return self::ensureTrailingBackslash(implode('/', $parts));
    }

    public static function array_insert(&$array, $position, $insert)
    {

        if (is_int($position)) {

            array_splice($array, $position, 0, $insert);

        } else {
            $pos = array_search($position, array_keys($array));
            $array = array_merge(array_slice($array, 0, $pos), $insert, array_slice($array, $pos));
        }
    }

    public static function recursive_search($folder, $extension = false)
    {

        if (!is_dir($folder)) {

            self::log(sprintf('[ Scanning Folder ] The given folder %s does not exist!', $folder), 'error');

            return [];
        }

        $files = [];
        $recursive_searcher = new \RecursiveDirectoryIterator($folder);


        foreach (new \RecursiveIteratorIterator($recursive_searcher) as $file) {

            if (!$file instanceof \SplFileInfo) {
                continue;
            }

            if ($file->isDir()) {
                continue;
            }

            if ($extension === false) {
                $files[] = $file->getPathname();
                continue;
            }

            $ext = strtolower(pathinfo($file->getPathname(), PATHINFO_EXTENSION));

            if ($ext !== strtolower($extension)) {
                continue;
            }

            $files[] = $file->getPathname();
        }

        return $files;
    }

    public static function wp_enqueue_script($handle, $src = '', $deps = array(), $ver = false, $in_footer = false)
    {

        if (wp_script_is($handle)) {
            return;
        }

        wp_enqueue_script($handle, $src, $deps, $ver, $in_footer);
    }

    public static function set_flag($flag)
    {

        $file = PG_CACHE_PATH.'/'.sha1($flag);
        @touch($file);
    }

    public static function remove_flag($flag)
    {

        if (!self::flag_exists($flag)) {
            return;
        }

        $file = PG_CACHE_PATH.'/'.sha1($flag);

        @unlink($file);
    }

    public static function flag_exists($flag)
    {

        $file = PG_CACHE_PATH.'/'.sha1($flag);

        return file_exists($file);
    }

    public static function num_cpus()
    {

        $numCpus = get_transient('num_cpus');

        if ($numCpus) {
            return $numCpus;
        }

        $numCpus = 1;
        if (is_file('/proc/cpuinfo')) {
            $cpuinfo = file_get_contents('/proc/cpuinfo');
            preg_match_all('/^processor/m', $cpuinfo, $matches);
            $numCpus = count($matches[0]);
        } elseif ('WIN' == strtoupper(substr(PHP_OS, 0, 3))) {
            $process = @popen('wmic cpu get NumberOfCores', 'rb');
            if (false !== $process) {
                fgets($process);
                $numCpus = intval(fgets($process));
                pclose($process);
            }
        } else {
            $process = @popen('sysctl -a', 'rb');
            if (false !== $process) {
                $output = stream_get_contents($process);
                preg_match('/hw.ncpu: (\d+)/', $output, $matches);
                if ($matches) {
                    $numCpus = intval($matches[1][0]);
                }
                pclose($process);
            }
        }

        set_transient('num_cpus', $numCpus, 24 * HOUR_IN_SECONDS);

        return $numCpus;
    }

}

