<?php declare(strict_types=1);

namespace Photobooth\Plugins;

class AdvertisementMixin
{
    public static function i18n(): array {
        return [
            __('Advertising', 'photobooth-gallery'),
            __('Show Ads', 'photobooth-gallery'),
            __('Recognizable Part in Filename', 'photobooth-gallery'),
            __('Enter a unique String that is part of the filename for this event configuration to identify pictures as Advertisement, e.g. _ad_', 'photobooth-gallery'),
            __('Distance between ads', 'photobooth-gallery'),
            __('Amount of User generated content between ads. Defaults to 5', 'photobooth-gallery'),
            __('Amount of Seconds to show each Ad', 'photobooth-gallery'),
        ];
    }
    public function init()
    {
        add_filter(
            'photobooth/scan_images/file_meta_data',
            fn(array $metaData, string $fileName, array $settings) => $this->addIdentifierToUpload(
                $metaData,
                $fileName,
                $settings
            ),
            10,
            3
        );

        add_filter(
            'photobooth/render_code_content/query_args',
            fn(array $queryArgs, array $settings) => $this->modifyQueryArgs($queryArgs, $settings),
            10,
            2
        );

        add_filter(
            'photobooth/render_code_content/query',
            fn(\WP_Query $query, array $settings, string $theCode) => $this->addAdsToQuery($query, $settings, $theCode),
            10,
            3
        );
    }

    /**
     * Marks a given file on upload as Advertisment if Ad Management is enabled
     *
     * @param array  $metaData
     * @param string $fileName
     * @param array  $settings
     *
     * @return array
     */
    private function addIdentifierToUpload(array $metaData, string $fileName, array $settings): array
    {

        if ($settings['is_show_ads'] === false) {
            return $metaData;
        }

        if (strlen($settings['ad_filename_pattern']) === 0) {
            return $metaData;
        }

        $fileName = basename($fileName);

        if (str_contains($fileName, $settings['ad_filename_pattern']) === false) {
            return $metaData;
        }

        $metaData['is_ad'] = true;

        return $metaData;
    }

    /**
     * Remove Ads from Default Query
     *
     * @param array $queryArgs
     * @param array $settings
     *
     * @return array
     */
    private function modifyQueryArgs(array $queryArgs, array $settings): array
    {
        if (array_key_exists('meta_query', $queryArgs) === false) {
            return $queryArgs;
        }

        $queryArgs['meta_query'][] = [
            'relation' => 'or',
            [
                'key'     => 'is_ad',
                'value'   => false,
            ],
            [
                'key' => 'is_ad',
                'compare' => 'NOT EXISTS'
            ]
        ];

        return $queryArgs;
    }

    /**
     * Load all Ads and mix them into the current paged view
     *
     * @param \WP_Query $query
     * @param array     $settings
     * @param string    $theCode
     *
     * @return \WP_Query
     */
    private function addAdsToQuery(\WP_Query $query, array $settings, string $theCode): \WP_Query
    {
        if ($settings['is_show_ads'] === false) {
            return $query;
        }

        $space = intval($settings['ad_distance'] ?? 5);

        $adsToShow = intval(max(1, round($query->found_posts / $space)));

        $idSet = $this->getAvailableAds($settings, $adsToShow, $theCode);

        if (sizeof($idSet) === 0) {
            return $query;
        }

        $query_args = [
            'post_type'   => 'attachment',
            'post_status' => 'any',
            'post__in'    => $idSet,
        ];

        $adQuery = new \WP_Query($query_args);

        if ($adQuery->have_posts() === false) {
            return $query;
        }

        $query->found_posts += $adQuery->found_posts;

        $mixedPosts = $query->posts;
        $offset = $space;
        foreach ($adQuery->posts as $post) {
            array_splice($mixedPosts, $offset, 0, [$post]);
            $offset += $space + 1;
        }

        $query->posts = $mixedPosts;

        return $query;
    }

    private function getAvailableAds(array $settings, int $amount, string $theCode): array
    {

        $cacheKey = $this->generateUniqueSettingsKey($settings);

        $cachedIdSet = get_transient($cacheKey);

        if ($cachedIdSet === false || sizeof($cachedIdSet) === 0) {
            $cachedIdSet = $this->generateAdIdSet($theCode);
        }

        shuffle($cachedIdSet);

        if (sizeof($cachedIdSet) <= $amount) {
            delete_transient($cacheKey);

            return $cachedIdSet;
        }

        $extractedIdSet = array_splice($cachedIdSet, 0, $amount);

        set_transient($cacheKey, $cachedIdSet, 30 * MINUTE_IN_SECONDS);

        return $extractedIdSet;
    }

    private function generateUniqueSettingsKey(array $settings): string
    {

        $base = json_encode($settings);

        $key = 'advertisement-mixin-ads';

        return sha1($base.$key);
    }

    private function generateAdIdSet(string $theCode): array
    {
        $query_args = [
            'post_type'   => 'attachment',
            'post_status' => 'any',
            'meta_query'  => [
                'relation' => 'AND',
                [
                    'ad_query'   => [
                        'key'     => 'is_ad',
                        'value'   => true,
                        'compare' => '=',
                    ],
                    'type_query' => [
                        'key'     => 'type',
                        'value'   => 'photobooth-gallery-image',
                        'compare' => '=',
                    ],
                    'code_query' => [
                        'key'     => 'real_filename',
                        'value'   => $theCode,
                        'compare' => 'LIKE',
                    ],
                ],
            ],
        ];

        $query = new \WP_Query($query_args);

        return array_values(array_map(fn(\WP_Post $post) => $post->ID, $query->posts));
    }
}
