<?php


namespace Photobooth\Plugins;


use Photobooth\Helper;

class UploaderPro {

    const SOURCE = 'https://license.codevision.io/api/auto-update/download/?download_id=5250A28D-F76F-4C07-A367-AB4D914A5C17';
    const CONFIG = 'Settings.ini';
    const FOLDER = 'uploader_pro';

    private $settings_id;

    /**
     * @param mixed $settings_id
     *
     * @return UploaderPro
     */
    public function setSettingsId( $settings_id ) {

        $this->settings_id = $settings_id;

        return $this;
    }


    private function download_source() {

        $response = wp_remote_get( self::SOURCE );

        // Test to write into cache folder

        if ( ! $this->write_test() ) {
            Helper::log( sprintf( 'Cannot write to cache directory %s, downloading source files aborted!', PG_CACHE_PATH ), 'error', 'UploaderPro' );

            return false;
        }

        // Handle download error
        if ( $response instanceof \WP_Error ) {
            Helper::log( sprintf( 'Error %s while downloading UploaderPro from source', $response->get_error_message() ), 'error', 'UploaderPro' );

            return false;
        }

        $zip = wp_remote_retrieve_body( $response );

        $file = PG_CACHE_PATH . '/' . uniqid() . '.zip';

        $fp = fopen( $file, "w" );
        fwrite( $fp, $zip );
        fclose( $fp );

        WP_Filesystem();

        $response = unzip_file( $file, PG_CACHE_PATH );

        @unlink( $file );

        if ( is_wp_error( $response ) ) {
            Helper::log( sprintf( 'Error %s while extracting UploaderPro from source', $response->get_error_message() ), 'error', 'UploaderPro' );

            return false;
        }

        return true;
    }

    public function get_download_link() {

        $cache = $this->get_cached_file();

        if ( $cache !== false ) {
            return $cache;
        }

        // Download from source
        if ( ! $this->download_source() ) {
            return false;
        }

        // Update the ini File with the values from this installation
        if ( ! $this->update_config() ) {
            return false;
        }

        // Now, combine all together into a new zip file and return its path
        $target = $this->pack_source();

        // Cache the result for 24 hours
        if ( $target !== false ) {
            set_transient( $this->settings_id, $target, 24 * HOUR_IN_SECONDS );
        }

        return $target;
    }

    private function get_cached_file() {

        $cached_file = get_transient( $this->settings_id );

        if ( ! $cached_file ) {
            self::remove_cache( $this->settings_id );

            return false;
        }

        if ( ! self::file_exists( $this->settings_id ) ) {
            self::remove_cache( $this->settings_id );

            return false;
        }

        return $cached_file;
    }

    private function pack_source() {

        $zip = new \ZipArchive();

        $zip_file = sanitize_title( get_the_title( $this->settings_id ) ) . '.zip';

        $target = [
            PG_CACHE_PATH,
            $zip_file,
        ];

        $target = implode( '/', $target );

        $source = PG_CACHE_PATH . '/' . self::FOLDER;

        if ( $zip->open( $target, \ZipArchive::CREATE | \ZipArchive::OVERWRITE ) !== true ) {
            Helper::log( sprintf( 'Could not generate Zip File <%s>...', $target ), 'error', 'UploaderPro' );

            return false;
        }

        $files = new \RecursiveIteratorIterator( new \RecursiveDirectoryIterator( $source ), \RecursiveIteratorIterator::LEAVES_ONLY );

        foreach ( $files as $name => $file ) {

            if ( $file->isDir() ) {
                continue;
            }

            // Get real and relative path for current file
            $filePath     = $file->getRealPath();
            $relativePath = substr( $filePath, strlen( $source ) + 1 );

            $zip->addFile( $filePath, $relativePath );
        }

        $zip->close();

        rmdir($source);

        return sprintf( '%s/%s', PG_CACHE_URL, $zip_file );
    }

    private function update_config() {

        $settings_file = [
            PG_CACHE_PATH,
            self::FOLDER,
            self::CONFIG,
        ];

        $settings_file = implode( '/', $settings_file );

        if ( ! file_exists( $settings_file ) ) {
            return false;
        }

        $config = file_get_contents( $settings_file );

        $url = self::get_webhook_url( $this->settings_id );

        if ( $url === false ) {
            return false;
        }

        $page = get_page_by_title(__('PBG', 'photobooth-gallery'));

        $base_url = get_page_link($page->ID);

        $event_url = [
            rtrim($base_url, '/'),
            'pg-code',
            ''
        ];

        $config = preg_replace( '/^URL=.+?$/m', sprintf( 'URL="%s"', home_url( 'pg-upload' ) ), $config, 1 );
        $config = str_replace( 'Password=', sprintf( 'Password="%s"', get_field( 'webhook_upload_password', $this->settings_id ) ), $config );
        $config = str_replace( 'SettingID=', sprintf( 'SettingID="%s"', $this->settings_id ), $config );
        $config = str_replace( '{GALLERY_URL_WITH_CODE_PREFIX}', implode('/', $event_url), $config );

        file_put_contents( $settings_file, $config );

        return true;
    }

    private function write_test() {

        $test_file = PG_CACHE_PATH . '/.test';

        if ( ! @touch( $test_file ) ) {
            return false;
        }

        if ( ! @unlink( $test_file ) ) {
            return false;
        }

        return true;
    }

    public static function get_webhook_url( $settings_id ) {

        $password = get_field( 'webhook_upload_password', $settings_id );

        if ( ! $password ) {
            return false;
        }

        $url = home_url( 'pg-upload/' . $password );

        $query = build_query( [
            'settings-id' => $settings_id,
        ] );

        $url .= '?' . $query;

        return $url;
    }

    public static function file_exists( $settings_id ) {

        $zip_file = sanitize_title( get_the_title( $settings_id ) ) . '.zip';

        $target = [
            PG_CACHE_PATH,
            $zip_file,
        ];

        $target = implode( '/', $target );

        return file_exists( $target );
    }

    public static function remove_cache( $settings_id ) {

        $zip_file = sanitize_title( get_the_title( $settings_id ) ) . '.zip';

        $target = [
            PG_CACHE_PATH,
            $zip_file,
        ];

        $target = implode( '/', $target );

        if ( file_exists( $target ) ) {
            @unlink( $target );
        }

        delete_transient( $settings_id );
    }
}
