<?php

namespace Photobooth\Plugins;

use Photobooth\FFMpeg\Coordinate\TimeCode;
use Photobooth\FFMpeg\FFMpeg;
use Photobooth\Helper;
use WP_Error;

class VideoImageMixin
{

    private const SOURCE = 'https://cloud.nextgen2rism.de/index.php/s/gr7wp3ceDrfb5o7/download?path=%%2Fbin&files=%s.zip';

    public function init()
    {
        add_action('photobooth/upload_media_file/after_upload', [$this, 'handle_file_upload'], 10, 3);
        add_action('delete_attachment', [$this, 'handle_file_removal'], 10, 1);
        add_filter('photobooth/render_code_content/twig_data', [$this, 'filter_image_render_data'], 10, 1);
        add_action('photobooth/init/admin', [$this, 'on_init'], 10, 0);
        add_action('photobooth/disable', [$this, 'on_disable'], 10, 0);
    }

    function filter_image_render_data($twig_data)
    {

        $thumbnail_id = get_post_meta($twig_data['id'], '_video_thumbnail_id', true);

        if (!$thumbnail_id) {
            return $twig_data;
        }

        $preview = wp_get_attachment_image_src($thumbnail_id, 'pg-preview');

        if (!$preview) {
            return $twig_data;
        }

        $twig_data['preview']     = reset($preview);
        $twig_data['has_preview'] = true;

        return $twig_data;
    }

    function handle_file_removal($post_id)
    {

        $thumbnail_id = get_post_meta($post_id, '_video_thumbnail_id', true);

        if (!$thumbnail_id) {
            return;
        }

        wp_delete_attachment($thumbnail_id, true);
    }

    function handle_file_upload($file_name, $attachment_id, $attachment)
    {

        if (strpos($attachment['post_mime_type'], 'video') === false) {
            return;
        }

        $previewImage = $this->extract_image_from_video($attachment_id);

        $preview_id = Helper::upload($previewImage, false, ['_reference_id' => $attachment_id]);

        update_post_meta($attachment_id, '_thumbnail_id', $preview_id);
        update_post_meta($attachment_id, '_video_thumbnail_id', $preview_id);

        @unlink($previewImage);
    }

    private function extract_image_from_video($attachment_id)
    {

        $sec = rand(1, 2);

        $attached_file = get_attached_file($attachment_id);

        if (!$architecture = get_transient(VideoImageMixin::class.'-architecture')) {
            $architecture = $this->get_architecture();
            set_transient(VideoImageMixin::class.'-architecture', $architecture, 7 * 24 * 60);
        }

        $ffmpeg_binary  = apply_filters(
            'photobooth/video_image_extractor/binary/ffmpeg',
            sprintf('%s/%s/ffmpeg', PG_BASE_DIR.'/bin', $architecture)
        );
        $ffprobe_binary = apply_filters(
            'photobooth/video_image_extractor/binary/ffprobe',
            sprintf('%s/%s/ffprobe', PG_BASE_DIR.'/bin', $architecture)
        );

        @chmod($ffmpeg_binary, 0755);
        @chmod($ffprobe_binary, 0755);

        $ffmpeg = FFMpeg::create([
            'ffmpeg.binaries'  => $ffmpeg_binary,
            'ffprobe.binaries' => $ffprobe_binary,
        ]);

        $video = $ffmpeg->open($attached_file);

        $frame = $video->frame(TimeCode::fromSeconds($sec));

        $preview = sprintf('%s/%s.jpg', PG_CACHE_PATH, sha1(wp_generate_uuid4()));

        $frame->save($preview);

        return $preview;
    }

    private function get_architecture()
    {

        $architecture = php_uname('m');

        $mappings = [
            'arm64'  => ['arm64', 'aarch64'],
            'x86_64' => ['x86_64'],
        ];

        foreach ($mappings as $folder => $mapping) {
            if (array_key_exists($architecture, $mapping)) {
                return $folder;
            }
        }

        return 'x86_64';
    }

    public function on_init()
    {

        $check = get_transient(VideoImageMixin::class.'-init-check');

        if (PG_DEBUG === false && $check === "1") {
            return;
        }

        set_transient(VideoImageMixin::class.'-init-check', "1", 6 * 60 * 60);

        if (!$architecture = get_transient(VideoImageMixin::class.'-architecture')) {
            $architecture = $this->get_architecture();
            set_transient(VideoImageMixin::class.'-architecture', $architecture, 7 * 24 * 60 * 60);
        }

        $ffprobe_binary = apply_filters(
            'photobooth/video_image_extractor/binary/ffprobe',
            sprintf('%s/%s/ffprobe', PG_BASE_DIR.'/bin', $architecture)
        );

        if (file_exists($ffprobe_binary)) {
            return;
        }

        $remote = sprintf(self::SOURCE, $architecture);

        $response = wp_remote_get($remote, [
            'timeout'     => 30
        ]);

        if ($response instanceof WP_Error) {

            Helper::log(
                sprintf(
                    '[ Video Image Extractor ] Could not download from remote url <%s> to bin folder ( %s )',
                    $remote,
                    $response->get_error_message()
                ),
                'error'
            );

            return;
        }

        $binary_path = dirname($ffprobe_binary);

        $zip = wp_remote_retrieve_body($response);

        $file = PG_CACHE_PATH.'/'.uniqid().'.zip';

        $fp = fopen($file, "w");
        fwrite($fp, $zip);
        fclose($fp);

        require_once(ABSPATH.'wp-admin/includes/file.php');

        WP_Filesystem();

        $response = unzip_file($file, $binary_path);

        @unlink($file);

        if (is_wp_error($response)) {
            Helper::log(
                sprintf('Error %s while extracting ffmpg binaries from source', $response->get_error_message()),
                'error',
                'Video Image Extractor'
            );

            return;
        }

        Helper::log('Successfully extracted ffmpg binaries from source', 'info', 'Video Image Extractor');
    }

    public function on_disable(): void {
        delete_transient(VideoImageMixin::class.'-init-check');
        delete_transient(VideoImageMixin::class.'-architecture');
    }
}
