<?php declare(strict_types=1);

namespace Photobooth\Plugins;

use DateInterval;
use DateTimeImmutable;
use Photobooth\Adbar\Dot;
use Photobooth\Helper;
use Photobooth\ReverseRegex\Generator\Scope;
use Photobooth\ReverseRegex\Lexer;
use Photobooth\ReverseRegex\Parser;
use Photobooth\ReverseRegex\Random\SimpleRandom;
use Photobooth\StringTemplate\Engine;
use Photobooth\Webmozart\Assert\Assert;
use Photobooth\Webmozart\Assert\InvalidArgumentException;

class WhatsappBusinessMixin
{
    private const ENDPOINT = 'https://graph.facebook.com/v20.0/';

    private const MAX_REQUEST_ATTEMPTS = 60;

    private const WHATSAPP_VERSION = '1.0.6';
    private const MESSAGE_VERSION = '1.0.2';
    private const CONSENT_VERSION = '1.0.1';
    private const UPLOAD_VERSION = '1.0.0';

    private const DELAY = 'PT23H55M';


    public static function registerTextDomain(): void {
        add_filter('pbg/acfpolylang/data', fn(array $config) => self::addResetButtonConfig($config));
    }

    public function init(): void
    {
        add_filter('acf/prepare_field/key=field_669fd3a62737d', function (array $field) {
            if ($this->isUploadFeatureEnabled() === false) {
                return [];
            }

            return $field;
        });
        add_filter('acf/prepare_field/key=field_669fd4012737e', function (array $field) {
            if ($this->isUploadFeatureEnabled() === false) {
                return [];
            }

            return $field;
        });

        add_filter(
            'acf/prepare_field/key=field_668ba8bf172c7',
            fn(array $field) => $this->renderWebhook($field)
        );

        add_action('photobooth/public/webhook', fn(string $token) => $this->onRequest($token));
        add_action('pg-cron-wb-webhook-check', $this);
        add_action('pg-cron-wb-upload-check', fn() => $this->customerUploadCron());
        add_action('pg-cron-wb-webhook-clear-data', fn() => $this->cronCleanup());

        add_action('photobooth/disable', function () {
            Helper::handle_cron(false, 'pg-cron-wb-webhook-check', 'every_minute');
            Helper::handle_cron(false, 'pg-cron-wb-webhook-clear-data', 'daily');
            Helper::handle_cron(false, 'pg-cron-wb-upload-check', 'every_minute');
        });

        if (isset($_GET['RESET_WHATSAPP_TABLES'])) {
            add_action('admin_notices', function () { ?>
                <div class="notice notice-success is-dismissible">
                    <h4>Photobooth Whatsapp factory reset</h4>
                    <p>Photobooth whatsapp tables have been reset to factory settings successfully.</p>
                </div>
            <?php });
        }

        $this->onInit();

        add_action('wp', function () {
            Helper::handle_cron(true, 'pg-cron-wb-webhook-check', 'every_minute');
            Helper::handle_cron(true, 'pg-cron-wb-upload-check', 'every_minute');
            Helper::handle_cron(true, 'pg-cron-wb-webhook-clear-data', 'daily');
        });
    }

    private function isUploadFeatureEnabled(): bool
    {
        return get_option(sprintf('%s-enable-customer-upload', self::class)) === 'enabled';
    }

    private function renderWebhook(array $field): array
    {
        $enabled = get_field('is_whatsapp_business_integration');
        $postId = get_the_ID();

        if ($enabled !== true) {
            return $field;
        }

        $savedToken = get_post_meta($postId, '_token_id', true);
        $token = get_field('webhook_whatsapp_business_user_token');

        if (strlen($token) <= 3 && strlen($savedToken) > 0) {
            delete_post_meta($postId, '_token_id');
            delete_option($savedToken);
        }

        if (strlen($token) <= 3) {
            return $field;
        }

        if (strlen($savedToken) > 0) {
            [$nonce,] = explode(':', $savedToken);
            $field['message'] = sprintf('<a href="%s" target="_blank">%1$s</a>', home_url('pg-webhook/'.$nonce));

            return $field;
        }

        $nonce = uniqid('pbg-whatsapp-token-');

        update_option($nonce, implode(':', [$nonce, $postId]));

        update_post_meta($postId, '_token_id', $nonce);

        $field['message'] = sprintf('<a href="%s" target="_blank">%1$s</a>', home_url('pg-webhook/'.$nonce));

        return $field;
    }

    private function onRequest(string $token): void
    {
        $savedToken = get_option($token, null);

        if ($savedToken === null) {
            return;
        }

        if (is_string($savedToken) === false) {
            return;
        }

        [$nonce, $postId] = explode(':', $savedToken);

        if ($nonce !== $token) {
            return;
        }

        if ($_SERVER['REQUEST_METHOD'] === 'GET') {
            $this->handleAuthenticationRequest(intval($postId));
            exit;
        }

        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $this->handleEventRequest(intval($postId));
            exit;
        }
    }

    private function handleAuthenticationRequest(int $postId): void
    {
        $verifyToken = get_field('webhook_whatsapp_business_password', $postId);

        $allowedArgs = [
            'hub_mode',
            'hub_challenge',
            'hub_verify_token',
        ];

        $args = array_filter($_GET, fn(string $key) => in_array($key, $allowedArgs) === true, ARRAY_FILTER_USE_KEY);

        if (sizeof($args) !== 3) {
            return;
        }

        if ($args['hub_verify_token'] !== $verifyToken) {
            return;
        }

        header("HTTP/1.1 200 OK");
        echo $args['hub_challenge'];
        exit;
    }

    private function handleEventRequest(int $postId): void
    {
        header("HTTP/1.1 200 OK");

        global $wpdb;

        $rawData = file_get_contents('php://input');

        $payload = json_decode($rawData, true);

        if (is_array($payload) === false) {
            return;
        }

        if (array_key_exists('object', $payload) === false) {
            return;
        }

        if ($payload['object'] !== 'whatsapp_business_account') {
            return;
        }

        $table_name = self::tableWebhook();

        try {
            $dot = new Dot($payload);

            $filteredPayload = $dot->get('entry.0.changes.0.value.messages.0', []);

            Assert::greaterThan(sizeof($filteredPayload), 0);

            $webhookRequestHash = hash('xxh3', $rawData);

            $eventCode = $this->validatePayload($filteredPayload, $postId);
            $event_url = null;

            if ($eventCode !== null) {
                $page = get_page_by_title(__('PBG', 'photobooth-gallery'));
                $base_url = get_page_link($page->ID);
                $event_url = [
                    rtrim($base_url, '/'),
                    'pg-code',
                    $eventCode,
                ];

                $event_url = implode('/', $event_url);
            }

            $result = $wpdb->insert($table_name, [
                'c_requested_at' => current_time('mysql', true),
                'c_state' => 'CREATED',
                'c_payload' => $rawData,
                'c_type' => $filteredPayload['type'],
                'c_user_id' => $filteredPayload['from'],
                'c_code' => $eventCode,
                'c_setting_id' => $postId,
                'c_gallery_link' => $event_url,
                'c_webhook_hash' => $webhookRequestHash,
                'c_upload_request' => $this->isUploadFeatureRequest($dot, $postId, $eventCode),
            ], ['%s', '%s', '%s', '%s', '%s', '%s', '%d', '%s', '%s', '%d']);

            Assert::eq($result, 1);

            $webhookId = $wpdb->insert_id;
            $webhook = $wpdb->get_row($wpdb->prepare("SELECT * FROM {$table_name} WHERE id = %d", [$webhookId]));
            Assert::notNull($webhook);

            if ($eventCode !== null && $this->isDoSAttempt($webhook) === true) {
                $wpdb->query($wpdb->prepare("DELETE FROM {$table_name} WHERE id=%s", $webhookId));

                return;
            }

            if ($webhook->c_upload_request === '1') {
                $wpdb->update($table_name, [
                    'c_state' => 'COMPLETE',
                    'c_updated_at' => current_time('mysql', true),
                ], ['id' => $webhook->id], ['%s', '%s']);

                $this->insertUploadRequest($webhook);
            }

            $this->insertConsentEntry($filteredPayload);

            if ($event_url === null) {
                return;
            }

            $this->isConsentRequired($webhook);

        } catch (\Exception $exception) {
            Helper::log(
                sprintf(
                    '[ WhatsappBusinessMixin ] Could not handle webhook payload %s Exception thrown: %s',
                    $rawData,
                    $exception->getMessage()
                ),
                'error'
            );
        }
    }

    public static function tableWebhook(): string
    {

        global $wpdb;

        $table_name = $wpdb->prefix.'pg_whatsapp_webhooks';

        return esc_sql($table_name);
    }

    private function validatePayload(array $payload, int $postId): ?string
    {
        $dot = new Dot($payload);

        $message = $dot->get('text.body');

        if ($message === null) {
            return null;
        }

        $message = sanitize_text_field($message);
        $regex = get_field('regex', $postId);
        $regex = '/^'.$regex.'$/';

        $message = preg_replace('/[^A-Za-z0-9 _]/', ' ', $message);

        $tokens = explode(' ', $message);

        $filteredTokens = array_filter($tokens, fn(string $token) => preg_match($regex, $token) === 1);

        if (sizeof($filteredTokens) !== 1) {
            return null;
        }

        $token = reset($filteredTokens);

        if ($token === false) {
            return null;
        }

        return $token;
    }

    private function isUploadFeatureRequest(Dot $dot, int $settingId, ?string $code): bool
    {
        try {
            Assert::notNull($code);

            Assert::true($this->isUploadFeatureEnabled());

            Assert::true(get_field('is_whatsapp_user_media_upload', $settingId));

            $message = get_field('whatsapp_business_user_media_request', $settingId);

            Assert::notNull($message);

            $message = str_replace('{code}', $code, $message);

            Assert::eq($dot->get('entry.0.changes.0.value.messages.0.text.body'), $message);

            return true;
        } catch (InvalidArgumentException $e) {
            return false;
        }
    }

    private function isDoSAttempt($webhook): bool
    {
        global $wpdb;
        $table = self::tableWebhook();

        $time = new DateTimeImmutable('-60min');

        $query = $wpdb->prepare(
            "SELECT COUNT(*) AS c_amount FROM {$table} 
                WHERE c_user_id = %s AND 
                      c_type IN ('text') AND 
                      c_state IN ('PENDING', 'CREATED') AND
                      c_setting_id = %d AND 
                      c_requested_at >= %s",
            [
                $webhook->c_user_id,
                $webhook->c_setting_id,
                $time->format('Y-m-d H:i:s'),
            ]
        );

        $amount = intval($wpdb->get_row($query)->c_amount);

        if ($amount < self::MAX_REQUEST_ATTEMPTS) {
            return false;
        }

        $this->whatsapp($webhook, [
            'messaging_product' => 'whatsapp',
            'recipient_type' => 'individual',
            'to' => $webhook->c_user_id,
            'type' => 'text',
            'text' => [
                'preview_url' => false,
                'body' => __('Too many requests! Please try again in a few minutes.', 'photobooth-gallery'),
            ],
        ]);

        return true;
    }

    private function whatsapp($webhook, array $payload, ?int $postId = null): void
    {
        global $wpdb;
        $data = json_decode($webhook->c_payload, true);

        $dot = new Dot($data);

        $url = sprintf(
            '%s%s/messages',
            self::ENDPOINT,
            $dot->get('entry.0.changes.0.value.metadata.phone_number_id')
        );
        $token = get_field('webhook_whatsapp_business_user_token', $webhook->c_setting_id);
        $table = self::tableMessage();

        try {
            $response = wp_remote_post($url, [
                'body' => $payload,
                'headers' => [
                    'Authorization' => sprintf('Bearer %s', $token),
                    'Content-Type' => 'application/json',
                ],
            ]);

            Assert::false(is_wp_error($response));

            $code = wp_remote_retrieve_response_code($response);

            Assert::eq($code, 200);

            $response = json_decode(wp_remote_retrieve_body($response), true);

            Assert::isArray($response);

            $wpdb->insert($table, [
                'c_sent_at' => current_time('mysql', true),
                'c_state' => 'SUCCESSFUL',
                'c_type' => $payload['type'],
                'c_payload' => json_encode($payload),
                'c_user_id' => $webhook->c_user_id,
                'c_post_id' => $postId,
                'c_code' => $webhook->c_code,
                'c_setting_id' => $webhook->c_setting_id,
            ], ['%s', '%s', '%s', '%s', '%s', '%d', '%s', '%d']);

            Helper::log(
                sprintf(
                    '[WhatsappBusinessMixin API ] Request: %s, Result: %s',
                    json_encode($payload),
                    json_encode($response)
                ),
                'INFO'
            );
        } catch (\Exception $e) {
            $wpdb->insert($table, [
                'c_sent_at' => current_time('mysql', true),
                'c_state' => 'FAILED',
                'c_type' => $payload['type'],
                'c_payload' => json_encode($payload),
                'c_user_id' => $webhook->c_user_id,
                'c_post_id' => $postId,
                'c_code' => $webhook->c_code,
                'c_setting_id' => $webhook->c_setting_id,
                'c_error_message' => $e->getMessage(),
            ], ['%s', '%s', '%s', '%s', '%s', '%d', '%s', '%d', '%s']);

            Helper::log(sprintf('[ WhatsappBusinessMixin API ] Error: %s', $e->getMessage()), 'ERROR');
        }
    }

    public static function tableMessage(): string
    {

        global $wpdb;

        $table_name = $wpdb->prefix.'pg_whatsapp_messages';

        return esc_sql($table_name);
    }

    private function insertUploadRequest($webhook): void
    {
        $table = self::tableUpload();

        global $wpdb;

        $result = $wpdb->insert($table, [
            'c_requested_at' => current_time('mysql', true),
            'c_user_id' => $webhook->c_user_id,
            'c_setting_id' => $webhook->c_setting_id,
            'c_code' => $webhook->c_code,
        ], ['%s', '%s', '%d', '%s']);

        Assert::eq($result, 1);

        $message = __('You can now start to send your memories. \uD83E\uDD17', 'photobooth-gallery');

        $override = get_field('wa_upload_ready_message', $webhook->c_setting_id);

        if ($override) {
            $message = $override;
        }

        $this->whatsapp($webhook, [
            'messaging_product' => 'whatsapp',
            'recipient_type' => 'individual',
            'to' => $webhook->c_user_id,
            'type' => 'text',
            'text' => [
                'preview_url' => false,
                'body' => $message,
            ],
        ]);
    }

    public static function tableUpload(): string
    {

        global $wpdb;

        $table_name = $wpdb->prefix.'pg_whatsapp_uploads';

        return esc_sql($table_name);
    }

    private function insertConsentEntry(array $payload): void
    {
        global $wpdb;

        $table = self::tableConsent();

        $query = $wpdb->prepare(
            "SELECT COUNT(*) as c_available FROM {$table} WHERE c_user_id = %s",
            $payload['from']
        );

        $result = $wpdb->get_row($query);
        $added = intval($result->c_available);

        if ($added >= 1) {
            return;
        }

        $result = $wpdb->insert($table, [
            'c_requested_at' => current_time('mysql', true),
            'c_user_id' => $payload['from'],
        ], ['%s', '%s']);

        Assert::eq($result, 1);
    }

    public static function tableConsent(): string
    {

        global $wpdb;

        $table_name = $wpdb->prefix.'pg_whatsapp_consent';

        return esc_sql($table_name);
    }

    private function isConsentRequired($webhook): bool
    {
        global $wpdb;
        $table = self::tableConsent();

        // Check 1: Consent is not required
        $isConsentRequired = get_field('is_whatsapp_business_gpdr', $webhook->c_setting_id);

        if ($isConsentRequired === false) {
            return false;
        }

        $query = $wpdb->prepare("SELECT * FROM {$table} WHERE c_user_id = %s", $webhook->c_user_id);

        $consent = $wpdb->get_row($query);

        if ($consent === null) {
            return false;
        }

        // Check 2: Consent has been requested and is given -> we are fine to continue
        if ($consent->c_consent_requested !== null && $consent->c_consent_given !== null) {
            return false;
        }

        // Check 3: Consent requested and not given we have to wait until consent is given
        if ($consent->c_consent_requested !== null && $consent->c_consent_given === null) {
            return true;
        }

        $consentMessage = get_field('whatsapp_business_gpdr_text', $webhook->c_setting_id);

        if (!$consentMessage) {
            return false;
        }

        // Check 4: We have to request consent now
        $wpdb->update($table, [
            'c_consent_requested' => current_time('mysql', true),
            'c_updated_at' => current_time('mysql', true),
        ], ['id' => $consent->id], ['%s', '%s']);

        // Send whatsapp message with consent text
        $this->whatsapp($webhook, [
            'messaging_product' => 'whatsapp',
            'recipient_type' => 'individual',
            'to' => $webhook->c_user_id,
            'type' => 'interactive',
            'interactive' => [
                'type' => 'button',
                'header' => [
                    'type' => 'text',
                    'text' => __('Consent is required', 'photobooth-gallery'),
                ],
                'body' => [
                    'text' => $consentMessage,
                ],
                'action' => [
                    'buttons' => [
                        [
                            'type' => 'reply',
                            'reply' => [
                                'id' => 'accept-button',
                                'title' => __('Accept', 'photobooth-gallery'),
                            ],
                        ],
                    ],
                ],
            ],
        ]);

        return true;
    }

    private function customerUploadCron(): void
    {
        $lock = LockMixin::createLock('pg-cron-wb-upload-check', 2 * MINUTE_IN_SECONDS);

        if ($lock->acquire() === false) {
            Helper::log('[ WhatsappBusinessMixin Cron ] Currently running, skipping this request...');

            return;
        }

        global $wpdb;
        $table = self::tableWebhook();

        try {
            Assert::true($this->isUploadFeatureEnabled());

            $query = "SELECT * FROM {$table} WHERE c_state IN ('CREATED','PENDING') AND c_type IN ('image','video', 'document') ORDER BY c_setting_id ASC, c_requested_at DESC LIMIT 5";
            $webhooks = $wpdb->get_results($query);

            if (sizeof($webhooks) === 0) {
                return;
            }

            array_walk($webhooks, fn($webhook) => $this->handleUserUpload($webhook));
        } catch (InvalidArgumentException $ex) {

        } catch (\Exception $e) {
            Helper::log(sprintf('[ WhatsappBusinessMixin Cron ] Error: %s', $e->getMessage()), 'ERROR');
        } finally {
            $lock->release();
        }
    }

    private function handleUserUpload($webhook): void
    {
        $table = self::tableWebhook();
        global $wpdb;

        $wpdb->update($table, [
            'c_state' => 'IN_PROGRESS',
            'c_updated_at' => current_time('mysql', true),
        ], ['id' => $webhook->id], ['%s', '%s']);

        $uploadTable = self::tableUpload();

        $result = $wpdb->get_row(
            $wpdb->prepare(
                "SELECT * FROM {$uploadTable} WHERE c_user_id = %s AND c_setting_id = %s ORDER BY c_requested_at DESC LIMIT 1",
                $webhook->c_user_id,
                $webhook->c_setting_id
            )
        );

        if (!$result) {
            $wpdb->update($table, [
                'c_state' => 'FAILED',
                'c_updated_at' => current_time('mysql', true),
                'c_error_message' => 'User not found in upload table',
            ], ['id' => $webhook->id], ['%s', '%s', '%s']);

            return;
        }

        $webhook->c_code = $result->c_code;

        $masterCode = $this->getDynamicContent($webhook, 'upload', null, $webhook->c_code);

        $data = json_decode($webhook->c_payload, true);
        $dot = new Dot($data);

        try {
            $xpath = sprintf('entry.0.changes.0.value.messages.0.%s.id', $webhook->c_type);
            $id = $dot->get($xpath);
            Assert::notNull($id);

            $token = get_field('webhook_whatsapp_business_user_token', $webhook->c_setting_id);
            $options = [
                'headers' => [
                    'Authorization' => sprintf('Bearer %s', $token),
                ],
            ];

            $request = wp_remote_get(sprintf('%s%s/', self::ENDPOINT, $id), $options);

            Assert::false(is_wp_error($request));

            $code = wp_remote_retrieve_response_code($request);

            Assert::eq($code, 200);

            $data = json_decode(wp_remote_retrieve_body($request), true);

            Assert::isArray($data);

            Assert::keyExists($data, 'url');
            Assert::keyExists($data, 'mime_type');

            $request = wp_remote_get($data['url'], $options);
            Assert::false(is_wp_error($request));
            $code = wp_remote_retrieve_response_code($request);
            Assert::eq($code, 200);
            $imgData = wp_remote_retrieve_body($request);

            $mime = explode('/', $data['mime_type']);

            Assert::inArray($mime[0], ['image', 'video']);

            $extension = $mime[1];

            wp_remote_request(sprintf('%s%s/', self::ENDPOINT, $id), array_merge(['method' => 'DELETE'], $options));

            $lexer = new  Lexer(get_field('regex', $webhook->c_setting_id));
            $gen = new SimpleRandom(mt_rand());
            $parser = new Parser($lexer, new Scope(), new Scope());
            $uniqueCode = null;
            $parser->parse()->getResult()->generate($uniqueCode, $gen);

            // date _ time _ telnumber _ mastercode _ groupcode _ singlecode . ext
            if ($masterCode === null) {
                $filename = sprintf(
                    '%s_%s_%s_%s_%s.%s',
                    wp_date_localised('ymd'),
                    wp_date_localised('His'),
                    trim($webhook->c_user_id),
                    trim($webhook->c_code),
                    $uniqueCode,
                    trim($extension)
                );
            } else {
                $filename = sprintf(
                    '%s_%s_%s_%s_%s_%s.%s',
                    wp_date_localised('ymd'),
                    wp_date_localised('His'),
                    trim($webhook->c_user_id),
                    trim(str_replace(PHP_EOL, '', $masterCode)),
                    trim($webhook->c_code),
                    $uniqueCode,
                    trim($extension)
                );
            }

            $folder = Helper::ensureTrailingBackslash(
                get_field('media_folder', $webhook->c_setting_id),
                PG_BACKSLASH_PREFIX | PG_BACKSLASH_SUFFIX
            );

            $target = [WP_CONTENT_DIR, $folder, $filename];
            $targetFilename = implode('', $target);

            file_put_contents($targetFilename, $imgData);

            Helper::log(
                sprintf(
                    '[ WhatsappBusinessMixin - UserUpload ] Successfully moved Uploaded File to %s',
                    $targetFilename
                )
            );

            $this->whatsapp($webhook, [
                'messaging_product' => 'whatsapp',
                'recipient_type' => 'individual',
                'to' => $webhook->c_user_id,
                'type' => 'reaction',
                'reaction' => [
                    'message_id' => $dot->get('entry.0.changes.0.value.messages.0.id'),
                    'emoji' => '👍',
                ],
            ]);

            $wpdb->update($table, [
                'c_state' => 'COMPLETE',
                'c_updated_at' => current_time('mysql', true),
            ], ['id' => $webhook->id], ['%s', '%s']);

        } catch (\Exception $e) {
            $wpdb->update($table, [
                'c_state' => 'FAILED',
                'c_updated_at' => current_time('mysql', true),
                'c_error_message' => $e->getMessage(),
            ], ['id' => $webhook->id], ['%s', '%s', '%s']);
        }
    }

    private function getDynamicContent($webhook, string $part, ?string $default, ?string $fileName = null): ?string
    {
        if ($fileName === null) {
            $query = [
                'post_type' => 'attachment',
                'post_status' => 'any',
                'posts_per_page' => 1,
                'meta_type' => 'DATETIME',
                'orderby' => 'meta_value_datetime',
                'meta_key' => 'file_date',
                'order' => 'DESC',
                'meta_query' => [
                    'relation' => 'AND',
                    [
                        'type_query' => [
                            'key' => 'type',
                            'value' => 'photobooth-gallery-image',
                            'compare' => '=',
                        ],
                        'code_query' => [
                            'key' => 'real_filename',
                            'value' => $webhook->c_code,
                            'compare' => 'LIKE',
                        ],
                    ],
                ],
            ];

            $query = new \WP_Query($query);

            if ($query->have_posts() === false) {
                return $default;
            }

            $post = $query->get_posts()[0];

            $fileName = get_post_meta($post->ID, 'real_filename', true);
        }

        if (!$fileName) {
            return $default;
        }

        $folder = Helper::ensureTrailingBackslash(
            get_field('media_folder', $webhook->c_setting_id),
            PG_BACKSLASH_PREFIX | PG_BACKSLASH_SUFFIX
        );

        $dynamicContentFiles = [WP_CONTENT_DIR, $folder, '*_', $part, '.txt'];

        $dynamicContentFiles = implode('', $dynamicContentFiles);
        $files = glob($dynamicContentFiles);

        if (sizeof($files) === 0) {
            return $default;
        }

        $dynamicContentFiles = array_filter($files, fn(string $file) => $this->isDynamicContentFile($file, $fileName));
        $messageFile = reset($dynamicContentFiles);

        if ($messageFile === false) {
            return $default;
        }

        return file_get_contents($messageFile);
    }

    private function isDynamicContentFile(string $file, string $fileName): bool
    {
        $urlParts = explode('/', $file);
        $file = end($urlParts);

        if ($file === false) {
            return false;
        }

        $file = explode('_', $file);

        $masterCode = reset($file);

        if ($masterCode === false) {
            return false;
        }

        return str_contains($fileName, $masterCode);
    }

    private function cronCleanup(): void
    {
        try {
            global $wpdb;
            $consentTable = self::tableConsent();
            $webhookTable = self::tableWebhook();
            $msgTable = self::tableMessage();

            $time = new DateTimeImmutable('-1day');

            $deletableWhatsappIds = $wpdb->get_results(
                $wpdb->prepare(
                    "SELECT * FROM {$consentTable} WHERE c_requested_at < %s AND c_consent_requested IS NOT NULL AND c_consent_given IS NULL",
                    $time->format('Y-m-d H:i:s')
                )
            );

            Assert::isArray($deletableWhatsappIds);

            if (sizeof($deletableWhatsappIds) === 0) {
                return;
            }

            $deletableWhatsappIds = array_map('esc_sql', $deletableWhatsappIds);
            $deletableWhatsappIds = implode("','", $deletableWhatsappIds);

            $wpdb->query(sprintf("DELETE FROM {$consentTable} WHERE c_user_id IN ('%s')", $deletableWhatsappIds));
            $wpdb->query(sprintf("DELETE FROM {$webhookTable} WHERE c_user_id IN ('%s')", $deletableWhatsappIds));
            $wpdb->query(sprintf("DELETE FROM {$msgTable} WHERE c_user_id IN ('%s')", $deletableWhatsappIds));

        } catch (\Exception $e) {
            Helper::log(
                sprintf(
                    '[ WhatsappBusinessMixin Cleanup Cron ] Exception thrown: %s',
                    $e->getMessage()
                ),
                'error'
            );
        }
    }

    private static function addResetButtonConfig(array $config): array
    {
        $admin_url = admin_url('edit.php?post_type=pg_settings&RESET_WHATSAPP_TABLES');
        $config['whatsapp_link_reset_en'] = '<a href="'.$admin_url.'" target="_blank" class="btn btn-danger"><i class="fa fa-exclamation-triangle"></i> Click Here to reset <i class="fa fa-exclamation-triangle"></i></a><p>Warning: whatsapp integration will be reset to factory settings! Please create a database backup prior clicking this button!</p>';
        $config['whatsapp_link_reset_de'] = '<a href="'.$admin_url.'" target="_blank" class="btn btn-danger"><i class="fa fa-exclamation-triangle"></i> Werkseinstellungen wiederherstellen <i class="fa fa-exclamation-triangle"></i></a><p>Achtung: Die WhatsApp-Integration wird auf die Werkseinstellungen zurückgesetzt, bitte erstellen Sie vor dem Drücken ein Backup der Datenbank!</p>';

        return $config;
    }

    private function onInit(): void
    {
        if (is_admin() === false) {
            return;
        }

        $this->handleWhatsappTable(isset($_GET['RESET_WHATSAPP_TABLES']));
        $this->handleMessageTable(isset($_GET['RESET_WHATSAPP_TABLES']));
        $this->handleConsentTable(isset($_GET['RESET_WHATSAPP_TABLES']));
        $this->handleUploadTable(isset($_GET['RESET_WHATSAPP_TABLES']));

        if (isset($_GET['8c8e38ff-210a-446e-8b9e-c24d56653ed1']) === true) {
            update_option(sprintf('%s-enable-customer-upload', self::class), 'enabled');
        }
    }

    private function handleWhatsappTable(bool $forceReset = false): void
    {
        $deprecated = get_option('PHOTOBOOTH_GALLERY_DB_VERSION_'.self::class);

        if ($forceReset === false && $deprecated === '1.0.6' && self::WHATSAPP_VERSION === '1.0.0') {
            return;
        }

        $key = sprintf('PHOTOBOOTH_GALLERY_DB_VERSION_%s_whatsapp', self::class);

        $installed_ver = get_option($key);

        if ($forceReset === false && $installed_ver === self::WHATSAPP_VERSION) {
            return;
        }

        require_once(ABSPATH.'wp-admin/includes/upgrade.php');
        global $wpdb;

        $table = self::tableWebhook();
        $charset_collate = $wpdb->get_charset_collate();

        if ($forceReset === true) {
            $wpdb->query("DROP TABLE {$table}");
        }

        $sql = "CREATE TABLE {$table} (
                id mediumint(9) NOT NULL AUTO_INCREMENT,
                c_requested_at TIMESTAMP NULL DEFAULT NULL,
                c_updated_at TIMESTAMP NULL DEFAULT NULL,
                c_state VARCHAR(255) NOT NULL,
                c_payload TEXT NOT NULL,
                c_type VARCHAR(255) NOT NULL,
                c_user_id VARCHAR(255) NOT NULL,
                c_code VARCHAR(255) DEFAULT NULL,
                c_error_message VARCHAR(255) DEFAULT NULL,
                c_setting_id mediumint(9) NOT NULL,
                c_gallery_link VARCHAR(255) DEFAULT NULL,
                c_upload_request TINYINT(1) NOT NULL DEFAULT 0,
                c_webhook_hash VARCHAR(255) NOT NULL UNIQUE,
                
                INDEX i_state (c_state),
                INDEX i_upload_request (c_upload_request),
                INDEX i_type (c_type),
                INDEX i_user_id (c_user_id),
                INDEX i_code (c_code),
                PRIMARY KEY  (id)
        ) {$charset_collate};";

        dbDelta($sql);

        delete_option(get_option('PHOTOBOOTH_GALLERY_DB_VERSION_'.self::class));
        update_option($key, self::WHATSAPP_VERSION);
    }

    private function handleMessageTable(bool $forceReset = false): void
    {
        $deprecated = get_option('PHOTOBOOTH_GALLERY_DB_VERSION_'.self::class);

        if ($forceReset === false && $deprecated === '1.0.6' && self::MESSAGE_VERSION === '1.0.0') {
            return;
        }

        $key = sprintf('PHOTOBOOTH_GALLERY_DB_VERSION_%s_message', self::class);

        $installed_ver = get_option($key);

        if ($forceReset === false && $installed_ver === self::MESSAGE_VERSION) {
            return;
        }

        require_once(ABSPATH.'wp-admin/includes/upgrade.php');
        global $wpdb;

        $charset_collate = $wpdb->get_charset_collate();
        $table = self::tableMessage();

        if ($forceReset === true) {
            $wpdb->query("DROP TABLE {$table}");
        }

        $sql = "CREATE TABLE {$table} (
                id mediumint(9) NOT NULL AUTO_INCREMENT,
                c_sent_at TIMESTAMP NULL DEFAULT NULL,
                c_state VARCHAR(255) NOT NULL,
                c_type VARCHAR(255) NOT NULL,
                c_payload TEXT NOT NULL,
                c_user_id VARCHAR(255) NOT NULL,
                c_post_id mediumint(9) DEFAULT NULL,
                c_code VARCHAR(255) DEFAULT NULL,
                c_error_message VARCHAR(255) DEFAULT NULL,
                c_setting_id mediumint(9) NOT NULL,
                
                INDEX i_state (c_state),
                INDEX i_type (c_type),
                INDEX i_user_id (c_user_id),
                INDEX i_code (c_code),
                INDEX i_post_id (c_post_id),
                PRIMARY KEY  (id)
        ) {$charset_collate};";

        dbDelta($sql);

        delete_option(get_option('PHOTOBOOTH_GALLERY_DB_VERSION_'.self::class));
        update_option($key, self::MESSAGE_VERSION);
    }

    private function handleConsentTable(bool $forceReset = false): void
    {

        $key = sprintf('PHOTOBOOTH_GALLERY_DB_VERSION_%s_consent', self::class);

        $installed_ver = get_option($key);

        if ($forceReset === false && $installed_ver === self::CONSENT_VERSION) {
            return;
        }

        require_once(ABSPATH.'wp-admin/includes/upgrade.php');
        global $wpdb;

        $charset_collate = $wpdb->get_charset_collate();
        $table = self::tableConsent();

        if ($forceReset === true) {
            $wpdb->query("DROP TABLE {$table}");
        }

        $sql = "CREATE TABLE {$table} (
                id mediumint(9) NOT NULL AUTO_INCREMENT,
                c_requested_at TIMESTAMP NULL DEFAULT NULL,
                c_updated_at TIMESTAMP NULL DEFAULT NULL,
                c_consent_requested TIMESTAMP NULL DEFAULT NULL,
                c_consent_given TIMESTAMP NULL DEFAULT NULL,
                c_rated TIMESTAMP NULL DEFAULT NULL,
                c_user_id VARCHAR(255) NOT NULL,
                
                INDEX i_consent_requested (c_consent_requested),
                INDEX i_consent_given (c_consent_given),
                INDEX i_rated (c_rated),
                INDEX i_user_id (c_user_id),
                PRIMARY KEY  (id)
        ) {$charset_collate};";

        dbDelta($sql);

        update_option($key, self::CONSENT_VERSION);
    }

    private function handleUploadTable(bool $forceReset = false): void
    {
        $deprecated = get_option('PHOTOBOOTH_GALLERY_DB_VERSION_'.self::class);

        $key = sprintf('PHOTOBOOTH_GALLERY_DB_VERSION_%s_upload', self::class);

        $installed_ver = get_option($key);

        if ($forceReset === false && $installed_ver === self::UPLOAD_VERSION) {
            return;
        }

        require_once(ABSPATH.'wp-admin/includes/upgrade.php');
        global $wpdb;

        $table = self::tableUpload();
        $charset_collate = $wpdb->get_charset_collate();

        if ($forceReset === true) {
            $wpdb->query("DROP TABLE {$table}");
        }

        $sql = "CREATE TABLE {$table} (
                id mediumint(9) NOT NULL AUTO_INCREMENT,
                c_requested_at TIMESTAMP NULL DEFAULT NULL,
                c_user_id VARCHAR(255) NOT NULL,
                c_code VARCHAR(255) DEFAULT NULL,
                c_setting_id mediumint(9) NOT NULL,
                
                INDEX i_requested_at (c_requested_at),
                INDEX i_index (c_user_id,c_setting_id,c_code),
                PRIMARY KEY  (id)
        ) {$charset_collate};";

        dbDelta($sql);

        update_option($key, self::UPLOAD_VERSION);
    }

    public function __invoke(): void
    {
        $lock = LockMixin::createLock('pg-cron-wb-webhook-check', 2 * MINUTE_IN_SECONDS);

        if ($lock->acquire() === false) {
            Helper::log('[ WhatsappBusinessMixin Cron ] Currently running, skipping this request...');

            return;
        }

        global $wpdb;
        $table = self::tableWebhook();

        try {
            $query = "SELECT * FROM {$table} WHERE c_state IN ('CREATED','PENDING') AND c_type IN ('text','interactive') AND c_upload_request = false ORDER BY c_setting_id ASC, c_requested_at DESC";
            $webhooks = $wpdb->get_results($query);

            if (sizeof($webhooks) === 0) {
                return;
            }

            $limit = 50;
            $done = 0;
            foreach ($webhooks as $webhook) {

                if (get_field('is_whatsapp_business_integration', $webhook->c_setting_id) === false) {
                    continue;
                }

                $result = $this->handleWebhookObject($webhook);

                if ($result === 'DONE') {
                    $done++;
                }

                if ($done >= $limit) {
                    break;
                }
            }
        } catch (\Exception $e) {
            Helper::log(sprintf('[ WhatsappBusinessMixin Cron ] Error: %s', $e->getMessage()), 'ERROR');
        } finally {
            $lock->release();
        }
    }

    private function handleWebhookObject($webhook): string
    {
        $table = self::tableWebhook();
        global $wpdb;

        $now = new DateTimeImmutable();

        $wpdb->update($table, [
            'c_state' => 'IN_PROGRESS',
            'c_updated_at' => current_time('mysql', true),
        ], ['id' => $webhook->id], ['%s', '%s']);

        try {

            if ($webhook->c_code === null) {
                $wpdb->update($table, [
                    'c_state' => 'COMPLETE',
                    'c_updated_at' => current_time('mysql', true),
                ], ['id' => $webhook->id], ['%s', '%s']);

                $this->handleWebhookWithoutCode($webhook);

                return 'DONE';
            }

            $transitionState = 'PENDING';
            $requestedAt = DateTimeImmutable::createFromFormat('Y-m-d H:i:s', $webhook->c_requested_at);

            $timeInMinutes = get_field('whatsapp_business_rate_me_delay', $webhook->c_setting_id);
            $timeInMinutes = intval($timeInMinutes ?? 1435);

            $rateAt = $requestedAt->add(new DateInterval(sprintf('PT%dM', $timeInMinutes)));
            $requestedAt = $requestedAt->add(new DateInterval(self::DELAY));

            $query = [
                'post_type' => 'attachment',
                'post_status' => 'any',
                'posts_per_page' => -1,
                'meta_type' => 'DATETIME',
                'orderby' => 'meta_value_datetime',
                'meta_key' => 'file_date',
                'order' => 'DESC',
                'meta_query' => [
                    'relation' => 'AND',
                    [
                        'type_query' => [
                            'key' => 'type',
                            'value' => 'photobooth-gallery-image',
                            'compare' => '=',
                        ],
                        'code_query' => [
                            'key' => 'real_filename',
                            'value' => $webhook->c_code,
                            'compare' => 'LIKE',
                        ],
                    ],
                ],
            ];

            if ($webhook->c_state === 'PENDING' && $now > $requestedAt) {
                $transitionState = 'COMPLETE';
            }

            if ($webhook->c_state === 'PENDING') {
                $query = $this->excludeMessagesSent($webhook, $query);
            }

            $isRateMeAllowed = get_field(
                    'is_whatsapp_business_rate_me',
                    $webhook->c_setting_id
                ) === true && $now > $rateAt;

            $query = new \WP_Query($query);

            if ($query->have_posts() === false) {
                $wpdb->update($table, [
                    'c_state' => $transitionState,
                    'c_updated_at' => current_time('mysql', true),
                ], ['id' => $webhook->id], ['%s', '%s']);

                if ($isRateMeAllowed) {
                    $this->handleRateMe($webhook);
                }

                return $transitionState === 'COMPLETE' ? 'DONE' : 'SKIPPED';
            }

            $caption = get_field('webhook_whatsapp_business_caption_template', $webhook->c_setting_id);

            if ($caption === null) {
                $caption = self::translations()['caption_default'];
            }

            if ($this->isConsentRequired($webhook) === true) {

                $wpdb->update($table, [
                    'c_state' => $transitionState,
                    'c_updated_at' => current_time('mysql', true),
                ], ['id' => $webhook->id], ['%s', '%s']);

                return $transitionState === 'COMPLETE' ? 'DONE' : 'SKIPPED';
            }

            foreach ($query->posts as $post) {
                $url = wp_get_attachment_url($post->ID);

                if (!$url) {
                    continue;
                }


                $mime_type = 'image';

                if ($post->post_mime_type) {
                    $types = explode('/', $post->post_mime_type);
                    $mime_type = strtolower((string)reset($types));
                }

                switch ($mime_type) {
                    case 'image':
                        $this->sendImage($webhook, $url, $caption, $mime_type, $post->ID);
                        break;
                    case 'video':
                        $this->sendVideo($webhook, $url, $caption, $mime_type, $post->ID);
                        break;
                }
            }

            $wpdb->update($table, [
                'c_state' => $transitionState,
                'c_updated_at' => current_time('mysql', true),
            ], ['id' => $webhook->id], ['%s', '%s']);

            if ($isRateMeAllowed) {
                $this->handleRateMe($webhook);
            }

            return 'DONE';

        } catch (\Exception $exception) {
            $wpdb->update($table, [
                'c_state' => 'FAILED',
                'c_error_message' => $exception->getMessage(),
                'c_updated_at' => current_time('mysql', true),
            ], ['id' => $webhook->id], ['%s', '%s', '%s']);
        }

        return 'ERROR';
    }

    private function handleWebhookWithoutCode($webhook): void
    {
        global $wpdb;
        $data = json_decode($webhook->c_payload, true);
        $dot = new Dot($data);

        $button = $dot->get('entry.0.changes.0.value.messages.0.interactive.button_reply.id');

        if ($webhook->c_type === 'interactive' && $button === 'accept-button') {
            $table = self::tableConsent();

            $wpdb->update($table, [
                'c_consent_given' => current_time('mysql', true),
                'c_updated_at' => current_time('mysql', true),
            ], ['c_user_id' => $webhook->c_user_id], ['%s', '%s']);

            return;
        }

        if (get_field('is_whatsapp_business_mail', $webhook->c_setting_id) === false) {
            return;
        }

        if ($webhook->c_type === 'text') {
            $admin = get_user_by('email', get_bloginfo('admin_email'));
            $message = $dot->get('entry.0.changes.0.value.messages.0.text.body');
            $link = sprintf(
                '<a href="https://wa.me/%s?text=%s">%s</a>',
                $webhook->c_user_id,
                urlencode(__('Hey, thanks for your message, how can I help?', 'photobooth-gallery')),
                __('Answer now!', 'photobooth-gallery')
            );

            $body = sprintf(
                '<pre style="margin-bottom: 20px">%s</pre><p>Nachricht vom User: <pre>%s</pre></p><p>%s</p>',
                json_encode($data, JSON_PRETTY_PRINT),
                sanitize_text_field($message),
                $link
            );

            wp_mail(
                $admin->user_email,
                __('New Message from Whatsapp!', 'photobooth-gallery'),
                $body,
                ['Content-Type: text/html; charset=UTF-8']
            );
        }
    }

    private function excludeMessagesSent($webhook, array $queryArgs): array
    {
        global $wpdb;
        $table = self::tableMessage();

        $query = $wpdb->prepare(
            "SELECT DISTINCT c_post_id FROM {$table} WHERE c_state = %s AND c_user_id = %s AND c_setting_id = %d",
            [
                'SUCCESSFUL',
                $webhook->c_user_id,
                $webhook->c_setting_id,
            ]
        );

        $posts = $wpdb->get_results($query);

        if (sizeof($posts) === 0) {
            return $queryArgs;
        }

        $idSet = array_map(fn($post) => intval($post->c_post_id), $posts);

        $queryArgs['post__not_in'] = $idSet;

        return $queryArgs;
    }

    private function handleRateMe($webhook): void
    {
        global $wpdb;
        $table = self::tableConsent();

        $query = $wpdb->prepare("SELECT * FROM {$table} WHERE c_user_id = %s", [$webhook->c_user_id]);
        $consent = $wpdb->get_row($query);

        if ($consent === null) {
            return;
        }

        if ($consent->c_rated !== null) {
            return;
        }

        $isConsentRequired = get_field('is_whatsapp_business_gpdr', $webhook->c_setting_id);

        if ($isConsentRequired === true && $consent->c_consent_given === null) {
            return;
        }

        $message = get_field('whatsapp_business_rate_me', $webhook->c_setting_id);

        $message = $this->getDynamicContent($webhook, 'rateme', $message);

        if (strlen($message) <= 3) {
            return;
        }

        $wpdb->update($table, [
            'c_rated' => current_time('mysql', true),
            'c_updated_at' => current_time('mysql', true),
        ], ['c_user_id' => $webhook->c_user_id], ['%s', '%s']);

        $this->whatsapp($webhook, [
            'messaging_product' => 'whatsapp',
            'recipient_type' => 'individual',
            'to' => $webhook->c_user_id,
            'type' => 'text',
            'text' => [
                'preview_url' => true,
                'body' => $message,
            ],
        ]);
    }

    public static function translations(): array
    {
        return [
            'enable' => __('Enable Whatsapp Business Integration', 'photobooth-gallery'),
            'enable_hint' => __(
                'After enabling the whatsapp business integration and adding the token and a webhook password in this this form, after saving you will see a webhook url that can be pasted into your whatsapp webhook form for receiving image requests.',
                'photobooth-gallery'
            ),
            'password' => __('Enter your Whatsapp Business Webhook password', 'photobooth-gallery'),
            'password_hint' => __(
                'The password has to be lowercase and may only contain numbers and letters.',
                'photobooth-gallery'
            ),
            'token' => __('Enter your Whatsapp Business Webhook User Token', 'photobooth-gallery'),
            'token_hint' => __(
                'You can request an user token in your whatsapp business user configuration',
                'photobooth-gallery'
            ),
            'caption' => __('Enter the caption text', 'photobooth-gallery'),
            'caption_hint' => __(
                'You can use the following placeholders to replace parts of the caption:<br />{type} : Type will be either video or image depending on the source of the media < br />{code} : The code used to retrieve the image(s) / video(s) < br />{url} : The corresponding photobooth gallery page with the code already prefilled',
                'photobooth-gallery'
            ),
            'caption_default' => __(
                'Here is your requested {type} for the code {code}. You can download it in full size here: {url}',
                'photobooth-gallery'
            ),
            'enable_email' => __('Enable Text2Mail', 'photobooth-gallery'),
            'email_hint' => __(
                'When enabled, you will receive emails with the text content sent by whatsapp users to your configured whatsapp business telephone number.',
                'photobooth-gallery'
            ),
            'enable_dsgvo' => __('Enable GDPR consent mode', 'photobooth-gallery'),
            'dsgvo_label' => __('GDPR Message', 'photobooth-gallery'),
            'dsgvo_text' => __(
                'GDPR Message that asks the user for permission and also contains a link to your data privacy declaration',
                'photobooth-gallery'
            ),
            'dsgvo_hint' => __(
                'When enabled, whatsapp users will have to give consent prior receiving any media data from this gallery',
                'photobooth-gallery'
            ),
            'enable_feeback' => __('Enable RateMe Request', 'photobooth-gallery'),
            'feedback_hint' => __(
                'When enabled, users will receive a request once after first 24 hours to rate your service with a predefined text and url',
                'photobooth-gallery'
            ),
            'feedback_request' => __('Your RateMe Request', 'photobooth-gallery'),
            'feedback_request_hint' => __(
                'RateMe Request Message that will be sent to the user.It should contain the url to your rating service provider',
                'photobooth-gallery'
            ),
            'feedback_request_delay_hint' => __(
                'After delay minutes and after the user has accepted the GDPR (if enabled) the user will receive the request to rate your service with the message provided. Default is 23 hours and 55 minutes (1435 minutes)',
                'photobooth-gallery'
            ),
            'feedback_request_delay' => __('RateMe Request Delay in Minutes', 'photobooth-gallery'),
            'reset_button_link' => __('Reset whatsapp database tables', 'photobooth-gallery'),
            'whatsapp_link_reset' => __('{whatsapp_link_reset_en}', 'photobooth-gallery'),
            'upload_enabled' => __('Enable User Media Upload', 'photobooth-gallery'),
            'upload_hint' => __(
                'When enabled, whatsapp users will be able to upload own media files into the photo gallery',
                'photobooth-gallery'
            ),
            'upload_request' => __('User Media Upload Request', 'photobooth-gallery'),
            'upload_request_hint' => __(
                'Please enter the text that will be the trigger for enabling user media upload. You can use the {code} placeholder variable to denote the group code for clustering uploads.<br/><strong>Please note:</strong> The code is mandatory for the feature to work properly.',
                'photobooth-gallery'
            ),
            'upload_request_ready_message' => __('User Media Upload Ready Message', 'photobooth-gallery'),
            'upload_request_ready_hint' => __('This Message will be sent to the user after user media upload request has been successfully enabled', 'photobooth-gallery'),
        ];
    }

    private function sendImage($webhook, string $url, string $caption, string $mimeType, int $postId): void
    {
        $engine = new Engine();

        if (str_contains($caption, 'requested') === false) {
            $mimeType = 'Bild';
        }

        $galleryUrl = $this->getDynamicContent($webhook, 'url', $webhook->c_gallery_link);
        $galleryUrl = str_replace('{code}', $webhook->c_code, $galleryUrl);

        $payload = [
            'messaging_product' => 'whatsapp',
            'recipient_type' => 'individual',
            'to' => $webhook->c_user_id,
            'type' => 'image',
            'image' => [
                'link' => $url,
                'caption' => $engine->render($caption, [
                    'type' => ucfirst($mimeType),
                    'code' => $webhook->c_code,
                    'url' => $galleryUrl,
                ]),
            ],
        ];

        $this->whatsapp($webhook, $payload, $postId);
    }

    private function sendVideo($webhook, string $url, string $caption, string $mimeType, int $postId): void
    {
        $engine = new Engine();

        $galleryUrl = $this->getDynamicContent($webhook, 'url', $webhook->c_gallery_link);
        $galleryUrl = str_replace('{code}', $webhook->c_code, $galleryUrl);

        $payload = [
            'messaging_product' => 'whatsapp',
            'recipient_type' => 'individual',
            'to' => $webhook->c_user_id,
            'type' => 'video',
            'video' => [
                'link' => $url,
                'caption' => $engine->render($caption, [
                    'type' => ucfirst($mimeType),
                    'code' => $webhook->c_code,
                    'url' => $galleryUrl,
                ]),
            ],
        ];

        $payload = $this->handleFileSizeRestrictions($payload, $postId);


        $this->whatsapp($webhook, $payload, $postId);
    }

    private function handleFileSizeRestrictions(array $payload, int $postId): array
    {
        try {
            $constraints = self::whatsappLimits();
            Assert::keyExists($payload, 'type');
            $type = $payload['type'];
            Assert::keyExists($constraints, $type);

            $constraint = $constraints[$type];
            $filesize = filesize(get_attached_file($postId));
            Assert::greaterThanEq($filesize, $constraint);

            $payload['type'] = 'document';
            $payload['document'] = $payload[$type];
            unset($payload[$type]);

            return $payload;
        } catch (InvalidArgumentException) {
            return $payload;
        }
    }

    private static function whatsappLimits(): array
    {
        return [
            'image' => 5 * 1024 * 1024,
            'video' => 16 * 1024 * 1024,
            'audio' => 16 * 1024 * 1024,
        ];
    }
}

