<?php
/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 28.11.2018
 * Time: 18:47
 */

namespace Photobooth\Statistics;

use Photobooth\Helper;
use Photobooth\Renderer;
use Photobooth\Transients;

define('PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS', '1.0.1');
define('PHOTOBOOTH_GALLERY_STATISTICS_DEBUG', false);

class XMLStatistics
{


    /**
     * @var Renderer
     */
    private $render;

    /**
     * @var string
     */
    private $uuid;

    /**
     * XMLStatistics constructor.
     */
    public function __construct()
    {

        $this->render = new Renderer();
    }

    public static function init()
    {

        if (!defined('PG_ELEMENTOR_DIR')) {
            return;
        }

        $installed_ver = get_option('PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS');

        if ($installed_ver === PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS) {
            return;
        }

        $table_name = self::table();

        $sql = "CREATE TABLE {$table_name} (
                id mediumint(9) NOT NULL AUTO_INCREMENT,
                c_file varchar(255) NOT NULL,
                c_source mediumtext NOT NULL,
                c_breeze_version varchar(255) NOT NULL,
                c_output mediumtext DEFAULT '',
                c_user_data text DEFAULT '',
                c_images_folder mediumtext NOT NULL,
                c_download_folder mediumtext NOT NULL,
                c_timestamp timestamp DEFAULT CURRENT_TIMESTAMP,
                c_prints int NOT NULL,
                c_digital_zoom int NOT NULL,
                c_mirror_output int NOT NULL,
                c_greenscreen int NOT NULL,
                c_camera_orientation int NOT NULL,
                c_save_copy int NOT NULL,
                c_bw_mode int NOT NULL,
                c_photos int NOT NULL,
                c_xml_type varchar(255) NOT NULL,
                KEY file (c_file),
                KEY version (c_breeze_version),
                PRIMARY KEY (id)
        );";

        require_once(ABSPATH.'wp-admin/includes/upgrade.php');

        dbDelta($sql);

        Helper::log(
            sprintf(
                'Creating / Updating Table %s from version %s to %s',
                self::table(),
                $installed_ver,
                PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS
            )
        );

        update_option('PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS', PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS);

        self::migrate();
    }

    public static function table()
    {

        global $wpdb;

        $table_name = $wpdb->prefix.'pg_xml_statistics';

        return esc_sql($table_name);
    }

    private static function migrate()
    {

        global $wpdb;

        // V1.0.1 migration
        if (version_compare(PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS, '1.0.1', '=')) {

            $query = sprintf("UPDATE %s SET %s = '%s'", self::table(), 'c_xml_type', 'base application');

            $wpdb->query($query);
        }

    }

    public function deactivate()
    {

        $crons = [
            'pg-cron-xml-parser',
        ];

        foreach ($crons as $cron) {

            if (!wp_next_scheduled($cron)) {
                continue;
            }

            wp_clear_scheduled_hook($cron);
        }

        // delete all transient by sql query
        global $wpdb;

        $wpdb->query(
            sprintf(
                "DELETE FROM %s WHERE option_name LIKE '_transient_%s'",
                $wpdb->options,
                Transients::STATISTIC_FILTER
            )
        );
    }

    public function wp_init()
    {

        add_filter('photobooth/scan_images/skip_import', [$this, 'handle_import'], 10, 5);
        add_action('photobooth/plugin/xml_statistic/import', $this);

        if (!defined('PG_ELEMENTOR_DIR')) {
            return;
        }

        if (current_user_can('manage_options') === false) {
            return;
        }

        add_action('wp_enqueue_scripts', [$this, 'enqueue_js_css']);

        add_action('wp_ajax_pg-statistic-filter', [$this, 'ajax_filter']);
        add_action('wp_ajax_nopriv_pg-statistic-filter', [$this, 'ajax_filter']);

        add_shortcode('breeze-xml-statistics', [$this, 'do_shortcode']);

        $this->uuid = $this->get_user_identification();
    }

    private function get_user_identification()
    {

        if (!PG_COOKIE) {
            return uniqid('pg-xml-cookie', true);
        }

        if (array_key_exists('pg-xml-filter', $_COOKIE)) {
            return $_COOKIE['pg-xml-filter'];
        }

        $uuid = uniqid('pg-xml-cookie', true);

        $cookie_id = sprintf('%s-%s', Transients::STATISTIC_FILTER, sha1($this->get_table().$uuid));

        setcookie('pg-xml-filter', $cookie_id, strtotime('+7days'), '/');

        return $cookie_id;
    }

    private function get_table()
    {

        return self::table();
    }

    function ajax_filter()
    {

        check_ajax_referer('pg-statistic-filter', 'csrf');

        $from = sanitize_text_field($_REQUEST['from'] ?? false);
        $to = sanitize_text_field($_REQUEST['to'] ?? false);
        $source = sanitize_text_field($_REQUEST['source'] ?? false);
        $hours = sanitize_text_field($_REQUEST['hours'] ?? false);
        $reset = sanitize_text_field($_REQUEST['is_reset'] ?? false);

        $key = $this->uuid;

        set_transient($key, compact('from', 'to', 'source', 'hours'), 7 * DAY_IN_SECONDS);

        if ($reset === '1') {
            delete_transient($key);
        }

        wp_send_json([
            'reload' => true,
        ]);
    }

    function __invoke($xml_file): void
    {
        if (!file_exists($xml_file)) {
            return;
        }

        $xml = $this->parse_xml($xml_file);

        if (!$xml) {
            return;
        }

        $file = basename($xml_file);
        $file_path = dirname($xml_file);

        $this->handle_xml($xml, $file, $file_path);
    }

    private function parse_xml($file)
    {

        libxml_use_internal_errors(true);
        $xml_object = simplexml_load_file($file);

        if ($xml_object === false) {
            $errors = array_map(fn($error) => $error->message, libxml_get_errors());
            Helper::log(sprintf('[ XML-Statistic ] XML Error: %s ', implode(PHP_EOL, $errors)), 'error');
            libxml_clear_errors();

            return false;
        }

        $json = json_encode($xml_object);

        if (PG_LOG_VERBOSE) {
            Helper::log(sprintf('[ XML-Statistic ] XML Debug: %s ', $json));
        }

        return json_decode($json, true);
    }

    private function handle_xml($xml_data, $file_name, $file_path)
    {

        if (!$file_name) {
            return true;
        }

        $table_name = $this->get_table();

        $query = sprintf(
            "SELECT count(id) as file_count  FROM {$table_name} WHERE  c_file = '%s'",
            esc_sql($file_name)
        );
        $result = $this->get_single_result($query);

        if (!$result) {
            return true;
        }

        // Already added this file skip it
        if (intval($result->file_count) > 0) {
            return true;
        }


        $breeze_type = $xml_data['@attributes'];

        // base xml
        if (!array_key_exists('name', $breeze_type)) {
            return $this->handle_base_xml($xml_data, $file_name, $file_path);
        }

        $this->handle_app_xml($xml_data, $file_name, $file_path);
    }

    private function get_single_result($query)
    {

        global $wpdb;

        $result = $wpdb->get_results($query);

        if (!is_array($result)) {
            return false;
        }

        $result = reset($result);

        return $result;
    }

    private function handle_base_xml($xml_data, $file_name, $file_path)
    {

        $source = pathinfo($file_path, PATHINFO_DIRNAME);

        if (array_key_exists('video_information', $xml_data)) {
            return $this->handle_video_information($xml_data, $file_name, $file_path);
        }

        $info = $xml_data['photo_information'] ?? [];

        // Skip import if xml data could not be extracted
        if (!$info) {
            Helper::log(
                sprintf(
                    '[ XML Statistics - Deprecated ] Could not parse XML file %s correctly, skipping import ...',
                    $file_name
                ),
                'error'
            );

            // Remove the file
            return true;
        }

        $version = $xml_data['@attributes']['version'] ?? false;

        // Skip import if xml data could not be extracted
        if (!$version) {
            Helper::log(
                sprintf(
                    '[ XML Statistics ] Could not get Breeze version from xml file %s, skipping import ...',
                    $file_name
                ),
                'warning'
            );

            // Remove the file
            return true;
        }

        $time = $info['date'].' '.$info['time'];
        $time = str_replace('/', '-', $time);

        if (isset($info['photos']['output']['@attributes'])) {
            $info['photos']['output'] = $info['photos']['output']['0'];
        }

        $output = $info['photos']['output'] ?? false;

        if (!$output && array_key_exists('gif_file', $info)) {
            $output = $info['gif_file'];
        }

        if (isset($info['photos']['photo']['@attributes'])) {
            $info['photos']['photo'] = $info['photos']['photo']['0'];
        }

        $photos = $info['photos']['photo'] ?? [];

        if (is_string($photos)) {
            $photos = [
                $info['photos']['photo'],
            ];
        }


        $id = $this->insert_row([
            'c_file' => $file_name,
            'c_source' => $source,
            'c_breeze_version' => $version,
            'c_user_data' => json_encode($info['user_data'] ?? []),
            'c_images_folder' => $info['photobooth_images_folder'] ?? '',
            'c_download_folder' => $info['download_folder'] ?? '',
            'c_output' => $output,
            'c_timestamp' => $time,

            'c_prints' => intval($info['prints']),
            'c_digital_zoom' => intval($info['digital_zoom']),
            'c_mirror_output' => intval($info['mirror_output']),
            'c_greenscreen' => intval($info['greenscreen']),
            'c_camera_orientation' => intval($info['camera_orientation']),
            'c_save_copy' => intval($info['saveCopyOfProcessedPhotos']),
            'c_bw_mode' => intval($info['bw_mode']),
            'c_photos' => sizeof($photos),
            'c_xml_type' => 'base application',
        ],
            [
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',

                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%s',
            ]);

        if ($id > 0) {
            Helper::log(
                sprintf(
                    '[ XML Statistics ] Successfully added %s to statistics database with ID %d...',
                    $file_name,
                    $id
                )
            );
        }

        return $id;
    }

    /**
     * Video version with GIF and MP4 support
     *
     * @param $xml_data
     * @param $file_name
     * @param $file_path
     *
     * @return bool|int
     */
    private function handle_video_information($xml_data, $file_name, $file_path)
    {

        $breeze_data = $xml_data['@attributes'];
        $source = pathinfo($file_path, PATHINFO_DIRNAME);
        $info = $xml_data['video_information'];
        $version = $breeze_data['version'] ?? false;

        $time = $info['date'].' '.$info['time'];
        $time = str_replace('/', '-', $time);

        $mp4 = $info['mp4_file'] ?? '';
        $gif = $info['gif_file'] ?? '';

        if (isset($info['photos']['photo']['@attributes'])) {
            $info['photos']['photo'] = $info['photos']['photo']['0'];
        }

        $photos = $info['photos']['photo'] ?? [];

        if (is_string($photos)) {
            $photos = [
                $info['photos']['photo'],
            ];
        }

        $id = 0;

        if ($mp4) {
            $id = $this->insert_row([
                'c_file' => $file_name,
                'c_source' => $source,
                'c_breeze_version' => $version,
                'c_user_data' => '',
                'c_images_folder' => $info['photobooth_images_folder'] ?? '',
                'c_download_folder' => '',
                'c_output' => $mp4,
                'c_timestamp' => $time,

                'c_prints' => 0,
                'c_digital_zoom' => 0,
                'c_mirror_output' => 0,
                'c_greenscreen' => 0,
                'c_camera_orientation' => 0,
                'c_save_copy' => 0,
                'c_bw_mode' => 0,
                'c_photos' => sizeof($photos),
                'c_xml_type' => 'video application mp4',
            ],
                [
                    '%s',
                    '%s',
                    '%s',
                    '%s',
                    '%s',
                    '%s',
                    '%s',
                    '%s',

                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%s',
                ]);

            if ($id > 0) {
                Helper::log(
                    sprintf(
                        '[ XML Statistics - MP4] Successfully added %s to statistics database with ID %d...',
                        $file_name,
                        $id
                    )
                );
            }
        }

        if ($gif) {
            $id = $this->insert_row([
                'c_file' => $file_name,
                'c_source' => $source,
                'c_breeze_version' => $version,
                'c_user_data' => '',
                'c_images_folder' => $info['photobooth_images_folder'] ?? '',
                'c_download_folder' => '',
                'c_output' => $gif,
                'c_timestamp' => $time,

                'c_prints' => 0,
                'c_digital_zoom' => 0,
                'c_mirror_output' => 0,
                'c_greenscreen' => 0,
                'c_camera_orientation' => 0,
                'c_save_copy' => 0,
                'c_bw_mode' => 0,
                'c_photos' => sizeof($photos),
                'c_xml_type' => 'video application gif',
            ],
                [
                    '%s',
                    '%s',
                    '%s',
                    '%s',
                    '%s',
                    '%s',
                    '%s',
                    '%s',

                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%d',
                    '%s',
                ]);

            if ($id > 0) {
                Helper::log(
                    sprintf(
                        '[ XML Statistics ] Successfully added %s (GIF) to statistics database with ID %d...',
                        $file_name,
                        $id
                    )
                );
            }
        }

        return $id;
    }

    private function insert_row($data, $definition)
    {

        global $wpdb;

        $table_name = $this->get_table();

        $result = $wpdb->insert($table_name, $data, $definition);

        if (!$result) {
            Helper::log(
                sprintf('[ XML Statistics ] Could not INSERT %s into %s', json_encode($data), $table_name),
                'error'
            );

            return false;
        }

        return $wpdb->insert_id;
    }

    private function handle_app_xml($xml_data, $file_name, $file_path)
    {

        $breeze_data = $xml_data['@attributes'];
        $version = $breeze_data['version'] ?? false;

        // Skip import if xml data could not be extracted
        if (!$version) {
            Helper::log(
                sprintf(
                    '[ XML Statistics ] Could not get Breeze version from xml file %s, skipping import ...',
                    $file_name
                ),
                'warning'
            );

            // Remove the file
            return true;
        }

        if (array_key_exists('gif_information', $xml_data)) {
            return $this->handle_gif_information($xml_data, $file_name, $file_path);
        }

        if (array_key_exists('photo_information', $xml_data)) {
            return $this->handle_photo_information($xml_data, $file_name, $file_path);
        }

        Helper::log(
            sprintf('[ XML Statistics ] Could not parse XML file %s correctly, skipping import ...', $file_name),
            'error'
        );

        // Remove the file
        return true;
    }

    /**
     * ipad Video version
     *
     * @param $xml_data
     * @param $file_name
     * @param $file_path
     *
     * @return bool|int
     */
    private function handle_gif_information($xml_data, $file_name, $file_path)
    {

        $breeze_data = $xml_data['@attributes'];
        $source = pathinfo($file_path, PATHINFO_DIRNAME);
        $info = $xml_data['gif_information'];
        $version = $breeze_data['version'] ?? false;

        $time = $info['date'].' '.$info['time'];
        $time = str_replace('/', '-', $time);

        $output = $info['mp4_file'] ?? '';

        $id = $this->insert_row([
            'c_file' => $file_name,
            'c_source' => $source,
            'c_breeze_version' => $version,
            'c_user_data' => '',
            'c_images_folder' => $info['photobooth_images_folder'] ?? '',
            'c_download_folder' => '',
            'c_output' => $output,
            'c_timestamp' => $time,

            'c_prints' => 0,
            'c_digital_zoom' => 0,
            'c_mirror_output' => 0,
            'c_greenscreen' => 0,
            'c_camera_orientation' => 0,
            'c_save_copy' => 0,
            'c_bw_mode' => 0,
            'c_photos' => 1,
            'c_xml_type' => implode(' | ', [$breeze_data['app'], $breeze_data['name']]),
        ],
            [
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',

                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%s',
            ]);

        if ($id > 0) {
            Helper::log(
                sprintf(
                    '[ XML Statistics - GIF] Successfully added %s (%s) to statistics database with ID %d...',
                    $file_name,
                    $breeze_data['app'],
                    $id
                )
            );
        }

        return $id;
    }

    /**
     * ipad photo version with optional video handling (will create a second entry)
     *
     * @param $xml_data
     * @param $file_name
     * @param $file_path
     *
     * @return bool|int
     */
    private function handle_photo_information($xml_data, $file_name, $file_path)
    {

        $breeze_data = $xml_data['@attributes'];
        $source = pathinfo($file_path, PATHINFO_DIRNAME);
        $info = $xml_data['photo_information'];
        $version = $breeze_data['version'] ?? false;

        $time = $info['date'].' '.$info['time'];
        $time = str_replace('/', '-', $time);

        $output = $info['photos']['output'] ?? '';
        $mp4 = $info['mp4_file'] ?? '';

        if (isset($info['photos']['photo']['@attributes'])) {
            $info['photos']['photo'] = $info['photos']['photo']['0'];
        }

        $photos = $info['photos']['photo'] ?? [];

        if (is_string($photos)) {
            $photos = [
                $info['photos']['photo'],
            ];
        }

        $id = $this->insert_row([
            'c_file' => $file_name,
            'c_source' => $source,
            'c_breeze_version' => $version,
            'c_user_data' => json_encode($info['user_data'] ?? []),
            'c_images_folder' => $info['photobooth_images_folder'] ?? '',

            'c_download_folder' => $info['download_folder'] ?? '',
            'c_output' => $output,
            'c_timestamp' => $time,

            'c_prints' => intval($info['prints'] ?? 0),
            'c_digital_zoom' => intval($info['digital_zoom'] ?? 0),
            'c_mirror_output' => intval($info['mirror_output'] ?? 0),
            'c_greenscreen' => intval($info['greenscreen'] ?? 0),
            'c_camera_orientation' => intval($info['camera_orientation'] ?? 0),
            'c_save_copy' => intval($info['saveCopyOfProcessedPhotos'] ?? 0),
            'c_bw_mode' => intval($info['bw_mode'] ?? 0),
            'c_photos' => sizeof($photos),
            'c_xml_type' => implode(' | ', [$breeze_data['app'], $breeze_data['name']]),
        ],
            [
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',

                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%s',
            ]);

        if ($id > 0) {
            Helper::log(
                sprintf(
                    '[ XML Statistics - PHOTO] Successfully added %s (%s) to statistics database with ID %d...',
                    $file_name,
                    $breeze_data['app'],
                    $id
                )
            );
        }

        if (!$mp4) {
            return $id;
        }

        $id = $this->insert_row([
            'c_file' => $file_name,
            'c_source' => $source,
            'c_breeze_version' => $version,
            'c_user_data' => '',
            'c_images_folder' => $info['photobooth_images_folder'] ?? '',
            'c_download_folder' => '',
            'c_output' => $mp4,
            'c_timestamp' => $time,

            'c_prints' => 0,
            'c_digital_zoom' => 0,
            'c_mirror_output' => 0,
            'c_greenscreen' => 0,
            'c_camera_orientation' => 0,
            'c_save_copy' => 0,
            'c_bw_mode' => 0,
            'c_photos' => 1,
            'c_xml_type' => implode(' | ', [$breeze_data['app'], $breeze_data['name']]),
        ],
            [
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',

                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%d',
                '%s',
            ]);

        if ($id > 0) {
            Helper::log(
                sprintf(
                    '[ XML Statistics - MP4] Successfully added %s (%s) to statistics database with ID %d...',
                    $file_name,
                    $breeze_data['app'],
                    $id
                )
            );
        }

        return $id;
    }

    function handle_import($skip_import, $suffix, $file, $meta_data, $settings)
    {

        $suffix = strtolower($suffix);

        // We only want XML files
        if ($suffix !== 'xml') {
            return $skip_import;
        }

        if (!file_exists($file)) {
            return true;
        }

        $xml = $this->parse_xml($file);

        if (!$xml) {
            return true;
        }
        $this->handle_xml($xml, $meta_data['real_filename'], $meta_data['real_filepath']);

        if ($settings['unlink']) {
            unlink($file);
        }

        return true;
    }

    function enqueue_js_css()
    {

        if ($this->is_moment_js_disabled()) {
            return;
        }

        wp_enqueue_style(
            'bootstrap-datetimepicker',
            plugin_dir_url(PG_BOOTSTRAP_FILE)."public/css/bootstrap-datetimepicker.css",
            array('photobooth-gallery'),
            PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS,
            'all'
        );

        wp_enqueue_script(
            'moment',
            plugin_dir_url(PG_BOOTSTRAP_FILE)."public/js/moment.js",
            array('jquery'),
            PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS,
            true
        );
        wp_enqueue_script(
            'moment-locale-de',
            plugin_dir_url(PG_BOOTSTRAP_FILE)."public/js/de.js",
            array(
                'jquery',
                'moment',
            ),
            PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS,
            true
        );

        wp_enqueue_script(
            'bootstrap-datetimepicker',
            plugin_dir_url(PG_BOOTSTRAP_FILE)."public/js/bootstrap-datetimepicker.js",
            array(
                'jquery',
                'moment',
                'moment-locale-de',
            ),
            PHOTOBOOTH_GALLERY_DB_VERSION_STATISTICS,
            true
        );

        wp_localize_script(
            'bootstrap-datetimepicker',
            'pg_statistic',
            array(
                'ajaxurl' => admin_url('admin-ajax.php'),
                'pg_statistic_filter' => wp_create_nonce('pg-statistic-filter'),
            )
        );
    }

    private function is_moment_js_disabled()
    {

        $settings = Helper::get_plugin_settings();

        $settings_key = sprintf('is_%s_js_disabled', 'moment-js');

        if (!array_key_exists($settings_key, $settings)) {
            return false;
        }

        return $settings[$settings_key];
    }

    function do_shortcode($atts)
    {

        $atts = shortcode_atts(array(
            'from' => date('Y-m-d H:i:s', strtotime('-3day 00:00:00')),
            'to' => date('Y-m-d H:i:s', strtotime('now')),
            'source' => '',
            'view' => 'table',
            'hours' => 0,
        ),
            $atts,
            'breeze-xml-statistics');

        $filter = get_transient($this->uuid);

        if ($filter) {

            foreach ($filter as $key => $value) {

                if ($value === false) {
                    continue;
                }

                $atts[$key] = $value;
            }
        }

        $stats = $this->get_statistics($atts);

        $prev = get_transient($this->uuid.'-previous-stats');

        if (!$prev) {
            $prev = $stats;
        }

        $prev = $this->optimize_previous_stats($stats, $prev);

        set_transient($this->uuid.'-previous-stats', $stats, 7 * DAY_IN_SECONDS);

        $last_date = false;

        if ($prev) {
            $last_date = reset($prev);
            $last_date = $last_date->execution_date;
        }

        $sums = [
            'sessions' => 0,
            'photos' => 0,
            'prints' => 0,
        ];

        foreach ($stats as $stat) {

            foreach ($sums as $key => & $value) {
                $value += $stat->{$key};
            }
        }

        return $this->render->view(
            sprintf('xml-statistics-%s', $atts['view']),
            compact('stats', 'atts', 'prev', 'last_date', 'sums')
        );
    }

    public function get_statistics($atts)
    {


        if (PHOTOBOOTH_GALLERY_STATISTICS_DEBUG) {

            $sources = mt_rand(2, 2);
            $results = [];

            $begin = strtotime('-100days');
            for ($i = 0; $i < $sources; $i++) {

                $start_date = mt_rand($begin, time());
                $end_date = mt_rand($start_date, time());

                $result = [
                    'sessions' => mt_rand(100, 50000),
                    'photos' => mt_rand(100, 50000),
                    'prints' => mt_rand(100, 50000),
                    'start_date' => date('Y-m-d H:i:s', $start_date),
                    'end_date' => date('Y-m-d H:i:s', $end_date),
                    'source' => uniqid('source-'),
                    'execution_date' => date('Y-m-d H:i:s', time()),
                ];

                $db_object = new \stdClass();

                foreach ($result as $prop => $val) {
                    $db_object->{$prop} = $val;
                }

                $results[] = $db_object;
            }

            return $results;

        }

        $time = $this->get_time($atts);
        $source = $this->get_source($atts);

        $where = [];
        $group_by = '';

        if ($time) {

            $where[] = $time;
        }

        if ($source) {

            if ($where) {
                $where[] = 'AND';
            }

            $where[] = $source;
        }

        if (array_key_exists('where', $atts) && strlen($atts['where'])) {

            if ($where) {
                $where[] = 'AND';
            }

            $where[] = $atts['where'];
        }

        if (!$source) {
            $group_by = 'GROUP BY c_source';
        }

        if (array_key_exists('group', $atts) && strlen($atts['group'])) {

            $group_by = $atts['group'];
        }

        if ($where) {
            array_unshift($where, 'WHERE');
        }

        if (array_key_exists('group', $atts) && $atts['group'] === false) {
            $group_by = '';
        }

        $where[] = $group_by;

        // Add default order by c_source
        $where[] = 'ORDER BY c_source DESC';

        $query = sprintf(
            'SELECT 
									COUNT(*) AS sessions, 
									SUM(c_photos) AS photos, 
									SUM(c_prints) AS prints, 
									MIN(c_timestamp) AS start_date, 
									MAX(c_timestamp) AS end_date, 
									c_source AS source,
									now() AS execution_date
									FROM %s %s',
            $this->get_table(),
            implode(' ', $where)
        );

        if (PG_LOG_VERBOSE) {
            Helper::log(sprintf('[ XML-Statistic ] SQL Debug: %s ', $query));
        }

        $result = $this->execute_query($query);
        $result = $this->prepare_result($result);


        return $result;
    }

    private function get_time($atts)
    {

        $from = $atts['from'] ?? false;
        $to = $atts['to'] ?? false;

        if ($from && $to) {

            return sprintf("c_timestamp BETWEEN '%s' AND '%s'", esc_sql($from), esc_sql($to));
        }

        if ($from) {

            return sprintf("c_timestamp >= '%s'", esc_sql($from));
        }

        if ($to) {

            return sprintf("c_timestamp <= '%s'", esc_sql($to));
        }

        return '';
    }

    private function get_source($atts)
    {

        $source = $atts['source'] ?? '';

        if (!$source) {
            return '';
        }

        return sprintf("c_source like '%s'", '%'.esc_sql($source).'%');
    }

    private function execute_query($query)
    {

        global $wpdb;

        $results = $wpdb->get_results($query);

        if (!is_array($results)) {
            return [];
        }

        return $results;
    }

    private function prepare_result($result)
    {

        if (!$result) {
            return $result;
        }

        foreach ($result as & $res) {

            if (!$res->source) {
                continue;
            }

            $res->source = explode('/', $res->source);
            $res->source = end($res->source);
        }

        return $result;
    }

    private function optimize_previous_stats($current_stats, $previous_stats)
    {

        if (!$current_stats) {
            return $previous_stats;
        }

        if (!$previous_stats) {
            return $previous_stats;
        }

        if ($current_stats === $previous_stats) {
            return $previous_stats;
        }

        $clean_result = [];
        foreach ($current_stats as $stat) {

            foreach ($previous_stats as $prev_stat) {

                if ($stat->source === $prev_stat->source) {
                    $clean_result[$stat->source] = $prev_stat;
                    break;
                }

            }
        }

        return $clean_result;
    }
}
