<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.codevision.io
 * @since      1.0.0
 *
 * @package    Photobooth_Gallery
 * @subpackage Photobooth_Gallery/public
 */

use Photobooth\Elementor\Importer;
use Photobooth\FileManagement;
use Photobooth\FileUploader;
use Photobooth\Helper;
use Photobooth\Plugins\LockMixin;
use Photobooth\Ramsey\Uuid\Uuid;
use Photobooth\Renderer;
use Photobooth\SocialMedia;
use Photobooth\Transients;
use Photobooth\Symfony\Component\Stopwatch\Stopwatch;

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Photobooth_Gallery
 * @subpackage Photobooth_Gallery/public
 * @author     codevision.io <info@codevision.io>
 */
class Photobooth_Gallery_Public
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    /**
     * @var Renderer
     */
    private $render;

    /**
     * @var FileManagement
     */
    private $file_management;

    /**
     * @var SocialMedia
     */
    private $social_manager;

    /**
     * Initialize the class and set its properties.
     *
     * @param string $plugin_name The name of the plugin.
     * @param string $version The version of this plugin.
     *
     * @since    1.0.0
     *
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        $this->render = new Renderer();
        $this->social_manager = new SocialMedia();
        $this->file_management = new FileManagement();
    }

    function shortcode_callback_view($atts)
    {

        $atts = shortcode_atts(array(
            'settings' => 'Default Settings',
            'the-code' => false,
            'anchor' => sha1(uniqid('pb-uid', true)),
            'scrolling' => true,
        ),
            $atts,
            'view-photobooth-gallery');

        // make sure we deal with sluggable anchor tags
        $atts['anchor'] = sanitize_title($atts['anchor']);

        // Get the code from the payload (if any)
        $payload = SocialMedia::decode(get_query_var('pg-payload'));

        if ($payload) {
            $atts = array_merge($atts, $payload);
        }

        return $this->render_enter_the_code($atts);
    }

    /**
     * Generate the Code View with the current settings
     *
     * @param array $atts
     *
     * @return string
     */
    private function render_enter_the_code($atts = array())
    {

        $acf = Helper::load_setting($atts['settings']);

        $action = 'pg-enter-the-code';
        $template = 'enter-the-code';
        $preview = $atts['no-code'] ?? false;
        $preview = $preview && ($acf['is_preview_gallery'] ?? false);

        // In preview mode the Code is equal to the settings
        if ($preview === true) {
            $atts['the-code'] = $atts['settings'];
        }

        // Create the nonce to protect agains csrf
        $csrf = [
            'name' => 'csrf',
            'value' => wp_create_nonce($action),
        ];

        // Define the anchor and callback url
        $form = [
            'id' => $atts['anchor'],
            'callback' => admin_url('admin-ajax.php'),
            'settings' => $atts['settings'],
            'action' => $action,
        ];

        // Display error message for the current admin user if no settings are available
        if (!$acf && current_user_can('manage_options')) {

            $template = 'error-settings-missing';

            $msg = sprintf(
                __(
                    '<strong>Settings are missing!</strong> Please add at least one configuration <a href=%s>here</a>',
                    'photobooth-gallery'
                ),
                '"/wp-admin/'.Photobooth_Gallery::SETTINGS_SLUG.'"'
            );

            if ($atts['settings']) {
                $msg = sprintf(
                    __(
                        '<strong>Setting %s could not be found!</strong> Please make sure, that the given name matches at least one configuration <a href=%s>here</a>',
                        'photobooth-gallery'
                    ),
                    $atts['settings'],
                    '"/wp-admin/'.Photobooth_Gallery::SETTINGS_SLUG.'"'
                );
            }

            $acf = [
                'error_settings_missing' => $msg,
                'is_slick_gallery' => false,
                'is_code_in_url_allowed' => false,
            ];
        }

        if ($blocked = $this->is_banned()) {
            $template = 'error-settings-missing';

            $acf = [
                'error_settings_missing' => $blocked,
            ];
        }

        if ($acf['is_code_in_url_allowed'] && $the_code = get_query_var('pg-code', '')) {
            $atts['the-code'] = sanitize_text_field($the_code);
        }

        $the_code = $atts['the-code'] ?? false;

        // Allow the usage of a pre-entered code
        if (is_string($the_code)) {
            $form['code'] = $the_code;
        }

        if (is_string($atts['scrolling'])) {
            $atts['scrolling'] = $atts['scrolling'] === 'true';
        }

        // Combine everything and send it to the view
        $data = [
            'shortcode' => $atts,
            'acf' => $acf,
            'form' => $form,
            'csrf' => $csrf,
        ];

        if ($acf['is_slick_gallery']) {
            wp_enqueue_script('slick-gallery');
        }

        return $this->render->view($template, $data);
    }

    private function is_banned()
    {

        return get_transient('user_banned_'.$this->get_user_identification());
    }

    private function get_user_identification()
    {

        $ip_address = array_key_exists('REMOTE_ADDR', $_SERVER) ? $_SERVER['REMOTE_ADDR'] : 'no-ip-found';

        if ($ip_address === 'no-ip-found') {
            Helper::log(
                sprintf(
                    '[ Generate Unique User Identification ] Could not retrieve IP Address from Request, using generic user identification!'
                ),
                'error'
            );
        }

        return sha1(Photobooth_Gallery::PREFIX.$ip_address);
    }

    function shortcode_callback_preview($atts)
    {

        $atts = shortcode_atts(array(
            'settings' => 'Default Settings',
            'the-code' => false,
            'anchor' => sha1(uniqid('pb-uid', true)),
            'scrolling' => true,
        ),
            $atts,
            'preview-photobooth-gallery');

        // make sure we deal with sluggable anchor tags
        $atts['anchor'] = sanitize_title($atts['anchor']);

        // Disable the Code completely
        $atts['no-code'] = true;

        return $this->render_enter_the_code($atts);
    }

    function shortcode_callback($atts)
    {

        $atts = shortcode_atts(array(
            'settings' => 'Default Settings',
            'the-code' => false,
            'anchor' => sha1(uniqid('pb-uid', true)),
            'scrolling' => true,
        ),
            $atts,
            'photobooth-gallery');

        // make sure we deal with sluggable anchor tags
        $atts['anchor'] = sanitize_title($atts['anchor']);

        return $this->render_enter_the_code($atts);
    }

    function doctype_opengraph($output)
    {

        $payload = SocialMedia::decode(get_query_var('pg-payload'));

        if (!$payload) {
            return $output;
        }

        $output = $output.' xmlns:og="http://opengraphprotocol.org/schema/"';
        $output = $output.' xmlns:fb="http://www.facebook.com/2008/fbml"';

        return $output;
    }

    function webhooks()
    {

        $this->handle_download_file();

        $this->handle_download_site();

        $this->handle_statistic_data();

        $this->handle_external_scan_request();

        $uploader = new FileUploader();
        $uploader->setFileManagement($this->file_management);

        $uploader->handle_upload_file();

        $this->social_manager->view();

    }

    private function handle_download_file()
    {

        $force_download = get_query_var('pg-download');

        if (!$force_download) {
            return;
        }

        $data = base64_decode($force_download);

        if (!$data) {
            return;
        }

        $data = json_decode($data, true);

        if (!is_array($data)) {
            return;
        }

        // should always be 3 entries
        if (sizeof($data) !== 3) {
            $data[] = '';
        }

        list($code, $file, $hash) = array_values($data);

        if (!file_exists($file)) {
            return;
        }

        $file_data = $this->file_management->handle_file_download($code, $file);

        if (!$file_data) {
            return;
        }

        $file_data['meta'] = $this->social_manager->render_social_tags($hash);

        echo $this->render->view('view-microsite', $file_data);
        exit;
    }

    private function handle_download_site()
    {

        $social_share_code = get_query_var('pg-social-share');

        if (!$social_share_code) {
            return;
        }

        // get correct download link
        $data = $this->get_share_data($social_share_code);
        $error = $data['error'] ?? false;

        if ($error === true) {
            header("HTTP/1.0 404 Not Found");
            echo $data['message'];
            exit();
        }

        switch ($social_share_code) {

            case 'download':
                $headline = __(
                    'Please wait until your download Link is ready, we will redirect you in a few seconds...',
                    'photobooth-gallery'
                );
                break;
            case 'download-all':
                $headline = __('Please wait until your zip archive has been generated...', 'photobooth-gallery');
                break;
            case 'e-mail':
                $headline = __(
                    'Please wait until your e-mail provider has been opened, after sending the e-mail you can safely close this window...',
                    'photobooth-gallery'
                );
                break;
            default:
                $headline = sprintf(
                    __('Please stand by, you will be redirected to %s in a second...', 'photobooth-gallery'),
                    ucfirst($data['class'])
                );
        }

        $html = array_merge([
            'title' => __('Generating your Download Link, please wait...', 'photobooth-gallery'),
            'description' => $headline,
            'action' => $social_share_code,
        ],
            $data);

        $css = [
            'src' => plugin_dir_url(PG_BOOTSTRAP_FILE).'cache/redirect/css/redirect.min.css',
        ];

        $js = [
            'src' => '',
        ];

        $service = [
            'csrf' => wp_create_nonce($data['action']),
            'url' => admin_url('admin-ajax.php'),
            'action' => 'pg-zip-status',
            'payload' => $_REQUEST['payload'] ?? false,
        ];

        echo $this->render->view('share-microsite', compact('html', 'css', 'js', 'service'));
        exit;
    }

    private function get_share_data($action)
    {

        $response = [
            'error' => true,
            'message' => __('Could not generate Sharing Link!', 'photobooth-gallery'),
        ];

        $payload = SocialMedia::decode($_REQUEST['payload'] ?? false);

        if (!$payload) {

            Helper::log(sprintf('[ Ajax Share ] No valid Payload submitted...'), 'error');

            return $response;
        }

        $the_code = $payload['the-code'];
        $the_id = $payload['the-id'] ?? 0;
        $set_id = sanitize_title($payload['settings']);

        $app_settings = Helper::load_all_settings();

        if (!$app_settings) {

            Helper::log(sprintf('[ Ajax Share ] No settings found, skipping (should never happen!)...'), 'error');

            return $response;
        }

        if (!array_key_exists($set_id, $app_settings)) {
            Helper::log(
                sprintf('[ Ajax Share ] Settings Key <%s> in Application settings not found!...', $set_id),
                'error'
            );

            return $response;
        }

        $settings = $app_settings[$set_id];

        $share = new SocialMedia($settings);

        $payload['network'] = sanitize_text_field($action);

        $share_data = $share->share($payload);

        if (!$share_data) {
            Helper::log(sprintf('[ Ajax Share ] Could not generate Share URL from payload...'), 'error');

            return $response;
        }

        Helper::log(
            sprintf(
                '[ Ajax Share ] Generated new Sharing URL <%s> for Code <%s> Media ID <%d>...',
                $share_data['redirect'],
                $the_code,
                $the_id
            )
        );

        return $share_data;
    }

    private function handle_statistic_data()
    {

        if (!is_page()) {
            return;
        }

        $restricted = get_field('pg_is_statistic_secured', 'option');

        if (!$restricted) {
            return;
        }

        $post_id = get_queried_object_id();

        $imported = get_post_meta($post_id, 'imported_by', true);

        if ($imported !== Importer::IDENTIFIER) {
            return;
        }

        $code = get_query_var('pg-code');

        if (!$code) {
            header("HTTP/1.0 404 Not Found");
            die();
        }

        $password = false;

        if (strpos($code, 'password/') !== false) {
            list($code, $password) = explode('password/', $code);
        }

        $code = str_replace('/', '', $code);

        if ($password !== false) {
            set_query_var('pg-code', $code);
        }

        $settings = Helper::load_all_settings();

        $real_code = urldecode($code);
        $real_code = explode('|', $real_code);
        $real_code = strtoupper(reset($real_code));

        foreach ($settings as $s) {

            $regex = '/^'.$s['regex'].'$/';

            if (preg_match($regex, $real_code) === 1) {
                return;
            }

        }

        $stored_pw = get_field('pg_statistic_password', 'option');

        if ($stored_pw !== $password) {
            header("HTTP/1.0 404 Not Found");
            die();
        }

        set_query_var('pg-code', $code);

        return;
    }

    public function handle_external_scan_request()
    {

        $auth = $_REQUEST['pg_csrf_token'] ?? false;

        // skip further inspection upon auth request not found
        if ($auth === false) {
            return;
        }

        $csrf_token = get_transient('pg-scan-request-csrf-token');

        if (!$csrf_token) {
            return;
        }

        if ($auth !== $csrf_token) {
            return;
        }

        $setting_id = get_query_var('pg-async-scan-request');

        if (!$setting_id) {
            return;
        }

        $setting_id = intval($setting_id);

        $settings = Helper::load_all_settings();

        $lock = LockMixin::createLock(Transients::CURRENTLY_SCANNING, 5 * MINUTE_IN_SECONDS);

        if ($lock->isAcquired() === true) {
            Helper::log('[ Scan Folder Request ] Currently running, skipping this request...');
            return;
        }

        $sw = new Stopwatch();

        foreach ($settings as $setting) {

            if ($setting['settings-post-id'] != $setting_id) {
                continue;
            }

            $sw->start('async-scanner');

            $uuid = uniqid();

            Helper::log(sprintf('[ Async Scan Request ] Spawning new Image Scan with ScanID %s', $uuid));

            $this->file_management->scan_images($setting);

            Helper::log(
                sprintf('[ Async Scan Request ] Finished Image ScanID %s after %s', $uuid, $sw->stop('async-scanner'))
            );
        }

        header("HTTP/1.0 204 No Content");
        exit;
    }

    function add_meta_tags()
    {

        $tag_output = $this->social_manager->render_social_tags();

        if (!$tag_output) {
            return;
        }

        echo $tag_output;
    }

    /**
     * Handles "wrong" get parameters for Photobooth Codes with the pg-code parameter
     *
     * like /pg-code/?XYZ12345
     *
     * created by url shortening services
     *
     * @param $wp_query WP_Query
     */
    function handle_incorrect_pg_codes($wp_query)
    {

        if (!$wp_query->is_main_query()) {
            return;
        }

        // Only if the given pg-code parameter is in the query ...
        if (!array_key_exists('pg-code', $wp_query->query)) {
            return;
        }

        // ... and does not have a value ...
        if ($wp_query->get('pg-code')) {
            return;
        }

        // ... will we check the request for a malformed pg-code parameter
        $first_param = reset($_REQUEST);

        // No Parameter added
        if ($first_param === false) {
            return;
        }

        // If the first parameter contains a value, we skip here
        if (strlen($first_param)) {
            return;
        }

        // We are interested in the key of the very first parameter which is in fact the value
        // we are looking for
        $keys = array_keys($_REQUEST);
        $key = reset($keys);

        if (!$key) {
            return;
        }

        $wp_query->set('pg-code', $key);
    }

    function ajax_mp4_mailer()
    {

        check_ajax_referer('pg-mp4-mailer', 'csrf');

        $response = [
            'error' => true,
            'message' => __('Could not send E-Mail!', 'photobooth-gallery'),
        ];

        $payload = SocialMedia::decode($_REQUEST['payload'] ?? false);

        if (!$payload) {

            Helper::log(sprintf('[ Ajax iOS MP4 Download ] No valid Payload submitted...'), 'error');

            wp_send_json($response);
            exit;
        }

        $the_id = $payload['the-id'] ?? 0;
        $mail = $_REQUEST['mail'] ?? false;
        $set_id = sanitize_title($payload['settings']);

        if ($mail === false) {
            Helper::log(sprintf('[ Ajax iOS MP4 Download ] Mail Address not submitted...'), 'error');

            wp_send_json($response);
            exit;
        }

        $mail = sanitize_text_field($mail);

        $app_settings = Helper::load_all_settings();

        if (!$app_settings) {

            Helper::log(
                sprintf('[ Ajax iOS MP4 Download ] No settings found, skipping (should never happen!)...'),
                'error'
            );

            wp_send_json($response);
            exit;
        }

        if (!array_key_exists($set_id, $app_settings)) {
            Helper::log(
                sprintf('[ Ajax iOS MP4 Download ] Settings Key <%s> in Application settings not found!...', $set_id),
                'error'
            );

            wp_send_json($response);
            exit;
        }

        $settings = $app_settings[$set_id];

        $share = new SocialMedia($settings);

        $payload['network'] = 'download';

        $share_data = $share->share($payload);

        if (!$share_data) {
            Helper::log(sprintf('[ Ajax iOS MP4 Download ] Could not generate Share URL from payload...'), 'error');

            wp_send_json($response);
            exit;
        }

        // Send Email to User
        $sender = $settings['ios_video_share_sender_email'] ?? false;

        // Generate fallback email address
        if (!$sender) {
            $sender = 'info@'.parse_url(site_url(), PHP_URL_HOST);
            Helper::log(
                sprintf('[ Ajax iOS MP4 Download ] Could not find sender address in config using %s...', $sender),
                'error'
            );
        }

        $fn = function () use ($sender) {

            return $sender;
        };

        add_filter('wp_mail_from', $fn);

        wp_mail(
            $mail,
            __('Here is your Video', 'photobooth-gallery'),
            sprintf(__('Please find attached the Video you requested from %s', 'photobooth-gallery'), site_url()),
            '',
            [
                get_attached_file($the_id),
            ]
        );

        remove_filter('wp_mail_from', $fn);

        wp_send_json([
            'error' => false,
            'message' => __(
                'Your Request has been processed successfully. You should receive an email from us in no time. Please also consider checking your spam folder!',
                'photobooth-gallery'
            ),
        ]);
    }

    function ajax_delete_images()
    {

        check_ajax_referer('pg-admin-delete', 'csrf');

        $response = [
            'error' => true,
            'message' => __('Could not remove Images!', 'photobooth-gallery'),
        ];


        $images = $_REQUEST['images'] ?? false;
        $code = sanitize_text_field($_REQUEST['code'] ?? false);

        if (!$code) {

            Helper::log(sprintf('[ Admin Remove Images ] No valid Code submitted...'), 'error');

            wp_send_json($response);
            exit;
        }

        if (!$images) {

            Helper::log(sprintf('[ Admin Remove Images ] No images submitted...'), 'error');

            wp_send_json($response);
            exit;
        }

        if (!is_array($images)) {

            Helper::log(sprintf('[ Admin Remove Images ] Images are not an array!'), 'error');

            wp_send_json($response);
            exit;
        }

        $app_settings = Helper::load_all_settings();

        if (!$app_settings) {

            Helper::log(
                sprintf('[ Admin Remove Images ] No settings found, skipping (should never happen!)...'),
                'error'
            );

            wp_send_json($response);
            exit;
        }

        $setting = array_filter(
            $app_settings,
            function ($item) use ($code) {

                return $item['admin_code'] === $code;
            }
        );

        if (!$setting) {

            Helper::log(
                sprintf('[ Admin Remove Images ] No matching settings for admin code %s found...', $code),
                'error'
            );

            wp_send_json($response);
            exit;
        }

        $done = 0;
        $errors = 0;
        foreach ($images as $image) {

            $target = get_post_meta($image, 'real_filepath', true);

            if (!wp_delete_attachment($image, true)) {

                Helper::log(sprintf('[ Admin Remove Images ] Could not delete Attachment %d ...', $image), 'error');

                $errors++;
                continue;
            }

            if (!$target) {
                Helper::log(
                    sprintf(
                        '[ Admin Remove Images ] Could not get Target Filepath from Attachment %d Meta data ...',
                        $image
                    ),
                    'error'
                );

                $errors++;
                continue;
            }

            if (file_exists($target) && !unlink($target)) {
                Helper::log(sprintf('[ Admin Remove Images ] Could not remove orignal File %s...', $target), 'error');

                $errors++;
                continue;
            }

            $done++;
        }

        $msg = sprintf(
            __('Successfully removed %d/%d images (%d erroneous)', 'photobooth-gallery'),
            $done,
            sizeof($images),
            $errors
        );

        Helper::log(sprintf('[ Admin Remove Images ] %s...', $msg));

        $response = [
            'class' => 'alert alert-success',
            'message' => __($msg, 'photobooth-gallery'),
        ];

        wp_send_json($response);

        exit;
    }

    function ajax_generate_zip_archive()
    {

        check_ajax_referer('pg-download-zip', 'csrf');

        $response = [
            'error' => true,
            'message' => __('There was an error while trying to generate the zip archive.', 'photobooth-gallery'),
        ];

        $payload = SocialMedia::decode($_REQUEST['payload'] ?? false);

        if (!$payload) {

            Helper::log(sprintf('[ Generate Zip Archive ] No valid Payload submitted...'), 'error');

            wp_send_json($response);
            exit;
        }

        $the_code = $payload['the-code'];
        $set_id = sanitize_title($payload['settings']);
        $uuid = $_REQUEST['uuid'] ?? false;

        $app_settings = Helper::load_all_settings();

        if (!$app_settings) {

            Helper::log(
                sprintf('[ Generate Zip Archive ] No settings found, skipping (should never happen!)...'),
                'error'
            );

            wp_send_json($response);
            exit;
        }

        if (!array_key_exists($set_id, $app_settings)) {
            Helper::log(
                sprintf('[ Generate Zip Archive ] Settings Key <%s> in Application settings not found!...', $set_id),
                'error'
            );

            wp_send_json($response);
            exit;
        }

        $settings = $app_settings[$set_id];

        $query = new WP_Query([
            'post_type' => 'attachment',
            'post_status' => 'any',
            'posts_per_page' => -1,
            'orderby' => 'ID',
            'meta_query' => [
                [
                    'key' => 'real_filename',
                    'value' => $the_code,
                    'compare' => 'LIKE',

                ],
            ],
        ]);

        if (!$query->have_posts()) {

            Helper::log(
                sprintf(
                    '[ Generate Zip Archive ] No Media Data for Code <%s> and Setting ID <%s> found...',
                    $the_code,
                    $set_id
                ),
                'error'
            );

            wp_send_json($response);
            exit;
        }

        if ($uuid) {
            set_transient($uuid.'-files', $query->found_posts, 60 * MINUTE_IN_SECONDS);
        }


        //$query, $the_code, $settings, $manager
        $manager = $this->social_manager;
        $response = $this->file_management->get_zip_archive(
            compact('query', 'the_code', 'settings', 'manager', 'uuid')
        );

        wp_send_json($response);
    }

    function ajax_zip_status()
    {

        check_ajax_referer('pg-download-zip', 'csrf');

        $uuid = $_REQUEST['uuid'] ?? false;
        $action = 'pg-download-zip';

        if (!$uuid) {
            $uuid = Uuid::uuid4()
                ->toString();
            wp_send_json(compact('uuid', 'action'));
            exit;
        }

        $file_count = get_transient($uuid.'-files');
        $file_index = get_transient($uuid.'-file-index');

        wp_send_json(compact('uuid', 'file_count', 'file_index', 'action'));
    }

    function ajax_log_action()
    {

        check_ajax_referer('pg-log-action', 'csrf');

        $response = [
            'error' => true,
        ];

        $action = $_REQUEST['pg-action'] ?? false;
        $code = $_REQUEST['pg-code'] ?? false;
        $config = $_REQUEST['pg-config'] ?? false;
        $additional_data = $_REQUEST['pg-data'] ?? '';

        if (!$action) {
            wp_send_json($response);
            exit;
        }

        if (!$code) {
            wp_send_json($response);
            exit;
        }

        if (!$config) {
            wp_send_json($response);
            exit;
        }

        $data = stripslashes(json_encode($additional_data));

        Helper::u_log(
            sprintf(
                '[ Ajax Log Action ] Action: %s, Code: %s, Config: %s, Additional Data: %s',
                $action,
                $code,
                $config,
                $data
            )
        );

        $this->social_manager->log(compact('action', 'code', 'config', 'additional_data'));

        $response['error'] = false;
        wp_send_json($response);
    }

    function ajax_validate_code()
    {

        check_ajax_referer('pg-enter-the-code', 'csrf');

        $response = [
            'error' => true,
            'content' => false,
        ];

        if (!array_key_exists('the-code', $_REQUEST)) {
            wp_send_json($response);
            exit;
        }

        $page = $_REQUEST['page'] ?? 0;
        $admin_code = $_REQUEST['admin_code'] ?? false;
        $settings = $_REQUEST['settings'] ?? false;

        $data = $this->render_code_content($_REQUEST['the-code'], $page, $admin_code, $settings);

        if (is_array($data)) {
            [$content, ] = array_values($data);
        } else {
            $content = false;
        }

        if ($blocked = $this->is_banned()) {
            $response['message'] = $blocked;
        }

        if ($content === false) {
            wp_send_json($response);
            exit;
        }

        $response['content'] = $content;
        $response['error'] = false;

        Helper::u_log(
            sprintf('[ Ajax Validate Code ] The entered Code %s was successfully validated...', $_REQUEST['the-code'])
        );

        wp_send_json($response);
    }

    private function render_code_content(
        $the_code,
        $page = 1,
        $admin_code = false,
        $the_setting = false,
        $image_scan = false,
        $type = false
    ) {

        if ($image_scan) {
            Helper::log(sprintf('[ Render The Code ] Second try to get the image after scanning for images...'));
        }


        if ($this->is_banned()) {

            Helper::log(sprintf('[ Render The Code ] blocked access to Code input, because the user is blocked!...'));

            return false;
        }

        if (!$the_code) {

            Helper::log(sprintf('[ Render The Code ] No code entered...'));

            return false;
        }

        if (!is_string($the_code)) {

            Helper::log(sprintf('[ Render The Code ] Code is not a string...'), 'error');

            return false;
        }

        $the_code = sanitize_text_field($the_code);
        $page = sanitize_text_field($page);
        $admin_code = sanitize_text_field($admin_code);
        $the_setting = sanitize_text_field($the_setting);

        if ($type === false) {
            $type = 'default';
        }

        $template = 'photobooth-portfolio';

        if (!strlen($the_code)) {

            Helper::log(sprintf('[ Render The Code ] Code could not be converted...'), 'error');

            return false;
        }

        $page = max(1, intval($page));

        //Find the first config matching the regex
        $settings = Helper::load_all_settings();

        if (!$settings) {

            Helper::log(sprintf('[ Render the Code ] No settings found, skipping (should never happen!)...'), 'error');

            return false;
        }

        $_setting = [];
        $setting_name = '';

        foreach ($settings as $s) {

            // No regex has been entered
            if (!$s['regex']) {
                continue;
            }

            $regex = '/^'.$s['regex'].'$/';

            if ($the_code !== $the_setting && preg_match($regex, $the_code) !== 1) {
                continue;
            }

            if ($the_setting !== $s['settings-name']) {
                continue;
            }

            $_setting = $s;
            $setting_name = $s['settings-name'];
            break;
        }

        $settings = $_setting;

        if (!$settings) {
            Helper::log(
                sprintf('[ Render the Code ] Could not find matching setting for submitted code <%s>...', $the_code),
                'error'
            );

            return $this->handle_ip_ban($settings);
        }

        Helper::log(
            sprintf(
                '[ Render The Code ] Trying to find Media data for Code <%s, %s> Page %d...',
                $the_code,
                $settings['regex'],
                $page
            )
        );

        $is_admin = $admin_code === $settings['admin_code'];

        $query = [
            'post_type' => 'attachment',
            'post_status' => 'any',
            'posts_per_page' => apply_filters(
                'photobooth/settings/images_per_page',
                $settings['images_per_page'],
                $setting_name,
                $type
            ),
            'paged' => $page,
            'meta_type' => 'DATETIME',
            'orderby' => 'meta_value_datetime',
            'meta_key' => 'file_date',
            'order' => 'DESC',
            'meta_query' => [
                'relation' => 'AND',
                [
                    'type_query' => [
                        'key' => 'type',
                        'value' => 'photobooth-gallery-image',
                        'compare' => '=',
                    ],
                    'code_query' => [
                        'key' => 'real_filename',
                        'value' => $the_code,
                        'compare' => 'LIKE',
                    ],
                ],
            ],
        ];

        if ($_setting['is_custom_image_order']) {

            $query['order'] = $_setting['order_type'];
            $query['meta_key'] = $_setting['order_by'];

            if (strpos($_setting['order_by'], 'date') === false) {
                $query['orderby'] = 'meta_value';
                unset($query['meta_type']);
            }
        }

        $is_preview = $_setting['is_preview_gallery'] ?? false;
        $is_preview = $is_preview && $the_setting === $the_code;

        if ($is_preview) {

            Helper::log(sprintf('[ Render The Code ] Preview Mode for <%s> Page %d enabled...', $setting_name, $page));

            $query = [
                'post_type' => 'attachment',
                'post_status' => 'any',
                'posts_per_page' => apply_filters('photobooth/settings/images_per_page', $settings['images_per_page'], $setting_name, $type),
                'paged' => $page,
                'orderby' => 'meta_value_num',
                'meta_key' => 'file_date',
                'order' => 'DESC',
                'pg-settings-id' => $_setting['settings-post-id'],
            ];

        }

        $query = apply_filters('photobooth/render_code_content/query_args', $query, $settings, $the_code, $type);

        $query = new WP_Query($query);

        /**
         * @var WP_Query $query
         */
        $query = apply_filters('photobooth/render_code_content/query', $query, $settings, $the_code, $type);

        if (PG_LOG_VERBOSE) {
            Helper::log($query->request, 'DEBUG Query');
        }

        if ($page === 1 && !$query->have_posts()) {

            Helper::log(
                sprintf(
                    '[ Render The Code ] No Media Data for Code <%s, %s> Page %d found...',
                    $the_code,
                    $settings['regex'],
                    $page
                )
            );

            Helper::log('[ Render The Code ] Media Library scan started by missing media data');

            if ($image_scan) {
                return $this->handle_ip_ban($settings);
            }

            $this->file_management->scan_upload_folder();

            return $this->render_code_content($the_code, $page, $admin_code, $the_setting, true, $type);
        }

        $is_shareable = $settings['is_social_media'];

        $media = [];
        $social = new SocialMedia($settings);
        $social->setResultSize($query->found_posts);

        $items = $query->posts;

        $items = apply_filters('photobooth/render_code_content/items', $items, $settings, $the_code, $type);

        foreach ($items as $post) {

            $url = wp_get_attachment_url($post->ID);

            if (!$url) {
                continue;
            }

            $mime_type = 'image';

            if ($post->post_mime_type) {
                $mime_type = explode('/', $post->post_mime_type);
                $mime_type = strtolower(reset($mime_type));
            }

            $item = [
                'id' => $post->ID,
                'uuid' => uniqid(),
                'full' => $url,
                'link' => '#',
                'is_animated' => strpos($post->post_mime_type, 'gif') !== false,
                'css_class' => $mime_type.' '.sanitize_title($post->post_mime_type),
                'mime_type' => $post->post_mime_type,
                'template' => $mime_type.'.twig',
                'is_preview' => false,
                'payload' => Helper::encrypt(
                    json_encode([
                        'the-id' => $post->ID,
                        'the-code' => $the_code,
                        'settings' => $setting_name,
                    ])
                ),
            ];

            if ($mime_type === 'image') {

                $item['preview'] = $item['full'];

                $thumbnail = wp_get_attachment_image_src($post->ID, 'pg-thumbnail');
                $item['thumbnail'] = reset($thumbnail);

                if (!$item['is_animated']) {
                    $preview = wp_get_attachment_image_src($post->ID, 'pg-preview');
                    $item['preview'] = reset($preview);
                }
            }


            if ($is_shareable) {
                $item['buttons'] = $social->get_social_media($item);
            }

            if ($is_admin && $settings['is_show_filenames']) {
                $item['filename'] = get_post_meta($post->ID, 'real_filename', true);
            }

            $item = apply_filters('photobooth/render_code_content/twig_data', $item, $settings);

            if (apply_filters('photobooth/render_code_content/allow_item', true, $item, $settings) === false) {
                continue;
            }

            $media[] = $item;
        }

        Helper::u_log(
            sprintf(
                '[ Render The Code ] Found %d media Images for <%s, %s>...',
                sizeof($media),
                $the_code,
                $settings['regex']
            )
        );

        $first = reset($media);

        if ($first === false) {
            $first = [];
        }

        $first['is_preview'] = true;

        $data = [
            'version' => PHOTOBOOTH_GALLERY_VERSION,
            'form' => [
                'the_code' => $the_code,
                'the_config' => $setting_name,
                'auto_scroll' => $settings['is_auto_scroll'],
                'page' => $page,
            ],
            'settings' => [
                'auto_scroll' => $settings['is_auto_scroll'],
                'ad_seconds_to_show' => $settings['ad_seconds_to_show'],
                'ad_frequency' => $settings['ad_distance'],
            ],
            'i18n' => [
                'reload_button' => __('Enter a new Code', 'photobooth-gallery'),
                'admin_button' => __('Remove selected Items', 'photobooth-gallery'),
                'remove_label' => __('Remove Item', 'photobooth-gallery'),
            ],
            'item' => $first,
            'masonry' => [
                'is_admin' => $is_admin,
                'media' => $media,
            ],
            'pagination' => $this->get_pagination($settings, $query, $page),
        ];

        if ($settings['is_social_media']) {
            $data['social'] = [
                'headline' => $settings['social_media_headline'].' ',
                'custom_output' => $settings['social_media_output'],
            ];
        }

        $template = apply_filters('photobooth/render_code_content/twig_template', $template, $data, $settings, $type);

        $view = $this->render->view($template, $data);

        return compact('view', 'data');
    }

    private function handle_ip_ban($settings)
    {

        if (!$settings['is_ip_ban_enabled']) {
            return false;
        }

        $user_tries = get_transient('user_tries_'.$this->get_user_identification());

        if (!$user_tries) {
            $user_tries = 0;
        }

        $user_tries++;

        set_transient(
            'user_tries_'.$this->get_user_identification(),
            $user_tries,
            $settings['ip_ban_expire_in_seconds']
        );

        Helper::log(
            sprintf(
                '[ Handle IP Ban ] User %s tried to guess Code the %d/%d time (Settings: %s)...',
                $this->get_user_identification(),
                $user_tries,
                $settings['maxium_retries'],
                $settings['settings-name']
            ),
            'warning'
        );

        if ($user_tries < $settings['maxium_retries']) {
            return false;
        }

        Helper::u_log(
            sprintf(
                '[ Handle IP Ban ] User %s will be blocked for %d seconds (Settings: %s) ...',
                $this->get_user_identification(),
                $settings['ip_ban_expire_in_seconds'],
                $settings['settings-name']
            ),
            'error'
        );

        set_transient(
            'user_banned_'.$this->get_user_identification(),
            $settings['error_to_many_code_tries'],
            $settings['ip_ban_expire_in_seconds']
        );

        return false;
    }

    private function get_pagination($settings, $query, $page)
    {

        $size = abs($settings['pagination_size'] ?? 2);

        $config = [
            'base' => '%_%#',
            'format' => '?page=%#%',
            'current' => max(1, $page),
            'total' => $query->max_num_pages,
            'type' => 'array',
            'prev_next' => true,
            'prev_text' => '<i class="fa fa-backward"></i>',
            'next_text' => '<i class="fa fa-forward"></i>',
        ];

        $config['show_all'] = $size === 100;

        if (!$config['show_all']) {
            $size /= 2;
            $size = floor($size);

            $config['mid_size'] = max(1, $size - 1);
            $config['end_size'] = 1;
        }

        $pagination = paginate_links($config);

        if (!$pagination) {
            return $pagination;
        }

        if ($page > 0 && $settings['is_show_first_last_page']) {
            $page_link = str_replace('paged', 'page', get_pagenum_link(1, false));
            array_unshift(
                $pagination,
                sprintf(
                    '<a href="%s" class="prev page-numbers first-page" title="%s"><i class="fa fa-fast-backward"></i></a>',
                    esc_url($page_link),
                    __('Goto first page', 'photobooth-gallery')
                )
            );
        }

        if ($page != $query->max_num_pages && $settings['is_show_first_last_page']) {
            $page_link = str_replace('paged', 'page', get_pagenum_link($query->max_num_pages, false));
            array_push(
                $pagination,
                sprintf(
                    '<a href="%s" class="next page-numbers last-page" title="%s"><i class="fa fa-fast-forward"></i></a>',
                    esc_url($page_link),
                    __('Goto last page', 'photobooth-gallery')
                )
            );
        }

        return $pagination;
    }

    function add_query_vars($vars)
    {

        $vars[] = 'pg-payload';
        $vars[] = 'pg-webhook';
        $vars[] = 'pg-code';
        $vars[] = 'pg-download';
        $vars[] = 'pg-social-share';
        $vars[] = 'pg-upload';
        $vars[] = 'pg-async-scan-request';

        return $vars;
    }

    function enqueue_styles()
    {

        wp_enqueue_style(
            $this->plugin_name,
            plugin_dir_url(__FILE__)."css/{$this->plugin_name}.min.css",
            [],
            $this->version,
            'all'
        );
    }

    function enqueue_scripts()
    {

        wp_enqueue_script(
            $this->plugin_name,
            plugin_dir_url(__FILE__)."js/loader.min.js",
            ['jquery'],
            $this->version,
            false
        );

        wp_localize_script($this->plugin_name, 'pg_data', [
            'js-path' => plugin_dir_url(__FILE__).'js/',
            'css-path' => plugin_dir_url(__FILE__).'css/',
            'js-identifier' => $this->plugin_name.'-js',
            'ajaxurl' => admin_url('admin-ajax.php'),
            'baseurl' => site_url(),
            'pg-log-action' => wp_create_nonce('pg-log-action'),
            'pg-download-zip' => wp_create_nonce('pg-download-zip'),
            'pg-admin-delete' => wp_create_nonce('pg-admin-delete'),
            'pg-share-action' => wp_create_nonce('pg-share-action'),
            'pg-mp4-mailer' => wp_create_nonce('pg-mp4-mailer'),
            'pg-next-page' => wp_create_nonce('pg-next-page'),
        ]);

        wp_localize_script($this->plugin_name, 'pg_i18n', [
            'confirm_admin_delete' => __(
                'Do you really want to remove all <%- size %> marked Media Files? This cannot be undone!',
                'photobooth-gallery'
            ),
        ]);
    }

    function allow_images_by_settings($clauses, $wp_query)
    {

        $id = $wp_query->query['pg-settings-id'] ?? false;

        if (!$id) {
            return $clauses;
        }


        $id = intval(esc_sql($id));

        global $wpdb;

        $table = $this->file_management->get_table();
        $join = 'pg_fs';

        $clauses['join'] .= sprintf(
            ' RIGHT JOIN %1$s %2$s ON ( %2$s.c_reference = %3$s.ID ) ',
            $table,
            $join,
            $wpdb->posts
        );
        $clauses['where'] .= sprintf(' AND %s.c_config_id = %d', $join, $id);

        return $clauses;
    }

    public function init()
    {

        if (is_admin()) {
            return;
        }

        add_shortcode('photobooth-gallery', [$this, 'shortcode_callback']);
        add_shortcode('view-photobooth-gallery', [$this, 'shortcode_callback_view']);
        add_shortcode('preview-photobooth-gallery', [$this, 'shortcode_callback_preview']);

        do_action('photobooth/init/public');
    }
}
