<?php

use Elementor\Plugin;
use Photobooth\Helper;

class Pbg_Elementor
{

    const ELEMENTOR = 'disable-shipped-elementor';

    private $request_disabled = false;

    /**
     * Pbg_Elementor constructor.
     */
    public function __construct()
    {
        add_action('plugins_loaded', [$this, 'handle_shipped_plugins']);

        include_once(ABSPATH.'wp-admin/includes/plugin.php');
    }

    /**
     *
     * Disable bundled plugins before enabling our own
     *
     * @return $this
     */
    public function handle_shipped_plugins()
    {

        $bundles = $this->get_bundles();

        foreach ($bundles as $bundle => $name) {

            if (is_plugin_active($bundle)) {
                update_option('pbg-elementor-active', 'active');
            } else {
                delete_option('pbg-elementor-active');
            }

        }

        return $this;
    }

    /**
     * The currently shipped bundle plugins with this plugin
     * @return array
     */
    private function get_bundles()
    {

        return [
            'elementor/elementor.php' => 'Elementor',
        ];
    }

    /**
     * This message will interrupt any plugin enable process for bundled plugins to ensure
     * stability
     */
    public function will_bundle_be_enabled()
    {

        $action = $_GET['action'] ?? false;

        if (!$action) {
            return $this;
        }

        if ($action !== 'activate') {
            return $this;
        }

        $plugin = $_GET['plugin'] ?? false;

        if (!$plugin) {
            return $this;
        }

        $bundles = $this->get_bundles();

        $selected_bundle = false;

        foreach ($bundles as $bundle => $name) {

            if (strpos($plugin, $bundle) === false) {
                continue;
            }

            $selected_bundle = $name;
            break;
        }

        if ($selected_bundle === false) {
            return $this;
        }

        update_option('pbg-elementor-active', 'active');

        $this->request_disabled = true;

        return $this;
    }

    public function init_bundle()
    {

        if (Helper::flag_exists(self::ELEMENTOR)) {
            return $this;
        }

        if ($this->request_disabled) {
            return $this;
        }

        define('PG_ELEMENTOR_DIR', __DIR__.'/elementor');

        if (get_option('elementor_tracker_notice') !== '1') {
            update_option('elementor_tracker_notice', '1');
        }

        if (get_option('elementor_pro_tracker_notice') !== '1') {
            update_option('elementor_pro_tracker_notice', '1');
        }


        add_action('elementor/elements/categories_registered', [$this, 'elementor_widget_categories']);
        add_action('elementor/widgets/register', [$this, 'register_widgets']);
        add_action('elementor/editor/after_enqueue_styles', [$this, 'register_elementor_styles']);
        add_action('elementor/frontend/after_enqueue_scripts', [$this, 'enqueue_elementor_scripts']);
        add_action('elementor/frontend/before_register_styles', function () {
            Plugin::$instance->experiments->remove_feature('e_optimized_css_loading');
        });

        if (get_option('pbg-elementor-active') !== 'active') {


            add_action('admin_menu', [$this, 'remove_menu_pages'], 100);
            add_action('wp_dashboard_setup', [$this, 'remove_elementor_overview'], 40);

            include_once(__DIR__.'/elementor/elementor.php');
        }


        return $this;
    }

    function remove_elementor_overview()
    {

        remove_meta_box('e-dashboard-overview', 'dashboard', 'normal');
    }

    function remove_menu_pages()
    {

        remove_menu_page('edit.php?post_type=elementor_library');
        remove_menu_page('elementor');
    }

    function elementor_widget_categories($elements_manager)
    {

        $elements_manager->add_category(
            'pg-photobooth-gallery',
            [
                'title' => __('Photobooth Gallery', 'photobooth-gallery'),
                'icon' => 'fa fa-camera-retro',
            ]
        );

    }

    function register_elementor_styles()
    {

        wp_enqueue_style(
            'photobooth-elementor',
            plugin_dir_url(__FILE__).'elementor/assets/css/elementor.min.css',
            [],
            PHOTOBOOTH_GALLERY_VERSION,
            'all'
        );
        wp_enqueue_style(
            'premium-pro',
            plugin_dir_url(__FILE__).'elementor/assets/css/premium-addons.css',
            array(),
            PHOTOBOOTH_GALLERY_VERSION,
            'all'
        );
        wp_enqueue_style('premium-pro-elements', plugin_dir_url(__FILE__).'elementor/assets/css/pa-elements.css');
    }

    function register_widgets()
    {

        foreach (glob(__DIR__.'/elementor/widgets/*.php') as $widget) {
            require_once $widget;
        }

        \Elementor\Plugin::instance()->widgets_manager->register(new Photobooth_Counter_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register(new Photobooth_Image_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register(new Photobooth_Progressbar_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register(new \Elementor\Premium_Charts());
    }

    function enqueue_elementor_scripts()
    {

        wp_enqueue_script(
            'photobooth-elementor',
            plugin_dir_url(__FILE__).'elementor/assets/js/elementor.min.js',
            ['jquery'],
            PHOTOBOOTH_GALLERY_VERSION,
            true
        );
    }

    function __i18n()
    {

        return [
            __('Photobooth profile', 'photobooth-gallery'),
            __('Number of sequences', 'photobooth-gallery'),
            __('Number of prints', 'photobooth-gallery'),
            __('The Photobooth Gallery Statistics Page', 'photobooth-gallery'),
        ];
    }

}

$elementor = new Pbg_Elementor();

$elementor->handle_shipped_plugins()
    ->will_bundle_be_enabled()
    ->init_bundle();




