<?php
/**
 * @license proprietary?
 *
 * Modified by sebastian rassmann on 06-February-2024 using Strauss.
 * @see https://github.com/BrianHenryIE/strauss
 */

namespace Codevision\Util;

use Codevision\Environment;
use DateTimeZone;
use Photobooth\Monolog\Formatter\LineFormatter;
use Photobooth\Monolog\Handler\RotatingFileHandler;
use Photobooth\Monolog\Handler\StreamHandler;
use Photobooth\Monolog\Level;
use Photobooth\Monolog\Logger;
use Photobooth\Monolog\Processor\IntrospectionProcessor;
use Photobooth\Monolog\Processor\PsrLogMessageProcessor;
use Photobooth\Monolog\Processor\UidProcessor;

class Helper
{

    private static $logger = [];

    public static function setEnvironment(Environment $environment, $name = 'default')
    {

        self::createLogger($name, $environment->getBasePath(), false);
    }

    private static function createLogger($name = 'default', $base_dir = false, $stdout = false)
    {

        if (array_key_exists($name, self::$logger)) {
            return self::$logger[$name];
        }

        $logger = new Logger($name);
        $logger->setTimezone(new DateTimeZone('Europe/Berlin'));

        $base_dir = self::ensureTrailingBackslash(self::baseLogDir($base_dir));

        $stream = new RotatingFileHandler($base_dir."log/{$name}.log", 5, Level::Debug, true);

        $stream
            ->setFormatter(new LineFormatter("%datetime% - %level_name% - %message%\n", "Y-m-d H:i:s"))
            ->pushProcessor(new IntrospectionProcessor('debug'))
            ->pushProcessor(new UidProcessor())
            ->pushProcessor(new PsrLogMessageProcessor());

        $logger->pushHandler($stream);

        if ($stdout) {
            $sysout = new StreamHandler('php://stdout', Level::Debug);

            $sysout
                ->setFormatter(new LineFormatter("%datetime% - %level_name% - %message%\n", "Y-m-d H:i:s"))
                ->pushProcessor(new IntrospectionProcessor('debug'))
                ->pushProcessor(new UidProcessor())
                ->pushProcessor(new PsrLogMessageProcessor());

            $logger->pushHandler($sysout);
        }

        self::$logger[$name] = $logger;

        return $logger;
    }

    public static function ensureTrailingBackslash(string $path, $dir_separator = '/'): string
    {

        return rtrim($path, $dir_separator).$dir_separator;
    }

    public static function baseLogDir($base_dir = false): string
    {

        if ($base_dir) {
            return rtrim($base_dir, '/').'/';
        }

        if (defined('LOG_DIR')) {
            $base_dir = rtrim(LOG_DIR, '/').'/';
        }

        if (!$base_dir) {
            $base_dir = plugin_dir_path(dirname(__FILE__));
        }

        return $base_dir;
    }

    public static function log($msg, $type = 'info', $name = 'default'): void
    {

        if (is_array($msg)) {
            $msg = json_encode($msg, JSON_PRETTY_PRINT);
        }

        if (array_key_exists('plugin', self::$logger)) {

            $logger = self::$logger['plugin'];

            $logger->$type($msg);

            return;
        }

        $logger = self::getLogger($name);

        $logger->$type($msg);
    }

    public static function getLogger($name = 'default', $base_dir = false, $stdout = false)
    {

        return self::createLogger($name, $base_dir, $stdout);
    }

    public static function filesize($file, $is_file = true)
    {

        if ($is_file) {
            $size = filesize($file);
        } else {
            $size = $file;
        }

        $units = array('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB');
        $power = $size > 0 ? floor(log($size, 1024)) : 0;

        return number_format($size / pow(1024, $power), 2, '.', ',').' '.$units[$power];
    }


}
