<?php
/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 26.10.2018
 * Time: 09:09
 */

namespace Codevision\Licensing;


use Codevision\Compatibility\CompatibilityCheck;
use Codevision\Environment;
use Codevision\Util\Constants;
use Codevision\Util\Helper;
use Codevision\View\TwigRenderer;
use YahnisElsts\PluginUpdateChecker\v5\PucFactory;

class Manager
{

    private static array $instances = [];

    private ?string $key = null;

    private ?string $namespace = null;

    private bool $valid = false;

    private ?string $api_server = null;
    private ?string $auth_key = null;

    private ?string $license_info_field = null;

    private ?string $license_key_name = null;
    private ?string $license_view_slug = null;


    private array $additional_data = [];

    private array $messages = [];

    private ?TwigRenderer $renderer = null;

    private ?Environment $env = null;

    private ?CompatibilityCheck $compatibility_check = null;

    private bool $trial_allowed;

    public function __construct($is_compatibility_check = true)
    {
        $this->trial_allowed = true;

        add_action('admin_init', [$this, 'wp_init']);

        if (!$is_compatibility_check) {
            return;
        }

        $this->compatibility_check = new CompatibilityCheck();
    }

    public static function getInstance(Environment $env = null): self
    {
        if ($env && array_key_exists($env->getBaseFile(), self::$instances)) {
            return self::$instances[$env->getBaseFile()];
        }

        $manager = new Manager(false);

        if ($env) {
            $manager->setEnv($env);
            self::$instances[$env->getBaseFile()] = $manager;

            Helper::setEnvironment($env, $env->getSlug());
        }

        return $manager;
    }

    public function setEnv($env): self
    {

        $this->env = $env;

        $this->compatibility_check?->setEnv($env);

        return $this;
    }

    public static function getInstanceWithCompatiblityCheck(Environment $env = null): self
    {

        if ($env && array_key_exists($env->getBaseFile(), self::$instances)) {
            return self::$instances[$env->getBaseFile()];
        }

        $manager = new Manager(true);

        if ($env) {
            $manager->setEnv($env);
            self::$instances[$env->getBaseFile()] = $manager;

            Helper::setEnvironment($env, $env->getSlug());
        }

        return $manager;
    }

    public function init(): self
    {

        if ($this->compatibility_check === null) {
            return $this;
        }

        $this->compatibility_check->init();

        return $this;
    }

    function wp_init()
    {

        if (!is_admin()) {
            return;
        }

        if (!$this->is_valid()) {
            add_action('admin_notices', [$this, 'missing_or_wrong_api_key']);
        }

        if ($this->is_expired() === true) {
            add_action('admin_notices', [$this, 'plugin_expired']);
        }

        if ($this->is_trial()) {
            add_action('admin_notices', [$this, 'trial_period_activated']);
        }

        add_filter('acf/save_post', [$this, 'update_license_data'], 20);
        add_filter('acf/prepare_field/key='.$this->license_info_field, [$this, 'render_license_info']);
        add_action('plugins_loaded', [$this, 'load_text_domain']);

        if ($this->is_updateable()) {

            $updater = PucFactory::buildUpdateChecker(
                $this->api_server.'/api/auto-update/version.json',
                $this->env->getBaseFile(),
                $this->env->getSlug()
            );

            $updater->addQueryArgFilter([$this, 'update_license']);
            $updater->addHttpRequestArgFilter([$this, 'update_license']);

        }

        $this->renderer = new TwigRenderer($this->env->getSlug(), false, __DIR__.'/../../templates');
    }

    public function is_valid(): bool
    {

        if ($this->valid) {
            return true;
        }

        $this->load_key();

        if (!$this->key) {
            return false;
        }

        return $this->validate_key();
    }

    private function load_key(): void
    {

        if ($this->key) {
            return;
        }

        $key = get_field($this->license_key_name, 'options');

        if (!$key) {

            $this->clear();

            return;
        }

        $this->key = $key;
    }

    public function clear(): void
    {

        delete_transient($this->key(Constants::TRIAL_TIMEOUT_TRANSIENT));
        delete_transient($this->key(Constants::VERIFICATION_TIMEOUT_TRANSIENT));
        delete_transient($this->key(Constants::VERIFICATION_TRANSIENT));
        delete_transient($this->key(Constants::LAST_RESPONSE_TRANSIENT));
        delete_transient($this->key(Constants::UPDATE_CHECK_TRANSIENT));


        $this->key = null;
    }

    private function key($key): string
    {

        return $this->namespace.$key;
    }

    private function validate_key(): bool
    {

        $key_verification = get_transient($this->key(Constants::VERIFICATION_TRANSIENT));

        if ($this->key === $key_verification) {

            $this->valid = true;

            return true;
        }

        $valid_key = $this->verify_key();

        if (!$valid_key) {
            return false;
        }

        set_transient($this->key(Constants::VERIFICATION_TRANSIENT), $this->key, 12 * HOUR_IN_SECONDS);

        return true;
    }

    private function verify_key(string $action = 'validate'): bool
    {

        $verification_timeout = $this->key(Constants::VERIFICATION_TIMEOUT_TRANSIENT);

        try {

            if (get_transient($verification_timeout)) {
                return false;
            }

            set_transient($verification_timeout, true, 5 * MINUTE_IN_SECONDS);

            delete_transient($this->key(Constants::LAST_RESPONSE_TRANSIENT));

            if (!$this->additional_data) {
                $this->additional_data = [];
            }

            $response = wp_remote_get(
                $this->api_server.'/api/'.$action,
                [
                    'body' => array_merge(
                        $this->additional_data,
                        [
                            'auth' => $this->auth_key,
                            'key' => $this->key,
                            'domain' => get_site_url(),
                        ]
                    ),
                ]
            );

            if ($response instanceof \WP_Error) {

                Helper::log($response->get_error_message(), 'error', $this->env->getSlug());

                return false;
            }

            $body = wp_remote_retrieve_body($response);
            $response = json_decode($body, true);

            if ($response === null) {

                Helper::log(sprintf('Could not parse Server Response: %s', $body), 'error', $this->env->getSlug());

                return false;
            }

            Helper::log(sprintf('Got Server Response: %s', $body), 'info', $this->env->getSlug());

            // We have to activate the license here
            if ($action === 'validate' && array_key_exists('ErrorCode', $response) && $response['ErrorCode'] === 201) {

                delete_transient($verification_timeout);

                return $this->verify_key('activate');
            }

            // Try to activate with the given url, if the current domain is not allowed
            if ($action === 'validate' && array_key_exists('ErrorCode', $response) && $response['ErrorCode'] === 701) {

                delete_transient($verification_timeout);

                return $this->verify_key('activate');
            }

            if ($response['Success'] === false) {

                Helper::log(
                    sprintf('Could not validate license: %s', json_encode($response)),
                    'error',
                    $this->env->getSlug()
                );

                return false;
            }

            set_transient($this->key(Constants::LAST_RESPONSE_TRANSIENT), $response, 24 * HOUR_IN_SECONDS);

            $this->valid = true;

            return true;

        } catch (\Exception $e) {
            Helper::log($e, 'error', $this->env->getSlug());
        }

        return false;

    }

    private function is_expired(): bool
    {

        $response = $this->get_last_response();

        if (!$response) {
            return false;
        }

        if (!is_array($response)) {
            return false;
        }

        if (!array_key_exists('LicenseData', $response)) {
            return false;
        }

        if ($response['LicenseData']['type'] !== 'paddle') {
            return false;
        }

        if (strlen($response['LicenseData']['start_date']) === 0) {
            return false;
        }

        $end = new \DateTime($response['LicenseData']['start_date']);
        $end->add(new \DateInterval('P1Y'));

        $now = new \DateTime();

        return $now > $end;
    }

    public function get_last_response(): mixed
    {

        $response = get_transient($this->key(Constants::LAST_RESPONSE_TRANSIENT));

        return $response;
    }

    public function is_trial(): bool
    {

        return $this->get_trial_period() > 0;
    }

    public function get_trial_period(): bool|int
    {

        $response = $this->get_last_response();

        if (!$response) {
            return false;
        }

        if (!is_array($response)) {
            return false;
        }

        if (!array_key_exists('LicenseData', $response)) {
            return false;
        }

        if ($response['LicenseData']['type'] !== 'trial') {
            return false;
        }

        $end = new \DateTime($response['LicenseData']['end_date']);
        $now = new \DateTime();

        $diff = date_diff($end, $now);

        $days = intval($diff->format('%a'));

        // Handle the case that we do not have a full day left
        if ($days === 0 && intval($diff->format('%s')) > 0) {
            return 1;
        }

        return $days;
    }

    private function is_updateable(): bool
    {

        $last_check = get_transient($this->key(Constants::UPDATE_CHECK_TRANSIENT));

        if ($last_check === 'available') {
            return true;
        }

        if ($last_check === 'error') {
            return false;
        }

        if ($last_check === 'not_available') {
            return false;
        }

        $response = wp_remote_get(
            $this->api_server.'/api/auto-update/version.json',
            ['body' => $this->update_license([])]
        );

        if ($response instanceof \WP_Error) {

            Helper::log($response->get_error_message(), 'error', $this->env->getSlug());

            set_transient($this->key(Constants::UPDATE_CHECK_TRANSIENT), 'error', 12 * HOUR_IN_SECONDS);

            return false;
        }

        $response_code = wp_remote_retrieve_response_code($response);

        if ($response_code !== 200) {

            set_transient($this->key(Constants::UPDATE_CHECK_TRANSIENT), 'not_available', 12 * HOUR_IN_SECONDS);

            return false;
        }

        $content_type = wp_remote_retrieve_header($response, 'content-type');

        if (strpos($content_type, 'application/json') === false) {

            set_transient($this->key(Constants::UPDATE_CHECK_TRANSIENT), 'error', 12 * HOUR_IN_SECONDS);

            return false;
        }

        $data = json_decode(wp_remote_retrieve_body($response), true);

        if (!$data) {

            set_transient($this->key(Constants::UPDATE_CHECK_TRANSIENT), 'error', 12 * HOUR_IN_SECONDS);

            return false;
        }

        if (!array_key_exists('download_url', $data)) {

            set_transient($this->key(Constants::UPDATE_CHECK_TRANSIENT), 'not_available', 12 * HOUR_IN_SECONDS);

            return false;
        }

        if (!strlen($data['download_url'])) {

            set_transient($this->key(Constants::UPDATE_CHECK_TRANSIENT), 'not_available', 12 * HOUR_IN_SECONDS);

            return false;
        }

        set_transient($this->key(Constants::UPDATE_CHECK_TRANSIENT), 'available', 12 * HOUR_IN_SECONDS);

        return true;
    }

    public function update_license(array $args): array
    {

        $args['key'] = get_field($this->license_key_name, 'options');
        $args['domain'] = get_site_url();

        return $args;
    }

    public function load_text_domain(): void
    {

        load_plugin_textdomain('codevision-wp-license-manager', false, __DIR__.'/languages/');
    }

    public function deactivate(): void
    {

        $this->clear();

        $this->compatibility_check?->reset();

        $this->load_key();

        if (!$this->key) {
            return;
        }

        if (!$this->additional_data) {
            $this->additional_data = [];
        }

        wp_remote_get(
            $this->api_server.'/api/'.'deactivate',
            [
                'blocking' => false,
                'body' => array_merge(
                    $this->additional_data,
                    [
                        'auth' => $this->auth_key,
                        'key' => $this->key,
                        'domain' => get_site_url(),
                    ]
                ),
            ]
        );
    }

    public function render_license_info(array $field): array
    {

        $response = $this->get_last_response();

        if (!$response) {
            return $field;
        }

        if (array_key_exists('DigistoreData', $response)) {

            $response = $response['DigistoreData'];

        } else {

            $day_format = 'd.m.Y';

            if ($response['LicenseData']['type'] === 'trial') {
                $day_format = 'd.m.Y H:i';
            }

            $response['product_name'] = $response['LicenseData']['description'];

            $response['last_payment_at_msg'] = date($day_format, strtotime($response['LicenseData']['start_date']));

            if ($response['LicenseData']['end_date']) {
                $response['next_payment_at_msg'] = date($day_format, strtotime($response['LicenseData']['end_date']));
                $response['paid_until_msg'] = date($day_format, strtotime($response['LicenseData']['end_date']));
            }
        }

        $days_left = $this->get_trial_period();

        if ($days_left > 0) {
            $response['trial'] = sprintf($this->get_translated_text('trial'), $days_left);
        }

        if ($days_left === 1) {
            $response['trial'] = sprintf($this->get_translated_text('trial_one_day_left'));
        }

        $field['message'] = $this->renderer->view('license_info', $response);

        return $field;
    }

    private function get_translated_text($key): string
    {

        $msg = $this->messages[$key];

        if (!is_array($msg)) {
            return __($msg, $this->env->getSlug());
        }

        $text = array_shift($msg);

        return vsprintf(__($text, $this->env->getSlug()), $msg);
    }

    public function update_license_data(): void
    {

        $screen = get_current_screen();

        if (!str_contains($screen->id, $this->license_view_slug)) {
            return;
        }

        $this->activate();
    }

    public function activate(): void
    {

        $this->clear();

        // Check current key
        $this->load_key();

        // The current key is valid
        if ($this->key && $this->validate_key()) {
            return;
        }

        // clear transient, because otherwise we will not revalidate a possible new trial key
        $this->clear();

        // Check if trial license can be aquired
        $response = $this->ask_trial_key();

        if (!$response) {
            return;
        }

        // Update the Value for the license Key and verify it
        update_field($this->license_key_name, $response['license-key'], 'option');

        $this->key = $response['license-key'];

        $this->validate_key();
    }

    private function ask_trial_key(): bool|array
    {

        // Skip, if we already have a key (this could be an expired one, we don't care)
        if ($this->key) {
            return false;
        }

        if (!$this->trial_allowed) {
            return false;
        }

        $trial_timeout = $this->key(Constants::TRIAL_TIMEOUT_TRANSIENT);

        if (get_transient($trial_timeout)) {
            return false;
        }

        set_transient($trial_timeout, true, 5 * MINUTE_IN_SECONDS);

        $response = wp_remote_get(
            $this->api_server.'/api/'.'generate',
            [
                'body' => [
                    'auth' => $this->auth_key,
                    'type' => 'trial',
                    'domain' => get_site_url(),
                ],
            ]
        );

        if ($response instanceof \WP_Error) {

            Helper::log($response->get_error_message(), 'error', $this->env->getSlug());

            return false;
        }

        $body = wp_remote_retrieve_body($response);
        $response = json_decode($body, true);

        if ($response === null) {

            Helper::log(sprintf('Could not parse Server Response: %s', $body), 'error', $this->env->getSlug());

            return false;
        }

        Helper::log(sprintf('Got Server Response: %s', $body), 'info', $this->env->getSlug());

        if ($response['Success'] === false) {

            Helper::log(
                sprintf('Could not activate trial license: %s', json_encode($response)),
                'error',
                $this->env->getSlug()
            );

            return false;
        }

        return $response['LicenseData'];
    }

    public function missing_or_wrong_api_key(): void
    {

        $message = [
            'headline' => $this->get_translated_text('headline'),
            'message' => $this->get_translated_text('missing_license_admin_notice'),
        ];

        echo $this->renderer->view('missing_license_admin_notice', $message);
    }

    public function plugin_expired(): void
    {

        $hide = $this->key(Constants::HIDE_RENEWAL_TRANSIENT);

        if (get_transient($hide)) {
            return;
        }

        $expire_date = $this->get_renewal_date();

        $message = [
            'headline' => $this->get_translated_text('headline'),
            'message' => sprintf($this->get_translated_text('license_renewal'), $expire_date),
        ];

        set_transient($hide, true, 6 * HOUR_IN_SECONDS);

        echo $this->renderer->view('trial_period_admin_notice', $message);
    }

    public function get_renewal_date(): bool|string
    {

        $response = $this->get_last_response();

        if (!$response) {
            return false;
        }

        if (!is_array($response)) {
            return false;
        }

        if (!array_key_exists('LicenseData', $response)) {
            return false;
        }

        if ($response['LicenseData']['type'] !== 'paddle') {
            return false;
        }

        $end = new \DateTime($response['LicenseData']['start_date']);
        $end->add(new \DateInterval('P1Y'));

        return \IntlDateFormatter::formatObject($end);
    }

    public function trial_period_activated(): void
    {

        $hide_trial = $this->key(Constants::MESSAGE_TRIAL_TIMEOUT_TRANSIENT);

        if (get_transient($hide_trial)) {
            return;
        }

        $days_left = $this->get_trial_period();

        $message = [
            'headline' => $this->get_translated_text('headline'),
            'message' => sprintf($this->get_translated_text('trial_period_admin_notice'), $days_left),
        ];

        // Show nag screen only every 2 hours
        set_transient($hide_trial, true, 2 * HOUR_IN_SECONDS);

        echo $this->renderer->view('trial_period_admin_notice', $message);
    }

    public function setNamespace(string $namespace): self
    {

        $this->namespace = $namespace;

        return $this;
    }

    public function setApiServer(string $api_server): self
    {

        $this->api_server = $api_server;

        return $this;
    }

    public function setAuthKey(string $auth_key): self
    {

        $this->auth_key = $auth_key;

        return $this;
    }

    public function setLicenseInfoField(string $license_info_field): self
    {

        $this->license_info_field = $license_info_field;

        return $this;
    }

    public function setLicenseKeyName(string $license_key_name): self
    {

        $this->license_key_name = $license_key_name;

        return $this;
    }

    public function setLicenseViewSlug(string $license_view_slug): self
    {

        $this->license_view_slug = $license_view_slug;

        return $this;
    }

    public function setMessages(array $messages): self
    {

        $this->messages = $messages;

        return $this;
    }

    public function setAdditionalData(array $additional_data): self
    {

        $this->additional_data = $additional_data;

        return $this;
    }

    public function isTrialAllowed(): bool
    {

        return $this->trial_allowed;
    }

    public function setTrialAllowed(bool $trial_allowed): self
    {

        $this->trial_allowed = $trial_allowed;

        return $this;
    }

    public function getCompatibilityCheck(): ?CompatibilityCheck
    {

        return $this->compatibility_check;
    }

}
