<?php

/*
 * This file is part of PHP-FFmpeg.
 *
 * (c) Alchemy <info@alchemy.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * Modified by sebastian rassmann on 06-February-2024 using Strauss.
 * @see https://github.com/BrianHenryIE/strauss
 */

namespace Photobooth\FFMpeg\Driver;

use Photobooth\Alchemy\BinaryDriver\AbstractBinary;
use Photobooth\Alchemy\BinaryDriver\Configuration;
use Photobooth\Alchemy\BinaryDriver\ConfigurationInterface;
use Photobooth\Alchemy\BinaryDriver\Exception\ExecutableNotFoundException as BinaryDriverExecutableNotFound;
use Photobooth\FFMpeg\Exception\ExecutableNotFoundException;
use Photobooth\FFMpeg\Exception\RuntimeException;
use Photobooth\Psr\Log\LoggerInterface;

class FFMpegDriver extends AbstractBinary
{
    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'ffmpeg';
    }

    /**
     * Creates an FFMpegDriver.
     *
     * @param LoggerInterface     $logger
     * @param array|Configuration $configuration
     *
     * @return FFMpegDriver
     */
    public static function create(LoggerInterface $logger = null, $configuration = [])
    {
        if (!$configuration instanceof ConfigurationInterface) {
            $configuration = new Configuration($configuration);
        }

        $binaries = $configuration->get('ffmpeg.binaries', ['avconv', 'ffmpeg']);

        if (!$configuration->has('timeout')) {
            $configuration->set('timeout', 300);
        }

        try {
            return static::load($binaries, $logger, $configuration);
        } catch (BinaryDriverExecutableNotFound $e) {
            throw new ExecutableNotFoundException('Unable to load FFMpeg', $e->getCode(), $e);
        }
    }

    /**
     * Get ffmpeg version.
     *
     * @return string
     *
     * @throws RuntimeException
     */
    public function getVersion()
    {
        preg_match('#version\s(\S+)#', $this->command('-version'), $version);
        if (!isset($version[1])) {
            throw new RuntimeException('Cannot to parse the ffmpeg version!');
        }

        return $version[1];
    }
}
